/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty and the software's author, the holder of the
 * economic rights, and the successive licensors  have only  limited
 * liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading, using, modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean that it is complicated to manipulate, and that also
 * therefore means that it is reserved for developers and experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and, more generally,to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
import java.awt.Color;
import java.awt.Dimension;
import main.airfoil.ConstantAirfoil;
import main.atmosphere.Atmosphere;
import main.atmosphere.DensityAtmosphere;
import main.inducedvelocity.ConstantInducedVelocity;
import main.inducedvelocity.ShaydakovInducedVelocity;
import main.rotor.GlauertRotor;
import main.utils.PolynomialFitting;
import org.apache.commons.math3.analysis.MultivariateFunction;
import org.apache.commons.math3.optim.InitialGuess;
import org.apache.commons.math3.optim.MaxEval;
import org.apache.commons.math3.optim.PointValuePair;
import org.apache.commons.math3.optim.nonlinear.scalar.GoalType;
import org.apache.commons.math3.optim.nonlinear.scalar.ObjectiveFunction;
import org.apache.commons.math3.optim.nonlinear.scalar.noderiv.NelderMeadSimplex;
import org.apache.commons.math3.optim.nonlinear.scalar.noderiv.SimplexOptimizer;
import org.apache.commons.math3.util.FastMath;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartFrame;
import org.jfree.chart.ChartTheme;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.StandardChartTheme;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * Compute helicopter performance chart
 * 
 * @author J. Fourcade
 */
public class HeliPerfos {

    final private SimplexOptimizer optimizer;
    final private ZeroFunction f;
    private double velocity;

    /**
     * Constructor : define helicopter characteristics
     * 
     * @param rotorSpeed : helicopter rotor speed (rad/s)
     * @param rotor : helicopter rotor
     * @param bodyDragCoef : body drag coefficient
     * @param weight : helicopter weight (N)
     * @param profilPower : polynomial fitting of profil power
     * @param atmosphere : atmosphere to compute density
     */
    public HeliPerfos(double rotorSpeed, GlauertRotor rotor, double bodyDragCoef,
            double weight, PolynomialFitting profilPower, Atmosphere atmosphere) {

        // Optimizer
        optimizer = new SimplexOptimizer(1e-8, 1e-10);

        // Objective function
        f = new ZeroFunction(rotorSpeed, rotor, bodyDragCoef, weight,
                profilPower, atmosphere);
    }

    /**
     * Compute blade pitch, rotor incidence and blade profil drag coefficient so that
     * lift equation, drag equation are satisfied and that computed profile power
     * equal given profil power
     * 
     * @param velocity : helicopter air speed (m/s)
     * @param startPoint : start point (blade pitch, rotor incidence and profil drag)
     * @return blade pitch (rad), rotor incidence (rad) and balde profil drag 
     * coefficient
     */
    public PointValuePair compute(double velocity, double[] startPoint) {

        this.velocity = velocity;

        // Optimize
        return optimizer.optimize(new ObjectiveFunction(f),
                new NelderMeadSimplex(new double[]{FastMath.toRadians(5),
                    FastMath.toRadians(5), 0.1}),
                GoalType.MINIMIZE,
                new InitialGuess(startPoint),
                new MaxEval(500));
    }

    private class ZeroFunction implements MultivariateFunction {

        final private double rotorSpeed;
        final private GlauertRotor rotor;
        final private double bodyDragCoef;
        final private double weight;
        final private PolynomialFitting profilPower;
        final private Atmosphere atmosphere;

        public ZeroFunction(double rotorSpeed, GlauertRotor rotor,
                double bodyDragCoef, double weight, PolynomialFitting profilPower,
                Atmosphere atmosphere) {
            this.rotorSpeed = rotorSpeed;
            this.rotor = rotor;
            this.bodyDragCoef = bodyDragCoef;
            this.weight = weight;
            this.profilPower = profilPower;
            this.atmosphere = atmosphere;
        }

        @Override
        public double value(double[] doubles) {
            final double bladePitch = doubles[0];
            final double incidence = doubles[1];
            final double delta = doubles[2];

            // Set delta to blade airfoil
            rotor.getBladeAirfoil().setProfileDrag(delta);

            // Compute thrust state
            rotor.computeState(velocity, incidence, bladePitch, 0, 0, rotorSpeed, atmosphere);

            // Compute drag
            final double bodyDrag = 0.5 * atmosphere.getDensity() * bodyDragCoef
                    * FastMath.pow(velocity, 2);

            // Lift equation
            final double eq1 = rotor.getLift() - weight;

            // Drag equation (rotor drag is negative in helicopter mode)
            final double eq2 = bodyDrag + rotor.getDrag();

            // Profile power
            final double velocityKT = velocity / 1.852 * 3.6;
            final double eq3 = profilPower.value(velocityKT)
                    + rotor.getProfileDragTorque() * rotorSpeed / 735.5;

            // Return zero function
            return FastMath.pow(eq1, 2) + FastMath.pow(eq2, 2) + FastMath.pow(eq3, 2);
        }
    }

    /**
     *
     * @param args
     */
    public static void main(String[] args) {

        // Profil power tabulated values (hp and kt)
        final double[] profilPowerTab = {28.5588, 28.2135, 28.4248, 28.9033,
            29.1322, 29.9001, 30.3992, 31.1714, 31.9525, 33.2724, 35.1416, 36.4733};
        final double[] velocityTab = {0, 12.7896, 24.3192, 37.191,
            45.5036, 55.6964, 64.815, 74.2036, 81.9836, 91.6439, 101.308, 108.824};

        // Least square interpolation of rotor speed
        final PolynomialFitting profilPower = new PolynomialFitting(4, profilPowerTab,
                velocityTab);

        // Define R22 rotor 
        ConstantAirfoil bladeProfile = new ConstantAirfoil(5.7, 0.0115, 0, 0);
        ConstantInducedVelocity inducedVelocity = new ShaydakovInducedVelocity();
        Atmosphere atmosphere = new DensityAtmosphere(1.225);

        final int b = 2; // number of blade
        final double R = 3.835; // rotor radius
        final double c = 0.183; // rotor chord
        final double B = 0.97; // Tip loss factor
        final double ml = 3; // blade mass per unit length
        final double thetaTW = FastMath.toRadians(-7);
        final double Ib = ml * FastMath.pow(R, 3) / 3; // blade moment of inertia
        GlauertRotor rotor = new GlauertRotor(b, R, c, 0, thetaTW, 0, B, Ib, Ib,
                bladeProfile, inducedVelocity);

        System.out.println("Ib : " + Ib + " kg.m2");

        // Rotor speed
        final double rotorSpeed = 520 * FastMath.PI / 30;

        // Body drag coeffcient Scx
        final double Scx = 0.49;

        // Weight
        final double weight = 6080;

        // Helicopter performance class computation
        HeliPerfos heliPerfos = new HeliPerfos(rotorSpeed, rotor, Scx, weight,
                profilPower, atmosphere);

        // XY Series
        final XYSeries profileDrag = new XYSeries("Blade profile drag");
        final XYSeries pitch = new XYSeries("Blade pitch");
        final XYSeries incidence = new XYSeries("Rotor incidence");

        // XY Series (fuselage power, profile drag power
        // induced power, total power)
        final XYSeries F = new XYSeries("Fuselage");
        final XYSeries FMes = new XYSeries("FuselageMes");
        FMes.add(0, 0);
        FMes.add(11.2596, 0.001);
        FMes.add(24.1333, 0.691259);
        FMes.add(33.7936, 2.0112);
        FMes.add(45.0735, 4.96985);
        FMes.add(55.0222, 9.3088);
        FMes.add(64.442, 14.7491);
        FMes.add(74.6845, 22.9308);
        FMes.add(83.0467, 30.5737);
        FMes.add(88.7244, 37.6821);
        FMes.add(94.4094, 45.8888);
        FMes.add(101.461, 57.9325);
        FMes.add(107.452, 71.9042);
        final XYSeries P = new XYSeries("Profil");
        final XYSeries PMes = new XYSeries("ProfilMes");
        for (int i = 0; i < profilPowerTab.length; i++) {
            PMes.add(velocityTab[i], profilPowerTab[i]);
        }
        final XYSeries I = new XYSeries("Induced");
        final XYSeries IMes = new XYSeries("InducedMes");
        IMes.add(0, 65.6298);
        IMes.add(6.85737, 63.1202);
        IMes.add(16.4441, 53.4566);
        IMes.add(24.4167, 42.978);
        IMes.add(34.8022, 32.4862);
        IMes.add(44.411, 26.1177);
        IMes.add(55.1013, 21.1161);
        IMes.add(64.196, 18.0456);
        IMes.add(73.8306, 15.5213);
        IMes.add(84.0068, 13.8178);
        IMes.add(94.1886, 12.9381);
        IMes.add(103.298, 12.0643);
        final XYSeries T = new XYSeries("Total");

        // Start point (blade pitch at root, rotor incidence and blade profil drag)
        final double[] startPoint = {FastMath.toRadians(8), FastMath.toRadians(0.0),
            0.011};

        // Loop : speed in knt
        for (double U = 0; U < 110; U = U + 1) {
            final double velocity = U * 1.852 / 3.6;
            // Compute value : blade pitch and rotor incidence
            PointValuePair result = heliPerfos.compute(velocity, startPoint);

            // Outputs
            System.out.println("Velocity " + velocity + " m/s");
            System.out.println("Blade pitch " + FastMath.toDegrees(result.getPoint()[0]));
            System.out.println("Rotor incidence " + FastMath.toDegrees(result.getPoint()[1]));
            System.out.println("Blade profil drag " + result.getPoint()[2]);
            System.out.println();

            // Rotor characteristics
            profileDrag.add(U, result.getPoint()[2]);
            pitch.add(U, FastMath.toDegrees(result.getPoint()[0]));
            incidence.add(U, FastMath.toDegrees(result.getPoint()[1]));

            // Compute profil power
            final double PPower = -rotor.getProfileDragTorque() * rotorSpeed / 735.5;

            // Compute induced power adding 8% hp for tail rotor
            final double IPower = (rotor.getThrust()
                    * rotor.getInducedVelocity().getVelocity()
                    - 0.08 * rotor.getTotalTorque() * rotorSpeed) / 735.5;

            // Compute fuselage power
            final double FPower = 0.5 * atmosphere.getDensity() * Scx
                    * FastMath.pow(velocity, 3) / 735.5;

            // Compute total power
            final double TPower = (PPower + IPower + FPower);

            // Add to series
            F.add(U, FPower);
            P.add(U, PPower);
            I.add(U, IPower);
            T.add(U, TPower);
        }

        // Performance chart ---------------------------------------------------
        // Serie collection
        final XYSeriesCollection data = new XYSeriesCollection();
        data.addSeries(FMes);
        data.addSeries(F);
        data.addSeries(PMes);
        data.addSeries(P);
        data.addSeries(IMes);
        data.addSeries(I);
        data.addSeries(T);

        // V Plot
        final NumberAxis xPerformanceAxis = new NumberAxis("Velocity KT");
        final NumberAxis yPerformanceAxis = new NumberAxis("Hp");

        final XYPlot performancePlot = new XYPlot(data, xPerformanceAxis, yPerformanceAxis, null);
        performancePlot.setOrientation(PlotOrientation.VERTICAL);

        // V Renderer
        final XYLineAndShapeRenderer performanceRenderer = new XYLineAndShapeRenderer(true, false);
        performanceRenderer.setSeriesLinesVisible(0, false);
        performanceRenderer.setSeriesVisibleInLegend(0, false);
        performanceRenderer.setSeriesShapesVisible(0, true);
        performanceRenderer.setSeriesPaint(0, Color.red);
        performanceRenderer.setSeriesPaint(1, Color.red);
        performanceRenderer.setSeriesLinesVisible(2, false);
        performanceRenderer.setSeriesVisibleInLegend(2, false);
        performanceRenderer.setSeriesShapesVisible(2, true);
        performanceRenderer.setSeriesPaint(2, Color.green);
        performanceRenderer.setSeriesPaint(3, Color.green);
        performanceRenderer.setSeriesLinesVisible(4, false);
        performanceRenderer.setSeriesVisibleInLegend(4, false);
        performanceRenderer.setSeriesShapesVisible(4, true);
        performanceRenderer.setSeriesPaint(4, Color.blue);
        performanceRenderer.setSeriesPaint(5, Color.blue);
        performanceRenderer.setSeriesPaint(6, Color.yellow);

        // Chart
        final JFreeChart performanceChart = new JFreeChart("",
                JFreeChart.DEFAULT_TITLE_FONT, performancePlot, true);
        ChartTheme deformationChartTheme = new StandardChartTheme("JFree");
        deformationChartTheme.apply(performanceChart);
        performancePlot.setRenderer(performanceRenderer);

        // blade profil drag chart ---------------------------------------------
        // Serie collection
        final XYSeriesCollection profileDragSeries = new XYSeriesCollection();
        profileDragSeries.addSeries(profileDrag);

        // Create Chart
        final JFreeChart profileDragChart = ChartFactory.createXYLineChart(
                "",
                "Speed kt", // x axis label
                "", // y axis label
                profileDragSeries// data
        );
        profileDragChart.removeLegend();

        // blade pitch chart ---------------------------------------------
        // Serie collection
        final XYSeriesCollection pitchSeries = new XYSeriesCollection();
        pitchSeries.addSeries(pitch);

        // Create Chart
        final JFreeChart pitchChart = ChartFactory.createXYLineChart(
                "",
                "Speed kt", // x axis label
                "deg", // y axis label
                pitchSeries// data
        );
        pitchChart.removeLegend();

        // rotor incidence chart -----------------------------------------------
        // Serie collection
        final XYSeriesCollection incidenceSeries = new XYSeriesCollection();
        incidenceSeries.addSeries(incidence);

        // Create Chart
        final JFreeChart incidenceChart = ChartFactory.createXYLineChart(
                "",
                "Speed kt", // x axis label
                "deg", // y axis label
                incidenceSeries// data
        );
        incidenceChart.removeLegend();

        // Show chart
        // performanceChart or profileDragChart or pitchChart or incidenceChart
        ChartFrame frame = new ChartFrame("R22", incidenceChart);
        //frame.setMinimumSize(new Dimension(400, 700));
        frame.setMinimumSize(new Dimension(400, 400));
        frame.pack();
        frame.setVisible(true);
    }
}
