/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */


import main.airfoil.ConstantAirfoil;
import main.atmosphere.Atmosphere;
import main.atmosphere.ConstantAtmosphere;
import main.inducedvelocity.ConstantInducedVelocity;
import main.inducedvelocity.MomentumInducedVelocity;
import main.inducedvelocity.ShaydakovInducedVelocity;
import main.rotor.GlauertRotor;
import org.apache.commons.math3.util.FastMath;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartFrame;
import org.jfree.chart.JFreeChart;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * Glauert rotor test. Display polar chart (CL versus DC), CLChart (CL versus 
 * rotor incidence), CDChart (CD versus incidence) and etat chart (rotor state
 * versus incidence)
 *
 * @author J. Fourcade
 */
public class PolarAndDragCurves {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

        // Define rotor
        ConstantAirfoil bladeProfile = new ConstantAirfoil(5.7, 0.011,0, 0);
        ConstantInducedVelocity inducedVelocity = new ShaydakovInducedVelocity();
        Atmosphere atmosphere = new ConstantAtmosphere(1.225, 0, 0);
        GlauertRotor rotor = new GlauertRotor(2, 4, 0.2, 0, Math.toRadians(0), 0, 
                0.97, 64, 64.04, bladeProfile, inducedVelocity);

        // Velocity
        double U = 100 / 3.6;

        // Curves data
        int n = 91;
        double step = 1.0;
        double[] incidence1 = new double[n];
        double[] CDS = new double[n];
        double[] CLS = new double[n];
        double[] eta = new double[n];

        // Simulate in autorotative mode : 80 km/h, 3 degres blade pitch, 0 pitch and rool rate
        // for given incidence
        for (int i = 0; i < n; i++) {
            incidence1[i] = i * step;
            rotor.computeState(U, FastMath.toRadians(incidence1[i]),
                    Math.toRadians(3), 0, 0, atmosphere);
            CDS[i] = rotor.getDragCoefficient();
            CLS[i] = rotor.getLiftCoefficient();
            eta[i] = -rotor.getVelocity() * FastMath.sin(rotor.getIncidence())
                    / rotor.getInducedVelocity().getVelocity();
        }

        // Same simulations but with momentum theory
        n = 81;
        double[] incidence2 = new double[n];
        double[] CDM = new double[n];
        double[] CLM = new double[n];
        inducedVelocity = new MomentumInducedVelocity();
        rotor = new GlauertRotor(2, 4, 0.2, 0, Math.toRadians(0), 0, 
                0.97, 64, 64.04, bladeProfile, inducedVelocity);
        for (int i = 0; i < n; i++) {
            incidence2[i] = i * step;
            rotor.computeState(U, FastMath.toRadians(incidence2[i]),
                    Math.toRadians(3), 0, 0, atmosphere);

            CDM[i] = rotor.getDragCoefficient();
            CLM[i] = rotor.getLiftCoefficient();
        }
        
        // Polar curve ---------------------------------------------------------
        final XYSeries polar = new XYSeries("Polar");
        for(int i=0;i<CDS.length;i++) {
           polar.add(CDS[i],CLS[i]);  
        }
        // Serie collection
        final XYSeriesCollection polarData = new XYSeriesCollection();
        polarData.addSeries(polar);
        
         // Create Chart
        final JFreeChart polarChart = ChartFactory.createXYLineChart(
                "",
                "CD", // x axis label
                "CL", // y axis label
                polarData        );
        polarChart.removeLegend();
        
        // CLchart ------------------------------------------------------------
        final XYSeries CLSG = new XYSeries("CL Shaydakov");
        final XYSeries CLMG = new XYSeries("CL Froude");
        for(int i=0;i<CLM.length;i++) {
           CLSG.add(incidence2[i],CLS[i]);  
           CLMG.add(incidence2[i],CLM[i]);  
        }
        // Serie collection
        final XYSeriesCollection CLdata = new XYSeriesCollection();
        CLdata.addSeries(CLSG);
        CLdata.addSeries(CLMG);

        // Create Chart
        final JFreeChart CLChart = ChartFactory.createXYLineChart(
                "",
                "Incidence (deg)", // x axis label
                "CL", // y axis label
                CLdata// data
        );
        
        // CD chart ------------------------------------------------------------
        final XYSeries CDSG = new XYSeries("CD Shaydakov");
        final XYSeries CDMG = new XYSeries("CD Froude");
        for(int i=0;i<CDM.length;i++) {
           CDSG.add(incidence2[i],CDS[i]);  
           CDMG.add(incidence2[i],CDM[i]);  
        }
        // Serie collection
        final XYSeriesCollection CDdata = new XYSeriesCollection();
        CDdata.addSeries(CDSG);
        CDdata.addSeries(CDMG);

        // Create Chart
        final JFreeChart CDChart = ChartFactory.createXYLineChart(
                "",
                "Incidence (deg)", // x axis label
                "CD", // y axis label
                CDdata// data
        );
        
        // Etat chart ------------------------------------------------------------
        final XYSeries etaG = new XYSeries("Etat");
        for(int i=0;i<eta.length;i++) {
           etaG.add(incidence1[i],eta[i]);  
        }
        // Serie collection
        final XYSeriesCollection etaData = new XYSeriesCollection();
        etaData.addSeries(etaG);

        // Create Chart
        final JFreeChart etatChart = ChartFactory.createXYLineChart(
                "",
                "Incidence (deg)", // x axis label
                "", // y axis label
                etaData// data
        );
        etatChart.removeLegend();
        
        // Show chart
        // polarChart or CLChart or CDChart or etatChart
        ChartFrame frame = new ChartFrame("", etatChart);
        frame.pack();
        frame.setVisible(true);
    }
}
