/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty and the software's author, the holder of the
 * economic rights, and the successive licensors  have only  limited
 * liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading, using, modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean that it is complicated to manipulate, and that also
 * therefore means that it is reserved for developers and experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and, more generally,to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
package main.airfoil;

import main.atmosphere.Atmosphere;
import main.utils.Torsor;
import org.apache.commons.math3.geometry.euclidean.threed.Vector3D;
import org.apache.commons.math3.util.FastMath;

/**
 * The constant slope airfoil profile is an airfoil defined only by its lift
 * curve slope, profile drag coefficient, section moment coefficient and
 * chordwise location of aerodynamic center.
 *
 * @author J. Fourcade
 */
public class ConstantAirfoil implements Airfoil {

    private double liftSlope;
    private double profileDrag;
    private double momentCoefficient;
    private double aerodynamicCenterLocation;

    /**
     * Constructor defining lift curve slope, profile drag coefficient, section 
     * moment coefficient and chordwise location of aerodynamic center.
     *
     * @param liftCurveSlope : lift slope (1/rad)
     * @param profileDragCoeff : profile drag 
     * @param momentCoefficient : section moment coefficient
     * @param aerodynamicCenterLocation : chordwise location of aerodynamic center 
     * from leading edge (ratio versus blade chord)
     */
    public ConstantAirfoil(double liftCurveSlope, double profileDragCoeff, double momentCoefficient,
            double aerodynamicCenterLocation) {
        this.liftSlope = liftCurveSlope;
        this.profileDrag = profileDragCoeff;
        this.momentCoefficient = momentCoefficient;
        this.aerodynamicCenterLocation = aerodynamicCenterLocation;
    }

    /**
     * @return lift curve slope
     */
    public double getLiftSlope() {
        return liftSlope;
    }

    /**
     * @return profile drag coefficient
     */
    public double getProfileDrag() {
        return profileDrag;
    }

    /**
     * @return moment coefficient
     */
    public double getMomentCoefficient() {
        return momentCoefficient;
    }

    /**
     * @return aerodynamic center location
     */
    public double getAerodynamicCenterLocation() {
        return aerodynamicCenterLocation;
    }
    
    /**
     *
     * @param liftSlope : lift curve slope
     */
    public void setLiftSlope(double liftSlope) {
        this.liftSlope = liftSlope;
    }
    
    /**
     *
     * @param profileDrag : profile drag coefficient
     */
    public void setProfileDrag(double profileDrag) {
        this.profileDrag = profileDrag;
    }
    
    /**
     *
     * @param momentCoefficient : moment coefficient
     */
    public void setMomentCoefficient(double momentCoefficient) {
        this.momentCoefficient = momentCoefficient;
    }
    
    /**
     *
     * @param aerodynamicCenterLocation : aerodynamic center location
     */
    public void setAerodynamicCenterLocation(double aerodynamicCenterLocation) {
        this.aerodynamicCenterLocation = aerodynamicCenterLocation;
    } 
    
    @Override
    public Torsor computeAerodynamicTorsor(double UT, double UR, double UP,
            Atmosphere atmosphere, double chord, double length, double chordLocation) {

        // Compute velocity norms
        final double U2 = FastMath.pow(UT, 2) + FastMath.pow(UP, 2) + FastMath.pow(UR, 2);

        // Compute angle of attack;
        final double alpha = FastMath.atan2(UP, UT);
        final double cosAlpha = FastMath.cos(alpha);
        final double sinAlpha = FastMath.sin(alpha);

        // Compute lift and drag
        final double factor = 0.5 * U2 * atmosphere.getDensity() * chord * length;
        final double L = factor * liftSlope * alpha;
        final double D = factor * profileDrag;

        // Transformation to reference frame
        Vector3D force = new Vector3D(D * cosAlpha - L * sinAlpha,
                0, D * sinAlpha + L * cosAlpha);
        // Return torsor
        return new Torsor(force, new Vector3D(0, 0, 0));
    }
}
