/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package inducedvelocity;

import java.util.ArrayList;
import java.util.List;
import main.inducedvelocity.MomentumInducedVelocity;
import main.inducedvelocity.ShaydakovInducedVelocity;
import main.rotor.NormalizedRotor;
import org.jfree.chart.ChartFrame;
import org.jfree.chart.ChartTheme;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.StandardChartTheme;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/**
 * Compare Shaydakov and momentum induced velocity.
 *
 * @author J. Fourcade
 */
public class ConstantInducedVelocityTest {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

        // Momentum induced theory
        MomentumInducedVelocity momentumInducedVelocity = new MomentumInducedVelocity();
        ShaydakovInducedVelocity ShaydakovInducedVelocity = new ShaydakovInducedVelocity();

        // Compute tabs for a givent barMu
        final double barMu = 0;

        // Outputs
        List<Number> momentumBarLambdaTable = new ArrayList<>();
        List<Number> momemtumBarVTable = new ArrayList<>();
        List<Number> ShaydakovBarLambdaTable = new ArrayList<>();
        List<Number> ShaydakovumBarVTable = new ArrayList<>();

        // Loop on barLambda for momentum theorie
        for (double barLambda = -5; barLambda < 0.5; barLambda = barLambda + 0.1) {

            // Normaized rotor (vi0=1)        
            NormalizedRotor rotor = new NormalizedRotor(barMu, barLambda);

            // Calculate induced velocity
            try {
                momentumInducedVelocity.compute(rotor);

                // Check value
                final double barV = momentumInducedVelocity.getVelocity();
                final double f = barV * Math.sqrt(
                        Math.pow(barMu, 2) + Math.pow(barLambda + barV, 2)) - 1;
                if (Math.abs(f) > 1e-10) {
                    System.out.println("Momentum induced velocity error barLambda = " + barLambda + " " + f);
                }

                // Output
                momentumBarLambdaTable.add(barLambda);
                momemtumBarVTable.add(momentumInducedVelocity.getVelocity());
            } catch (Exception e) {
                System.out.println("Momentum failed to converge barLambda = " + barLambda);
            }
        }

        // Loop on barLambda for Shaydakov theorie
        for (double barLambda = -5; barLambda < 0.5; barLambda = barLambda + 0.05) {

            // Normaized rotor (vi0=1)        
            NormalizedRotor rotor = new NormalizedRotor(barMu, barLambda);

            // Calculate induced velocity
            try {

                ShaydakovInducedVelocity.compute(rotor);

                // Check value
                double f;
                final double barV = ShaydakovInducedVelocity.getVelocity();
                final double y1 = 2 * (Math.sqrt(
                        Math.pow(barMu, 4) + 1) - Math.pow(barMu, 2));
                if (barLambda > -Math.sqrt(y1)) {
                    final double y2 = barLambda * Math.sqrt(
                            Math.pow(barMu, 2) + Math.pow(barLambda, 2) / 4);
                    final double y3 = (barLambda + barV) * Math.sqrt(
                            Math.pow(barMu, 2) + Math.pow(barLambda + barV, 2));
                    f = y3 - y2 - 1;
                } else {
                    f = barV * Math.sqrt(Math.pow(barMu, 2) + Math.pow(barLambda + barV / 2, 2)) - 1;
                }
                if (Math.abs(f) > 1e-10) {
                    System.out.println("Shaydakov induced velocity error barLambda = " + barLambda + " " + f);
                }

                // Output
                ShaydakovBarLambdaTable.add(barLambda);
                ShaydakovumBarVTable.add(ShaydakovInducedVelocity.getVelocity());
            } catch (Exception e) {
                System.out.println("Shaydakov failed to converge barLambda = " + barLambda);
            }
        }

        final XYSeries Momentum = new XYSeries("Momentum");
        for (int i = 0; i < momentumBarLambdaTable.size(); i++) {
            Momentum.add(momentumBarLambdaTable.get(i), momemtumBarVTable.get(i));
        }
        final XYSeries Shaydakov = new XYSeries("Shaydakov");
        for (int i = 0; i < ShaydakovBarLambdaTable.size(); i++) {
            Shaydakov.add(ShaydakovBarLambdaTable.get(i), ShaydakovumBarVTable.get(i));
        }

        // Collection
        final XYSeriesCollection InducedVelocityData = new XYSeriesCollection();
        InducedVelocityData.addSeries(Momentum);
        InducedVelocityData.addSeries(Shaydakov);

        // Plot
        final NumberAxis xAxis = new NumberAxis("");
        xAxis.setAutoRangeIncludesZero(false);
        final NumberAxis yAxis = new NumberAxis("");
        yAxis.setAutoRangeIncludesZero(false);
        final XYItemRenderer renderer = new XYLineAndShapeRenderer(false, true);
        renderer.setBaseToolTipGenerator(new StandardXYToolTipGenerator());
        final XYPlot plot = new XYPlot(InducedVelocityData, xAxis,
                yAxis, renderer);

        // Chart
        final JFreeChart inducedVelocityChart = new JFreeChart("",
                JFreeChart.DEFAULT_TITLE_FONT, plot, true);
        ChartTheme torqueTheme = new StandardChartTheme("JFree");
        torqueTheme.apply(inducedVelocityChart);

        // Show chart
        ChartFrame frame = new ChartFrame("", inducedVelocityChart);
        frame.pack();
        frame.setVisible(true);
    }
}
