/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package rotor;

import static junit.framework.Assert.assertEquals;
import main.airfoil.ConstantAirfoil;
import main.atmosphere.Atmosphere;
import main.atmosphere.DensityAtmosphere;
import main.inducedvelocity.ConstantInducedVelocity;
import main.inducedvelocity.ShaydakovInducedVelocity;
import main.rotor.GlauertRotor;
import org.apache.commons.math3.util.FastMath;

/**
 * Glauert rotor test.
 *
 * @author J. Fourcade
 */
public class GlauertRotorTest {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

        // Define rotor
        final double a = 5.7;
        final double delta = 0.011;
        final double cm = 0.005;
        final double xac = 0.278;
        ConstantAirfoil bladeProfile = new ConstantAirfoil(a, delta, cm, xac);
        ConstantInducedVelocity inducedVelocity = new ShaydakovInducedVelocity();
        Atmosphere atmosphere = new DensityAtmosphere(1.225);
        final int b = 2;
        final double R = 4;
        final double c = 0.2;
        final double xcg = 0.3;
        final double thetaTW = FastMath.toRadians(2);
        final double GJ = 6350;
        final double B = 0.97;
        final double Ib = 64.00;
        final double Ic = 64.04;
        GlauertRotor rotor = new GlauertRotor(b, R, c, xcg, thetaTW, GJ, B, Ib, Ic,
                bladeProfile, inducedVelocity);

        // Simulate in autorotative mode : 100 km/h, 10 degres incidence, 
        // 2 degres blade pitch, 2 deg/s pitch rate, -3 deg/s roll rate
        final double U = 90 / 3.6;
        final double alphaS = Math.toRadians(7);
        final double theta0 = FastMath.toRadians(2);
        final double q = FastMath.toRadians(2);
        final double p = FastMath.toRadians(-3);
        rotor.computeState(U, alphaS, theta0, q, p, atmosphere);

        // Tol for test
        final double tola = 1e-9; // angles
        final double tolf = 1e-6; // forces

        // Print rotor
        System.out.println("Rotor speed (rad/s) : "+rotor.getRotorSpeed());
        System.out.println();
        rotor.print();

        // Pitch deformation coefficient asserts
        assertEquals("u0", 0.211135519025380, FastMath.toDegrees(rotor.computeMeanPitchDeformation(1)), tola);
        assertEquals("u1", 0.00972186712599437, FastMath.toDegrees(rotor.computePitchDeformation1C(1)), tola);
        assertEquals("v1", 0.0450938074819225, FastMath.toDegrees(rotor.computePitchDeformation1S(1)), tola);
        assertEquals("u2", 0.00896996365225126, FastMath.toDegrees(rotor.computePitchDeformation2C(1)), tola);
        assertEquals("v2", -0.682589578911081e-4, FastMath.toDegrees(rotor.computePitchDeformation2S(1)), tola);

        // Flapping coefficient asserts
        assertEquals("a0", 3.03409110919717, FastMath.toDegrees(rotor.getConingAngle()), tola);
        assertEquals("a1", 1.76342065631248, FastMath.toDegrees(rotor.getLongitudinalFlapping()), tola);
        assertEquals("b1", 0.894100707571944, FastMath.toDegrees(rotor.getLateralFlapping()), tola);
        assertEquals("a2", 0.0627955850265409, FastMath.toDegrees(rotor.get2ndLongitudinalFlapping()), tola);
        assertEquals("b2", -0.478424378401763e-3, FastMath.toDegrees(rotor.get2ndLateralFlapping()), tola);

        // Thurst assert
        assertEquals("T", 3232.77629410444, rotor.getThrust(), tolf);
        assertEquals("TP1S", 143.732273638064, rotor.getPeriodicThrust1S(), tolf);
        assertEquals("TP1C", 58.609531428018, rotor.getPeriodicThrust1C(), tolf);
        assertEquals("TP2S", -6.8148797423167, rotor.getPeriodicThrust2S(), tolf);
        assertEquals("TP2C", 105.851485980626, rotor.getPeriodicThrust2C(), tolf);

        // Rear force assert
        assertEquals("Hp", 19.7763495505108, rotor.getProfileDragRearForce(), tolf);
        assertEquals("Hi", 105.385938819551, rotor.getInducedRearForce(), tolf);
        assertEquals("HpP1S", 41.7716899076188, rotor.getProfileDragPeriodicRearForce1S(), tolf);
        assertEquals("HpP2C", -9.88817477525544, rotor.getProfileDragPeriodicRearForce2C(), tolf);
        assertEquals("HiP1S", -44.5459585312471, rotor.getInducedPeriodicRearForce1S(), tolf);
        assertEquals("HiP1C", -84.4368573604294, rotor.getInducedPeriodicRearForce1C(), tolf);
        assertEquals("HiP2S", 6.92035703966987, rotor.getInducedPeriodicRearForce2S(), tolf);
        assertEquals("HiP2C", -2.86034930028224, rotor.getInducedPeriodicRearForce2C(), tolf);

        // Lateral force assert
        assertEquals("Yi", 19.3624497231699, rotor.getLateralForce(), tolf);
        assertEquals("YpP1C", -40.1123314756457, rotor.getProfileDragPeriodicLateralForce1C(), tolf);
        assertEquals("YpP2S", -9.88817477525544, rotor.getProfileDragPeriodicLateralForce2S(), tolf);
        assertEquals("YiP1S", -82.0144521253675, rotor.getInducedPeriodicLateralForce1S(), tolf);
        assertEquals("YiP1C", 53.0443573290818, rotor.getInducedPeriodicLateralForce1C(), tolf);
        assertEquals("YiP2S", -6.35851561244483, rotor.getInducedPeriodicLateralForce2S(), tolf);
        assertEquals("YiP2C", -6.7115183003554, rotor.getInducedPeriodicLateralForce2C(), tolf);

        // Torque assert
        assertEquals("Qp", -242.309982685552, rotor.getProfileDragTorque(), tolf);
        assertEquals("Qi", 242.309982665036, rotor.getInducedTorque(), tolf);
        assertEquals("QpP1S", -52.7369321346955, rotor.getProfileDragPeriodicTorque1S(), tolf);
        assertEquals("QpP2C", 3.31871686394616, rotor.getProfileDragPeriodicTorque2C(), tolf);
        assertEquals("QiP1S", -166.773362271325, rotor.getInducedPeriodicTorque1S(), tolf);
        assertEquals("QiP1C", 25.3555997321762, rotor.getInducedPeriodicTorque1C(), tolf);
        assertEquals("QiP2S", -8.1460530084397, rotor.getInducedPeriodicTorque2S(), tolf);
        assertEquals("QiP2C", 8.09729727467305, rotor.getInducedPeriodicTorque2C(), tolf);
    }
}
