/*
 * GyroKit Copyright J. Fourcade
 * E-mail : autogire@volucres.fr
 * Web site : http://www.volucres.fr
 * This software is a flight dynamics library whose purpose is to simulate
 * the behavior of rotating-wing aircrafts.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer and aerodynamic knowledge.
 * Users are therefore encouraged to load and test the software's suitability
 * as regards their requirements in conditions enabling the security of their
 * systems and/or data to be ensured and,  more generally, to use and
 * operate it in the same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms
 */
package main.rotor;

import main.airfoil.ConstantAirfoil;
import main.atmosphere.Atmosphere;
import main.exception.RotorSpeedException;
import main.inducedvelocity.ConstantInducedVelocity;
import org.apache.commons.math3.analysis.UnivariateFunction;
import org.apache.commons.math3.analysis.solvers.BrentSolver;
import org.apache.commons.math3.exception.NoBracketingException;
import org.apache.commons.math3.exception.TooManyEvaluationsException;
import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.ArrayRealVector;
import org.apache.commons.math3.linear.DecompositionSolver;
import org.apache.commons.math3.linear.LUDecomposition;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.linear.RealVector;
import org.apache.commons.math3.util.FastMath;

/**
 * Implement analitycal rotor in tribute to Glauert work.
 *
 * @author J. Fourcade
 */
public class GlauertRotor extends Rotor implements TrustVelocitylRotor {

    // Input caracteristics
    final private int b;
    final private double R;
    final private double c;
    final private double xcg;
    final private double thetaTW;
    final private double B;
    final private double GJ;
    final private double Ib;
    final private double Ic;
    private final ConstantAirfoil bladeAirfoil;
    private final ConstantInducedVelocity inducedVelocity;
    // Computed geometric caracteristics
    private double Gamma;
    final private double A;
    final private double s;
    // Aerodynamic parameters
    private double U;
    private double alphaS;
    private double theta0;
    private double omega;
    private double mu;
    private double lambda;
    private double pitchRate, p;
    private double rollRate, q;
    private double rho;
    // Flapping coefficient ;
    private double a0, a1, b1, a2, b2;
    // Torsionnal pitch deformation coefficient
    private double u01, u02, u03, u04, u05;
    private double u11, u12, u13, u14, u15;
    private double v11, v12, v13, v14, v15;
    private double u21, u22, u23, u24, u25;
    private double v21, v22, v23, v24, v25;
    // Rotor torque
    private double cqi, Qi, cqp, Qp;
    private double qpP1S, qpP2C;
    private double qiP1S, qiP1C, qiP2S, qiP2C;
    // Forces
    private double ct, T;
    private double tp1S, tp1C, tp2S, tp2C;
    private double chp, Hp, chi, Hi;
    private double hpP1S, hpP2C;
    private double hiP1S, hiP1C, hiP2S, hiP2C;
    private double cyi, Yi;
    private double ypP1C, ypP2S;
    private double yiP1S, yiP1C, yiP2S, yiP2C;
    // Lift and drag
    private double L, D, CL, CD;
    // Force and moment scale factors
    private double forceScaleFactor, momentScaleFactor;
    // Derivative speed and angular velocity scale factors
    private double velocityScaleFactor, angularVelocityScaleFactor;

    /**
     * Glauert rotor is modeled with a constant airfoil and twisted rotor blade.
     *
     * @param numberOfBlade : number of blade
     * @param bladeRadius : blade radius (m)
     * @param bladeChord : blade chord (m)
     * @param bladeCenterOfGravityLocation : chordwide location of center of
     * gravity from leading edge (ratio versus chord length)
     * @param bladeTwist : blade twist (deg)
     * @param torsionalRigidity : GJ factor of the blade (N.m2/rad)
     * @param tipLossFactor : tip loss factor.
     * @param bladeFlappingMomentOfInertia : blade flapping moment of inertia (kg.m2)
     * @param bladeLaggingMomentOfInertia : blade lagging moment of inertia (kg.m2)
     * @param bladeAirfoil : blade airfoil section.
     * @param inducedVelocity : induced velocity model.
     */
    public GlauertRotor(int numberOfBlade,
            double bladeRadius, double bladeChord, double bladeCenterOfGravityLocation,
            double bladeTwist, double torsionalRigidity, double tipLossFactor,
            double bladeFlappingMomentOfInertia, double bladeLaggingMomentOfInertia,
            ConstantAirfoil bladeAirfoil,
            ConstantInducedVelocity inducedVelocity) {
        b = numberOfBlade;
        R = bladeRadius;
        c = bladeChord;
        xcg = bladeCenterOfGravityLocation;
        thetaTW = bladeTwist;
        GJ = torsionalRigidity;
        B = tipLossFactor;
        Ib = bladeFlappingMomentOfInertia;
        Ic = bladeLaggingMomentOfInertia;
        this.bladeAirfoil = bladeAirfoil;
        this.inducedVelocity = inducedVelocity;

        // Computation of final datas
        A = FastMath.PI * FastMath.pow(R, 2);
        s = b * c * R / A;
    }

    /**
     *
     * @param velocity
     * @param incidence
     * @param bladePitch
     * @param pitchRate
     * @param rollRate
     * @param rotorSpeed
     * @param atmosphere
     */
    @Override
    public void computeState(double velocity, double incidence, double bladePitch,
            double pitchRate, double rollRate, double rotorSpeed, Atmosphere atmosphere) {
        // Compute atmospheric factor dependant
        Gamma = atmosphere.getDensity() * bladeAirfoil.getLiftSlope()
                * c * FastMath.pow(R, 4) / Ib;
        rho = atmosphere.getDensity();
        U = velocity;
        alphaS = incidence;
        theta0 = bladePitch;
        omega = rotorSpeed;
        this.pitchRate = pitchRate;
        this.rollRate = rollRate;
        q = pitchRate / rotorSpeed;
        p = rollRate / rotorSpeed;

        // Calculate induced velocity
        inducedVelocity.compute(this);

        // Calculate rotor state
        computeAllStates();
    }

    /**
     *
     * @param velocity
     * @param incidence
     * @param bladePitch
     * @param pitchRate
     * @param rollRate
     * @param atmosphere
     */
    @Override
    public void computeState(double velocity, double incidence, double bladePitch,
            double pitchRate, double rollRate, Atmosphere atmosphere) {
        Gamma = atmosphere.getDensity() * this.bladeAirfoil.getLiftSlope()
                * c * FastMath.pow(R, 4) / this.Ib;
        rho = atmosphere.getDensity();
        U = velocity;
        alphaS = incidence;
        theta0 = bladePitch;
        this.pitchRate = pitchRate;
        this.rollRate = rollRate;

        // Calculate rotor speed
        iterativeRotorSpeedComputation(this);

        // Normalisation of omega
        q = pitchRate / omega;
        p = rollRate / omega;

        // Recompute induced velocity once omega has converged
        inducedVelocity.compute(this);

        // Calculate rotor state
        computeAllStates();
    }

    @Override
    public double computeTrustVelocity(double vi) {

        // Calculate advance and inflow ratio
        mu = getTangentialVelocity() / R / omega;
        lambda = (getNormalVelocity() - vi) / R / omega;

        // Compute pitch deformation coefficient
        computePitchDeformationCoefficient();

        // Compute first order flapping coefficients
        computeFlappingCoefficient();

        // Compute thrust coefficient
        computeThrustCoefficient();

        // Return TrustVelocity
        return FastMath.signum(ct) * FastMath.sqrt(s * FastMath.pow(R * omega, 2)
                * FastMath.abs(ct) / 2);
    }

    private class RotorSpeedZeroFunction implements UnivariateFunction {

        final private TrustVelocitylRotor rotor;

        public RotorSpeedZeroFunction(TrustVelocitylRotor rotor) {
            this.rotor = rotor;
        }

        @Override
        public double value(double inputOmega) {

            // Initialise rotor speed for 
            omega = inputOmega;

            // Normalized pitch and roll rate
            q = pitchRate / omega;
            p = rollRate / omega;

            // Calculate induced velocity
            inducedVelocity.compute(rotor);

            // Calculate advance and inflow ratio with new induced velocity
            mu = U * FastMath.cos(alphaS) / R / omega;
            lambda = (U * FastMath.sin(alphaS) - inducedVelocity.getVelocity())
                    / R / omega;

            // Compute pitch deformation coefficients
            computePitchDeformationCoefficient();

            // Compute first order flapping coefficients
            computeFlappingCoefficient();

            // Compute torque coefficients
            computeTorqueCoefficient();

            // Return zero function (total torque)
            return cqp + cqi;
        }
    }

    private void iterativeRotorSpeedComputation(TrustVelocitylRotor rotor) {

        RotorSpeedZeroFunction f = new RotorSpeedZeroFunction(rotor);

        // Compute start value with mu=0.5
        final double startValue = U * FastMath.cos(alphaS) / (0.5 * R);

        // Compute end value so that evaluation at start and end
        // have not the same sign
        double endValue = startValue;
        do {
            endValue = endValue + 5;
        } while (f.value(endValue) > 0 && endValue < 500);

        // Solve with BrentSolver
        BrentSolver solver = new BrentSolver(1.e-8, 1.e-10);
        try {
            omega = solver.solve(100, f, startValue, endValue);
        } catch (TooManyEvaluationsException | NoBracketingException ex) {
            throw new RotorSpeedException(
                    "Glauert rotor speed failed to converge :\n"
                    + "Flight velocity = " + String.format("%.2f", U) + " m/s"
                    + " Incidence NFP = " + String.format("%.2f",
                            FastMath.toDegrees(alphaS)) + " deg"
                    + " Blade pitch = " + String.format("%.2f",
                            FastMath.toDegrees(theta0)) + " deg"
                    + " Blade twist = " + String.format("%.2f",
                            FastMath.toDegrees(thetaTW)) + " deg"
                    + " Normalized Roll rate = " + String.format("%.4f", p)
                    + " Normalized Pitch rate = " + String.format("%.4f", q),
                    ex);
        }
    }

    private void computeAllStates() {

        // Calculate advance and inflow ratio
        mu = U * FastMath.cos(alphaS) / R / omega;
        lambda = (U * FastMath.sin(alphaS) - inducedVelocity.getVelocity())
                / R / omega;

        // Compute pitch deformation coefficients
        computePitchDeformationCoefficient();

        // Compute first order flapping coefficients
        computeFlappingCoefficient();

        // Compute torque coefficients
        computeTorqueCoefficient();

        // Compute thrust coefficient
        computeThrustCoefficient();

        // Compute H and Y coefficients
        computeRearAndLateralForcesCoefficients();

        // Compute harmonic forces and torque coefficients
        computeHarmonicCoefficients();

        // Calculate scale factor
        // Factor for b blades
        forceScaleFactor = rho * s * A * FastMath.pow(R * omega, 2);
        momentScaleFactor = forceScaleFactor * R;
        velocityScaleFactor = R * omega;
        angularVelocityScaleFactor = omega;

        // Calculate force and torque
        T = forceScaleFactor * ct;
        Hp = forceScaleFactor * chp;
        Hi = forceScaleFactor * chi;
        Yi = forceScaleFactor * cyi;
        Qp = momentScaleFactor * cqp;
        Qi = momentScaleFactor * cqi;

        // Harmonic coefficients
        // Factor for one blade
        final double KF1 = forceScaleFactor / b;
        final double KQ1 = momentScaleFactor / b;
        tp1S = KF1 * tp1S;
        tp1C = KF1 * tp1C;
        tp2S = KF1 * tp2S;
        tp2C = KF1 * tp2C;
        hpP1S = KF1 * hpP1S;
        hpP2C = KF1 * hpP2C;
        hiP1S = KF1 * hiP1S;
        hiP1C = KF1 * hiP1C;
        hiP2S = KF1 * hiP2S;
        hiP2C = KF1 * hiP2C;
        ypP1C = KF1 * ypP1C;
        ypP2S = KF1 * ypP2S;
        yiP1S = KF1 * yiP1S;
        yiP1C = KF1 * yiP1C;
        yiP2S = KF1 * yiP2S;
        yiP2C = KF1 * yiP2C;
        qpP1S = KQ1 * qpP1S;
        qpP2C = KQ1 * qpP2C;
        qiP1S = KQ1 * qiP1S;
        qiP1C = KQ1 * qiP1C;
        qiP2C = KQ1 * qiP2C;
        qiP2S = KQ1 * qiP2S;

        // Lift and drag
        L = T * FastMath.cos(alphaS) - (Hi + Hp) * FastMath.sin(alphaS);
        D = T * FastMath.sin(alphaS) + (Hi + Hp) * FastMath.cos(alphaS);

        // Polar coefficient
        CL = getLift() / (0.5 * rho * A * FastMath.pow(U, 2));
        CD = getDrag() / (0.5 * rho * A * FastMath.pow(U, 2));
    }

    // Compute pitch deformation coefficients from rotor characteristics and
    // omega, mu, lambda
    private void computePitchDeformationCoefficient() {
        // Compute deformation only if GJ!=0
        if (GJ != 0) {

            // Airfoil section numbers
            final double a = bladeAirfoil.getLiftSlope();
            final double Cm = bladeAirfoil.getMomentCoefficient();
            final double Icb = Ic - Ib;
            final double l = (xcg - bladeAirfoil.getAerodynamicCenterLocation()) * c;
            final double K = GJ / R;

            // Declaration
            final double[][] m = new double[25][25];
            final double[] n = new double[25];

            // Mapple generated code -------------------------------------------
            {
                final double t1 = 0.1e1 / FastMath.PI;
                final double t2 = B * B;
                final double t4 = t1 * t2 * mu;
                final double t5 = R * R;
                final double t6 = t5 * R;
                final double t7 = rho * t6;
                final double t8 = t7 * c;
                final double t9 = t4 * t8;
                final double t10 = omega * omega;
                final double t11 = t10 * l;
                final double t12 = t11 * a;
                final double t13 = t2 * t2;
                final double t14 = mu * mu;
                final double t15 = t14 * mu;
                final double t16 = t13 * t15;
                final double t18 = t14 * t14;
                final double t19 = t18 * t15;
                final double t20 = Gamma * Gamma;
                final double t21 = t19 * t20;
                final double t22 = t13 * t13;
                final double t23 = t21 * t22;
                final double t24 = 56 * t23;
                final double t25 = t13 * B;
                final double t27 = t25 * t14 * FastMath.PI;
                final double t29 = t18 * t18;
                final double t30 = t29 * mu;
                final double t31 = t30 * t20;
                final double t32 = t13 * t2;
                final double t33 = t31 * t32;
                final double t35 = t22 * t13;
                final double t37 = t35 * t15 * t20;
                final double t39 = t2 * B;
                final double t40 = t13 * t39;
                final double t41 = t22 * t40;
                final double t43 = t41 * FastMath.PI * t20;
                final double t46 = t39 * t18 * FastMath.PI;
                final double t48 = t22 * t2;
                final double t49 = t48 * t20;
                final double t50 = t18 * mu;
                final double t51 = t49 * t50;
                final double t53 = t40 * FastMath.PI;
                final double t56 = t22 * t39;
                final double t58 = FastMath.PI * t20;
                final double t59 = t56 * t18 * t58;
                final double t61 = t22 * t25;
                final double t63 = t14 * FastMath.PI;
                final double t64 = t61 * t20 * t63;
                final double t66 = t22 * B;
                final double t67 = t18 * t14;
                final double t69 = t66 * t67 * t58;
                final double t71 = t2 * t50;
                final double t73 = -27648 * t16 - t24 - 2592 * t27 + 48 * t33 - 192
                        * t37 + 72 * t43 - 19440 * t46 - 304 * t51 + 10368 * t53
                        - 17280 * t19 - 135 * t59 - 34 * t64 + 36 * t69 - 43776 * t71;
                final double t75 = 2 * t2;
                final double t77 = 0.1e1 / (t75 + t14);
                final double t78 = t22 * t20;
                final double t80 = 0.1e1 / (144 + t78);
                final double t81 = t77 * t80;
                final double t88 = t8 * t10 * t39 * a;
                final double t89 = l * mu;
                final double t90 = t89 * Gamma;
                final double t105 = -552960 * t16 - 928 * t23 - 70848 * t27 + 924
                        * t33 - 3840 * t37 + 1440 * t43 - 360288 * t46 - 5760 * t51
                        + 207360 * t53 - 311040 * t19 - 2502 * t59 - 800 * t64
                        + 693 * t69 - 829440 * t71;
                final double t111 = t1 * t13;
                final double t112 = t111 * mu;
                final double t113 = t112 * t8;
                final double t128 = -414720 * t16 - 616 * t23 - 61920 * t27 + 672
                        * t33 - 2880 * t37 + 1080 * t43 - 257040 * t46 - 4176 * t51
                        + 155520 * t53 - 217728 * t19 - 1785 * t59 - 654 * t64 + 504
                        * t69 - 601344 * t71;
                final double t134 = t25 * t1;
                final double t135 = t134 * mu;
                final double t136 = t135 * t8;
                final double t151 = -387072 * t16 - 528 * t23 - 63360 * t27 + 612
                        * t33 - 2688 * t37 + 1008 * t43 - 231552 * t46 - 3808 * t51
                        + 145152 * t53 - 193536 * t19 - 1608 * t59 - 644 * t64 + 459
                        * t69 - 548352 * t71;
                final double t159 = t6 * c;
                final double t160 = t159 * t10;
                final double t161 = Gamma * t1 * rho * t160;
                final double t162 = t32 * a;
                final double t163 = t162 * l;
                final double t178 = -193536 * t16 - 248 * t23 - 33696 * t27 + 300
                        * t33 - 1344 * t37 + 504 * t43 - 112752 * t46 - 1872 * t51
                        + 72576 * t53 - 93312 * t19 - 783 * t59 - 334 * t64 + 225
                        * t69 - 269568 * t71;
                final double t184 = K * FastMath.PI;
                final double t185 = t184 * t2;
                final double t186 = 1244160 * t185;
                final double t187 = t184 * t14;
                final double t188 = 622080 * t187;
                final double t189 = t10 * Icb;
                final double t191 = FastMath.PI * t48 * t20;
                final double t192 = t189 * t191;
                final double t194 = t184 * t49;
                final double t195 = 8640 * t194;
                final double t197 = t78 * t14;
                final double t198 = t189 * FastMath.PI * t197;
                final double t200 = t184 * t197;
                final double t201 = 4320 * t200;
                final double t202 = t10 * rho;
                final double t203 = t202 * t159;
                final double t204 = a * l;
                final double t208 = t203 * t204 * t18 * t2 * FastMath.PI;
                final double t210 = t25 * a;
                final double t211 = l * t30;
                final double t214 = t203 * t210 * t211 * t20;
                final double t217 = t22 * rho * t160;
                final double t218 = t20 * t67;
                final double t220 = t204 * t218 * FastMath.PI;
                final double t221 = t217 * t220;
                final double t222 = 9 * t221;
                final double t223 = FastMath.PI * t2;
                final double t224 = t189 * t223;
                final double t226 = t189 * t63;
                final double t228 = t40 * a;
                final double t229 = l * t19;
                final double t232 = t203 * t228 * t229 * t20;
                final double t235 = a * FastMath.PI;
                final double t237 = t203 * t32 * l * t235;
                final double t239 = t22 * t32;
                final double t243 = t203 * t239 * l * t235 * t20;
                final double t245 = t13 * a;
                final double t246 = l * t14;
                final double t249 = t203 * t245 * t246 * FastMath.PI;
                final double t251 = c * t10;
                final double t253 = t7 * t251 * t35;
                final double t255 = t204 * t63 * t20;
                final double t256 = t253 * t255;
                final double t259 = t7 * t251 * t48;
                final double t260 = t18 * t20;
                final double t262 = t204 * t260 * FastMath.PI;
                final double t263 = t259 * t262;
                final double t265 = t186 + t188 + 4320 * t192 + t195 + 2160 * t198
                        + t201 + 12960 * t208 + 12 * t214 + t222 + 622080 * t224
                        + 311040 * t226 + 16 * t232 + 10368 * t237 + 72 * t243 + 38016
                        * t249 + 260 * t256 + 90 * t263;
                final double t269 = t10 * t1;
                final double t273 = 20736 * t8 * t40 * l * t235;
                final double t275 = t7 * c * t25;
                final double t276 = t204 * t63;
                final double t277 = t275 * t276;
                final double t280 = t7 * c * t41;
                final double t281 = t204 * t58;
                final double t282 = t280 * t281;
                final double t286 = t18 * FastMath.PI;
                final double t287 = t204 * t286;
                final double t288 = t7 * c * t39 * t287;
                final double t290 = c * a;
                final double t291 = t7 * t290;
                final double t292 = t32 * t20;
                final double t297 = t66 * rho * t159;
                final double t298 = t297 * t220;
                final double t301 = t7 * c * t61;
                final double t302 = t301 * t255;
                final double t305 = t7 * c * t56;
                final double t306 = t305 * t262;
                final double t308 = Icb * FastMath.PI;
                final double t309 = t308 * t49;
                final double t311 = t308 * t197;
                final double t313 = t308 * t2;
                final double t315 = t308 * t14;
                final double t318 = t7 * c * t22;
                final double t319 = t204 * t21;
                final double t320 = t318 * t319;
                final double t322 = t273 + 62208 * t277 + 144 * t282 + 19440 * t288
                        + 24 * t291 * t211 * t292 + 18 * t298 + 424 * t302 + 135
                        * t306 + 4320 * t309 + 2160 * t311 + 622080 * t313 + 311040
                        * t315 + 32 * t320;
                final double t329 = t7 * c * t13 * t287;
                final double t332 = t48 * rho * t159;
                final double t333 = t332 * t220;
                final double t335 = t40 * t20;
                final double t344 = t7 * c * t32;
                final double t345 = t344 * t276;
                final double t347 = t22 * a;
                final double t348 = l * FastMath.PI;
                final double t350 = t8 * t347 * t348;
                final double t352 = t297 * t319;
                final double t354 = t22 * t22;
                final double t356 = t7 * c * t354;
                final double t357 = t356 * t281;
                final double t360 = t7 * c * t239;
                final double t361 = t360 * t255;
                final double t364 = t7 * c * t35;
                final double t365 = t364 * t262;
                final double t367 = 81648 * t329 + 90 * t333 + 120 * t291 * t211
                        * t335 + 15120 * t309 + 7560 * t311 + 2177280 * t313 + 1088640
                        * t315 + 277344 * t345 + 103680 * t350 + 160 * t352 + 720
                        * t357 + 1886 * t361 + 567 * t365;
                final double t372 = t280 * t255;
                final double t374 = t275 * t287;
                final double t376 = t301 * t262;
                final double t378 = t204 * t31;
                final double t381 = t305 * t220;
                final double t387 = t66 * l;
                final double t389 = t8 * t387 * t235;
                final double t392 = t291 * t229 * t49;
                final double t396 = t7 * c * t40 * t276;
                final double t398 = t354 * B;
                final double t401 = t7 * c * t398 * t281;
                final double t403 = 874 * t372 + 36288 * t374 + 252 * t376 + 60
                        * t318 * t378 + 45 * t381 + 6048 * t309 + 3024 * t311 + 870912
                        * t313 + 435456 * t315 + 51840 * t389 + 80 * t392 + 128736
                        * t396 + 360 * t401;
                final double t409 = t48 * a;
                final double t411 = t8 * t409 * t348;
                final double t416 = t56 * t20;
                final double t418 = t291 * t229 * t416;
                final double t420 = t344 * t287;
                final double t422 = t318 * t276;
                final double t424 = t356 * t255;
                final double t426 = t360 * t262;
                final double t428 = t354 * t2;
                final double t431 = t7 * c * t428 * t281;
                final double t435 = t364 * t220;
                final double t437 = 36288 * t309 + 362880 * t411 + 18144 * t311
                        + 5225472 * t313 + 2612736 * t315 + 560 * t418 + 233280
                        * t420 + 855360 * t422 + 5800 * t424 + 1620 * t426 + 2520
                        * t431 + 420 * t297 * t378 + 315 * t435;
                final double t444 = t202 * t6;
                final double t445 = t1 * t39 * Gamma * t444;
                final double t446 = t290 * l;
                final double t448 = t20 * t50;
                final double t449 = t448 * t22;
                final double t451 = t39 * t14;
                final double t452 = t451 * FastMath.PI;
                final double t454 = t20 * t14;
                final double t455 = t56 * FastMath.PI;
                final double t456 = t454 * t455;
                final double t458 = t21 * t32;
                final double t459 = 12 * t458;
                final double t460 = t20 * t66;
                final double t461 = t460 * t286;
                final double t462 = 9 * t461;
                final double t463 = t2 * t15;
                final double t466 = t48 * t15 * t20;
                final double t468 = t25 * FastMath.PI;
                final double t471 = t61 * FastMath.PI * t20;
                final double t479 = t444 * t446;
                final double t480 = t14 * Gamma;
                final double t481 = t480 * t13;
                final double t482 = 62208 * t50;
                final double t515 = t1 * t32;
                final double t517 = t515 * rho * t160;
                final double t518 = t204 * t14;
                final double t519 = 41472 * t50;
                final double t534 = t269 * rho;
                final double t536 = t159 * a;
                final double t554 = 9 * t59;
                final double t556 = 12 * t23;
                final double t557 = 864 * t27;
                final double t558 = 6 * t64;
                final double t560 = 16 * t51;
                final double t601 = 1728 * t53;
                final double t608 = t204 * mu;
                final double t619 = t89 * t1;
                final double t621 = 4320 * t63;
                final double t622 = FastMath.PI * t22;
                final double t623 = t622 * t454;
                final double t625 = t21 * t39;
                final double t627 = t292 * t286;
                final double t629 = t25 * t20;
                final double t630 = t629 * t50;
                final double t643 = 60 * t625;
                final double t644 = 45 * t627;
                final double t645 = 80 * t630;
                final double t663 = t32 * mu;
                final double t664 = t663 * t1;
                final double t670 = 84 * t625;
                final double t671 = 63 * t627;
                final double t672 = 112 * t630;
                final double t679 = t40 * mu;
                final double t692 = t15 * Gamma;
                final double t693 = t692 * B;
                final double t706 = t1 * (1440 * t50 + 10 * t449 + 6480 * t452
                        + 37 * t456 + 1728 * t463 + 12 * t466 + 5184 * t468 + 36
                        * t471 - 4 * t458 - 12 * t461) * t81;
                final double t710 = t463 * Gamma;
                final double t723 = t1 * (25920 * t50 + 180 * t449 + 120096 * t452
                        + 680 * t456 + 34560 * t463 + 240 * t466 + 103680 * t468
                        + 720 * t471 - 77 * t458 - 231 * t461) * t81;
                final double t727 = t692 * t39;
                final double t740 = t1 * (18144 * t50 + 126 * t449 + 85680 * t452
                        + 483 * t456 + 25920 * t463 + 180 * t466 + 77760 * t468
                        + 540 * t471 - 56 * t458 - 168 * t461) * t81;
                final double t744 = t692 * t1;
                final double t757 = t13 * (16128 * t50 + 112 * t449 + 77184 * t452
                        + 434 * t456 + 24192 * t463 + 168 * t466 + 72576 * t468 + 504
                        * t471 - 51 * t458 - 153 * t461) * t81;
                final double t762 = t15 * t25 * t1;
                final double t775 = Gamma * (7776 * t50 + 54 * t449 + 37584 * t452
                        + 211 * t456 + 12096 * t463 + 84 * t466 + 36288 * t468 + 252
                        * t471 - 25 * t458 - 75 * t461) * t81;
                final double t787 = (10368 * t223 + 72 * t191 + t621 + 32 * t623
                        + t625 + 3 * t627) * t77 * t80;
                final double t801 = t451 * t1;
                final double t810 = (25920 * t223 + 180 * t191 + 9072 * t63 + 73
                        * t623 + 5 * t625 + 15 * t627) * t77 * t80;
                final double t814 = t14 * t1;
                final double t824 = (9072 * t223 + 63 * t191 + 3024 * t63 + 25
                        * t623 + 2 * t625 + 6 * t627) * t77 * t80;
                final double t828 = t814 * t25;
                final double t837 = (145152 * t223 + 1008 * t191 + 46656 * t63
                        + 394 * t623 + 35 * t625 + 105 * t627) * t77 * t80;
                final double t841 = t18 * Gamma;
                final double t845 = t15 * t22 * t20;
                final double t847 = t39 * FastMath.PI;
                final double t849 = t455 * t20;
                final double t851 = t448 * t32;
                final double t853 = t454 * t66 * FastMath.PI;
                final double t854 = 3 * t853;
                final double t857 = t1 * (8640 * t15 + 60 * t845 + 26208 * t847
                        + 180 * t849 - t851 - t854) * t81;
                final double t868 = t1 * (5184 * t15 + 36 * t845 + 15840 * t847
                        + 108 * t849 - t851 - t854) * t81;
                final double t879 = t1 * (4032 * t15 + 28 * t845 + 12384 * t847
                        + 84 * t849 - t851 - t854) * t81;
                final double t890 = t1 * (3456 * t15 + 24 * t845 + 10656 * t847
                        + 72 * t849 - t851 - t854) * t81;
                final double t902 = 15552 * t15 + 108 * t845 + 48096 * t847 + 324
                        * t849 - 5 * t851 - 15 * t853;
                final double t904 = Gamma * t902 * t81;
                final double t909 = 3 * t456;
                final double t914 = t1 * (270 * t452 + t909 + 90 * t50 + t449 - 108
                        * t468) * t81;
                final double t925 = t1 * (432 * t50 + 5 * t449 + 1296 * t452 + 15
                        * t456 - 576 * t468) * t81;
                final double t934 = t1 * (84 * t50 + t449 + 252 * t452 + t909 - 120
                        * t468) * t81;
                final double t945 = t13 * (576 * t50 + 7 * t449 + 1728 * t452 + 21
                        * t456 - 864 * t468) * t81;
                final double t949 = t692 * t25;
                final double t955 = t1 * (81 * t50 + t449 + 243 * t452 + t909 - 126
                        * t468) * t81;
                final double t959 = t15 * t1;
                final double t962 = t622 * t20;
                final double t964 = t448 * t39;
                final double t967 = t454 * t32 * FastMath.PI;
                final double t982 = (5184 * FastMath.PI + 26 * t962 - 5 * t964
                        - 15 * t967) * t77 * t80;
                final double t992 = (1008 * FastMath.PI + 5 * t962 - t964 - 3
                        * t967) * t77 * t80;
                final double t1002 = (6912 * FastMath.PI + 34 * t962 - 7 * t964
                        - 21 * t967) * t77 * t80;
                final double t1006 = t959 * t32;
                final double t1013 = (3888 * FastMath.PI + 19 * t962 - 4 * t964
                        - 12 * t967) * t77 * t80;
                final double t1020 = 6480 * t14;
                final double t1024 = t18 * t32 * t20;
                final double t1032 = t202 * t536;
                final double t1037 = 231 * t1024;
                final double t1044 = t25 * Gamma;
                final double t1045 = t1044 * t444;
                final double t1049 = 25920 * t2;
                final double t1050 = 180 * t49;
                final double t1057 = t32 * Gamma;
                final double t1061 = 72576 * t2;
                final double t1062 = 504 * t49;
                final double t1070 = t40 * Gamma;
                final double t1071 = t1070 * t444;
                final double t1076 = 75 * t1024;
                final double t1083 = t8 * t518;
                final double t1085 = t204 * t454;
                final double t1086 = t318 * t1085;
                final double t1088 = l * t18;
                final double t1093 = t22 * Icb;
                final double t1094 = t1093 * t20;
                final double t1100 = t40 * t10;
                final double t1101 = t1100 * t8;
                final double t1103 = t75 + 3 * t14;
                final double t1105 = t20 * t1103 * t80;
                final double t1106 = t518 * t1105;
                final double t1111 = t10 * t14 * t204 * t20;
                final double t1112 = t332 * t1111;
                final double t1114 = a * t20;
                final double t1116 = t11 * t1114 * t18;
                final double t1117 = t318 * t1116;
                final double t1120 = t22 * K;
                final double t1121 = t1120 * t20;
                final double t1126 = t66 * t10;
                final double t1127 = t1126 * t8;
                final double t1132 = t13 * Gamma;
                final double t1135 = t14 * t32 * t20;
                final double t1136 = 60 * t78;
                final double t1143 = 36 * t78;
                final double t1150 = t32 * t10;
                final double t1151 = t1150 * t8;
                final double t1152 = 28 * t78;
                final double t1159 = 24 * t78;
                final double t1168 = 5 * t1135;
                final double t1169 = 108 * t78;
                final double t1177 = t7 * t251 * t39;
                final double t1187 = t7 * t251 * t13;
                final double t1189 = 5 * t197;
                final double t1197 = t25 * mu;
                final double t1199 = Gamma * t10 * rho;
                final double t1228 = t608 * t1105;
                final double t1238 = t56 * rho * t160;
                final double t1241 = Gamma * mu;
                final double t1242 = t1241 * B;
                final double t1247 = t2 * mu * Gamma;
                final double t1251 = t1241 * t39;
                final double t1255 = t1241 * t1;
                final double t1262 = l * B;
                final double t1268 = t291 * t246 * t191;
                final double t1270 = t8 * t287;
                final double t1273 = t291 * t1088 * t962;
                final double t1284 = t291 * t229 * t629;
                final double t1295 = l * t39;
                final double t1319 = t480 * t2;
                final double t1323 = t480 * t39;
                final double t1330 = t480 * t25;
                final double t1367 = t1 * mu * t39;
                final double t1380 = t479 / 40;
                final double t1382 = t444 * t290 * t246;
                final double t1383 = t1382 / 160;
                final double t1384 = t189 / 20;
                final double t1387 = t444 * t290 * t89;
                final double t1388 = t1387 / 40;
                final double t1389 = l * t15;
                final double t1390 = t1389 * t39;
                final double t1391 = 49 * t78;
                final double t1396 = 0.1e1 / (t75 - t14);
                final double t1412 = t25 * t10 * t8;
                final double t1413 = 1296 * t2;
                final double t1428 = 51 * t1024;
                final double t1448 = t32 * rho * t6;
                final double t1451 = t204 * Gamma;
                final double t1453 = t1103 * t80 * t1396;
                final double t1458 = t8 * t1100 * a;
                final double t1459 = t246 * Gamma;
                final double t1460 = t1459 * t1453;
                final double t1465 = t8 * t10 * t22 * a;
                final double t1469 = t8 * t1126 * a;
                final double t1474 = t8 * t10 * t48 * a;
                final double t1477 = t189 * t49;
                final double t1480 = K * t48 * t20;
                final double t1482 = t189 * t14;
                final double t1484 = t189 * t2;
                final double t1486 = t1120 * t454;
                final double t1488 = t189 * t197;
                final double t1491 = t8 * t1150 * t204;
                final double t1493 = t245 * t246;
                final double t1494 = t203 * t1493;
                final double t1496 = K * t14;
                final double t1498 = K * t2;
                final double t1500 = t364 * t1111;
                final double t1502 = t332 * t1116;
                final double t1506 = a * t18;
                final double t1508 = t2 * rho * t159 * t11 * t1506;
                final double t1510 = t11 * t1114;
                final double t1511 = t360 * t1510;
                final double t1513 = 1080 * t1477 + 2160 * t1480 - 77760 * t1482
                        + 155520 * t1484 - 1080 * t1486 - 540 * t1488 + 2592 * t1491
                        - 18144 * t1494 - 155520 * t1496 + 311040 * t1498 - 127 * t1500
                        - 69 * t1502 - 9720 * t1508 + 18 * t1511;
                final double t1518 = t8 * t210 * t246;
                final double t1520 = t301 * t1085;
                final double t1522 = t228 * l;
                final double t1523 = t8 * t1522;
                final double t1527 = t8 * t41 * l * t1114;
                final double t1531 = t8 * t39 * a * t1088;
                final double t1533 = t204 * t260;
                final double t1534 = t305 * t1533;
                final double t1536 = Icb * t2;
                final double t1538 = Icb * t14;
                final double t1541 = Icb * t48 * t20;
                final double t1543 = t1093 * t454;
                final double t1547 = t80 * t1396;
                final double t1551 = t8 * t162 * t246;
                final double t1553 = t360 * t1085;
                final double t1557 = t8 * t22 * l * a;
                final double t1561 = t8 * t354 * l * t1114;
                final double t1564 = t8 * t245 * t1088;
                final double t1566 = t364 * t1533;
                final double t1576 = t14 * t40;
                final double t1578 = t8 * t204 * t1576;
                final double t1580 = t280 * t1085;
                final double t1583 = t8 * t387 * a;
                final double t1587 = t8 * t398 * l * t1114;
                final double t1590 = t8 * t210 * t1088;
                final double t1592 = t301 * t1533;
                final double t1604 = t8 * t204 * t14 * t22;
                final double t1606 = t356 * t1085;
                final double t1608 = t409 * l;
                final double t1609 = t8 * t1608;
                final double t1612 = l * t20;
                final double t1614 = t8 * t428 * a * t1612;
                final double t1617 = t8 * t162 * t1088;
                final double t1619 = t360 * t1533;
                final double t1648 = t1197 * t444;
                final double t1651 = 12096 * t14;
                final double t1659 = t663 * t444;
                final double t1670 = t679 * t444;
                final double t1689 = t90 * t1453;
                final double t1696 = t8 * t10 * t56 * a;
                final double t1699 = t1389 * t2;
                final double t1700 = 45 * t78;
                final double t1704 = (6480 + t1700 + 8 * t1135) * t80 * t1396;
                final double t1708 = 432 * t78;
                final double t1712 = (62208 + t1708 + 77 * t1135) * t80 * t1396;
                final double t1716 = t1389 * t13;
                final double t1720 = t1389 * t25;
                final double t1725 = (13824 + 96 * t78 + 17 * t1135) * t80 * t1396;
                final double t1729 = t1389 * t32;
                final double t1734 = (81648 + 567 * t78 + 100 * t1135) * t80 * t1396;
                final double t1740 = t25 * t80 * t1396;
                final double t1744 = t1506 * l;
                final double t1746 = Gamma * t80 * t1396;
                final double t1747 = t1744 * t1746;
                final double t1763 = (3240 * t14 + 23 * t197 + 2592 * t2 + 18 * t49) * t80 * t1396;
                final double t1779 = t246 * t39;
                final double t1780 = 27216 * t14;
                final double t1781 = 199 * t197;
                final double t1784 = (t1780 + t1781 + t1049 + t1050) * t80 * t1396;
                final double t1788 = t246 * t13;
                final double t1789 = 9072 * t14;
                final double t1795 = (t1789 + 67 * t197 + 9072 * t2 + 63 * t49) * t80 * t1396;
                final double t1799 = t246 * t25;
                final double t1804 = (69984 * t14 + 521 * t197 + t1061 + t1062) * t80 * t1396;
                final double t1818 = (2592 + 13 * t78) * t80 * t1396;
                final double t1822 = 5 * t78;
                final double t1825 = (1008 + t1822) * t80 * t1396;
                final double t1832 = (3456 + 17 * t78) * t80 * t1396;
                final double t1839 = (3888 + 19 * t78) * t80 * t1396;
                final double t1843 = t1389 * Gamma;
                final double t1845 = t32 * t80 * t1396;
                final double t1847 = t1032 * t1843 * t1845;
                final double t1849 = t15 * a;
                final double t1851 = t1849 * l * t1746;
                final double t1852 = t1101 * t1851;
                final double t1854 = t217 * t1851;
                final double t1856 = t1127 * t1851;
                final double t1860 = t335 * t80;
                final double t1866 = t1032 * t1389 * t78 * t80;
                final double t1870 = t204 * t15 * t20 * t80;
                final double t1871 = t1127 * t1870;
                final double t1873 = t259 * t1870;
                final double t1875 = t1238 * t1870;
                final double t1877 = t1132 * t80;
                final double t1881 = t1044 * t80;
                final double t1883 = t1032 * t246 * t1881;
                final double t1886 = t204 * t480 * t80;
                final double t1887 = t1151 * t1886;
                final double t1889 = t1101 * t1886;
                final double t1891 = t217 * t1886;
                final double t1901 = t460 * t80;
                final double t1903 = t1032 * t246 * t1901;
                final double t1911 = t204 * t454 * t80;
                final double t1912 = t1238 * t1911;
                final double t1914 = t253 * t1911;
                final double t1917 = t1032 * t89 * t1901;
                final double t1921 = t1032 * t89 * t49 * t80;
                final double t1925 = t204 * mu * t20 * t80;
                final double t1926 = t1238 * t1925;
                final double t1928 = t253 * t1925;
                final double t1931 = t7 * t251 * t61;
                final double t1935 = t1032 * t89 * t1881;
                final double t1937 = t1057 * t80;
                final double t1939 = t1032 * t89 * t1937;
                final double t1942 = t204 * t1241 * t80;
                final double t1943 = t1101 * t1942;
                final double t1945 = t217 * t1942;
                final double t1949 = 61 * t78;
                final double t1956 = t89 * t39;
                final double t1960 = t89 * t13;
                final double t1964 = t89 * t25;
                final double t1968 = t89 * t32;
                final double t1976 = t1032 * t1459 * t1845;
                final double t1979 = t40 * t80 * t1396;
                final double t1981 = t1032 * t1459 * t1979;
                final double t1985 = t1451 * t14 * t80 * t1396;
                final double t1986 = t217 * t1985;
                final double t1988 = t1127 * t1985;
                final double t1994 = t291 * t246 * t49;
                final double t2004 = t8 * t1744;
                final double t2007 = t291 * t1088 * t78;
                final double t2070 = t1451 * mu * t80 * t1396;
                final double t2079 = 38880 * t2;
                final double t2127 = (-t14 + t2) * t80 * t1396;
                final double t2128 = t90 * t2127;
                final double t2180 = 12960 * t2;
                final double t2202 = 6480 * t1494 + 13 * t1500 - 4320 * t1491 + 2
                        * t1511 + 34560 * t1498 - 17280 * t1496 + 240 * t1480 - 120
                        * t1486 + 2160 * t1508 + 15 * t1502 + 17280 * t1484 - 8640
                        * t1482 + 120 * t1477 - 60 * t1488;
                final double t2206 = 17280 * t1536;
                final double t2207 = 8640 * t1538;
                final double t2208 = 120 * t1541;
                final double t2209 = 60 * t1543;
                final double t2226 = 24192 * t1536;
                final double t2227 = 12096 * t1538;
                final double t2228 = 168 * t1541;
                final double t2229 = 84 * t1543;
                final double t2258 = t1451 * t2127;
                final double t2263 = rho * Gamma;
                final double t2289 = t246 * t32;
                final double t2325 = t204 * t14 * t2;
                final double t2371 = t259 * t1911;
                final double t2376 = t1032 * t89 * t1877 / 30;
                final double t2377 = 0.2e1 / 0.45e2 * t1935;
                final double t2378 = t1939 / 21;
                final double t2379 = t1943 / 21;
                final double t2380 = 0.5e1 / 0.108e3 * t1945;
                final double t2386 = 0.2e1 / 0.45e2 * t203 * t204 * mu * (135 + t78) * t80;
                final double t2387 = t1917 / 270;
                final double t2388 = t1921 / 252;
                final double t2389 = t1926 / 252;
                final double t2390 = 0.5e1 / 0.1296e4 * t1928;
                final double t2393 = t203 * t204 * t1901 / 30;
                final double t2404 = t10 * (-432 * t1083 - 3 * t1086 + 1728 * Icb + 12 * t1094 + 4 * t8 * t204 * t49) * t80 / 144;
                final double t2408 = t203 * t204 * t416 * t80 / 42;
                final double t2411 = t364 * t1510;
                final double t2414 = (-6912 * K - 48 * t1121 + t2411) * t80 / 48;
                final double t2416 = t1612 * t80;
                final double t2419 = t203 * t61 * a * t2416 / 54;
                final double t2422 = 0.2e1 / 0.5e1 * t203 * t204 * t1881;
                final double t2425 = t203 * t204 * t1937 / 3;
                final double t2429 = 0.2e1 / 0.7e1 * t203 * t204 * t1070 * t80;
                final double t2430 = l * Gamma;
                final double t2431 = t2430 * t80;
                final double t2434 = t203 * t347 * t2431 / 4;
                final double t2438 = 0.2e1 / 0.9e1 * t203 * t66 * a * t2431;
                final double t2439 = t1382 / 80;
                final double t2440 = t2439 - t1384;
                final double t2446 = t218 * t53;
                final double t2449 = B * t18 * FastMath.PI;
                final double t2456 = 20 * t471 + 24 * t456 + 12096 * t468 + 3456
                        * t452 + 57 * t461 - 18 * t2446 + 6480 * t2449 - 5760 * t50
                        - 40 * t449 - 6912 * t463 - 48 * t466 + 16 * t458;
                final double t2476 = 470016 * t468 + 155520 * t452 + 800 * t471
                        + 1080 * t456 + 2082 * t461 - 693 * t2446 + 233280 * t2449
                        - 207360 * t50 - 1440 * t449 - 276480 * t463 - 1920 * t466
                        + 616 * t458;
                final double t2483 = 432 * t456;
                final double t2485 = 62208 * t452;
                final double t2494 = 300 * t471 + t2483 + 172224 * t468 + t2485
                        + 735 * t461 - 252 * t2446 + 81648 * t2449 - 72576 * t50
                        - 504 * t449 - 103680 * t463 - 720 * t466 + 224 * t458;
                final double t2516 = 315648 * t468 + 120960 * t452 + 560 * t471
                        + 840 * t456 + 1314 * t461 - 459 * t2446 + 145152 * t2449
                        - 129024 * t50 - 896 * t449 - 193536 * t463 - 1344 * t466
                        + 408 * t458;
                final double t2531 = 280 * t471 + t2483 + 155520 * t468 + t2485
                        + 636 * t461 - 225 * t2446 + 69984 * t2449 - t482 - 432
                        * t449 - 96768 * t463 - 672 * t466 + 200 * t458;
                final double t2542 = t13 * t18 * t58;
                final double t2543 = 6 * t2542;
                final double t2545 = t2 * t67 * t58;
                final double t2546 = 9 * t2545;
                final double t2547 = t21 * B;
                final double t2548 = 8 * t2547;
                final double t2566 = t1 * t40 * t10 * t8;
                final double t2603 = t10 * t13;
                final double t2605 = t8 * t2603 * a;
                final double t2606 = 4608 * t468;
                final double t2609 = 9 * t2446;
                final double t2614 = 8 * t458;
                final double t2672 = t2430 * t1;
                final double t2673 = 6 * t59;
                final double t2674 = 9 * t69;
                final double t2677 = B * t67 * FastMath.PI;
                final double t2680 = 8 * t23;
                final double t2753 = -4320 * t237 + 2 * t243 + 2160 * t249 + 15
                        * t256 + 17280 * t224 + 120 * t192 + 8640 * t226 + 60 * t198
                        - 2160 * t208 - 21 * t263 + t222 - 8 * t232 + 34560 * t185
                        + 240 * t194 + 17280 * t187 + 120 * t200;
                final double t2768 = -t273 + 16 * t282 + 13824 * t277 + 96 * t302
                        + 69120 * t313 + 480 * t309 + 34560 * t315 + 240 * t311
                        - 8640 * t288 - 90 * t306 + 45 * t298 - 40 * t320;
                final double t2785 = -8064 * t350 + 8 * t357 + 6048 * t345 + 42
                        * t361 + 24192 * t313 + 168 * t309 + 12096 * t315 + 84 * t311
                        - 3024 * t329 - 33 * t365 + 18 * t333 - 16 * t352;
                final double t2802 = -138240 * t389 + 160 * t401 + 110592 * t396
                        + 768 * t372 + 387072 * t313 + 2688 * t309 + 193536 * t315
                        + 1344 * t311 - 48384 * t374 - 546 * t376 + 315 * t381 - 280
                        * t392;
                final double t2819 = -7776 * t411 + 10 * t431 + 6480 * t422 + 45
                        * t424 + 20736 * t313 + 144 * t309 + 10368 * t315 + 72 * t311
                        - 2592 * t420 - 30 * t426 + 18 * t435 - 16 * t418;
                final double t2864 = t246 * t40;
                final double t2882 = t50 * a * t2416;
                final double t2945 = 7 * t78;
                final double t2960 = t1032 * t1088 * t1860;
                final double t2968 = t204 * t260 * t80;
                final double t2969 = t1127 * t2968;
                final double t2971 = t259 * t2968;
                final double t2973 = t1238 * t2968;
                final double t2975 = t480 * B;
                final double t2985 = t480 * t1;
                final double t3195 = t1849 * t2431;
                final double t3196 = t1412 * t3195;
                final double t3198 = t1151 * t3195;
                final double t3200 = t1101 * t3195;
                final double t3202 = t217 * t3195;
                final double t3276 = t1127 * t1886;
                final double t3315 = (4 * Icb - t1083) * t10;
                m[0][0] = t9 * t12 * Gamma * t73 * t81 / 12960;
                m[0][1] = t88 * t90 * t1 * t105 * t81 / 311040;
                m[0][2] = t113 * t12 * Gamma * t128 * t81 / 272160;
                m[0][3] = t136 * t12 * Gamma * t151 * t81 / 290304;
                m[0][4] = t161 * t163 * mu * t178 * t81 / 163296;
                m[0][5] = t1 * t265 * t81 / 4320;
                m[0][6] = t269 * t322 * t77 * t80 / 6480;
                m[0][7] = t269 * t367 * t77 * t80 / 30240;
                m[0][8] = t269 * t403 * t77 * t80 / 15120;
                m[0][9] = t269 * t437 * t77 * t80 / 108864;
                m[0][10] = t445 * t446 * t14 * (-103680 * t50 - 704 * t449 - 78624
                        * t452 - 546 * t456 + t459 + t462 - 138240 * t463 - 960
                        * t466 + 52416 * t468 + 360 * t471) * t81 / 51840;
                m[0][11] = t479 * t481 * t1 * (-t482 - 416 * t449 - 47520 * t452
                        - 330 * t456 + t459 + t462 - 82944 * t463 - 576 * t466 + 31680
                        * t468 + 216 * t471) * t81 / 38880;
                m[0][12] = t25 * rho * t6 * t251 * a * t246 * t1 * Gamma * (-48384
                        * t50 - 320 * t449 - 37152 * t452 - 258 * t456 + t459 + t462
                        - 64512 * t463 - 448 * t466 + 24768 * t468 + 168 * t471)
                        * t81 / 36288;
                m[0][13] = t517 * t518 * Gamma * (-t519 - 272 * t449 - 31968 * t452
                        - 222 * t456 + t459 + t462 - 55296 * t463 - 384 * t466
                        + 21312 * t468 + 144 * t471) * t81 / 36288;
                m[0][14] = t480 * t40 * t534 * t536 * l * (-186624 * t50 - 1216
                        * t449 - 144288 * t452 - 1002 * t456 + 60 * t458 + 45 * t461
                        - 248832 * t463 - 1728 * t466 + 96192 * t468 + 648 * t471) * t81 / 186624;
                m[0][15] = -t9 * t12 * Gamma * (-810 * t46 - t554 - 1080 * t19
                        - t556 + t557 + t558 - 1440 * t71 - t560 - 216 * t53) * t81
                        / 1620;
                m[0][16] = -t88 * t90 * t1 * (-5184 * t19 - 60 * t23 - 3888 * t46
                        - 45 * t59 + 4320 * t27 - 6912 * t71 - 80 * t51 + 30 * t64
                        - 1152 * t53) * t81 / 10368;
                m[0][17] = -t113 * t12 * Gamma * (-1008 * t19 - t556 - 756 * t46
                        - t554 + t557 - 1344 * t71 - t560 + t558 - 240 * t53)
                        * t81 / 2520;
                m[0][18] = -t136 * t12 * Gamma * (-6912 * t19 - 84 * t23 - 5184
                        * t46 - 63 * t59 + 6048 * t27 - 9216 * t71 - 112 * t51 + 42
                        * t64 - t601) * t81 / 20736;
                m[0][19] = -t517 * t608 * Gamma * (-972 * t19 - t556 - 729 * t46
                        - t554 + t557 - 1296 * t71 - t560 + t558 - 252 * t53)
                        * t81 / 3402;
                m[0][20] = t88 * t619 * (4 * t191 + t621 + 30 * t623 - 12 * t625
                        - 9 * t627 - 16 * t630) * t77 * t80 / 360;
                m[0][21] = t112 * t444 * t446 * (5184 * t223 + 56 * t191 + 23328
                        * t63 + 162 * t623 - t643 - t644 - t645) * t77 * t80 / 2160;
                m[0][22] = t135 * t444 * t446 * (8064 * t223 + 76 * t191 + 24192
                        * t63 + 168 * t623 - t643 - t644 - t645) * t77 * t80 / 2520;
                m[0][23] = t664 * t444 * t446 * (13824 * t223 + 124 * t191 + 34560
                        * t63 + 240 * t623 - t670 - t671 - t672) * t77 * t80 / 4032;
                m[0][24] = t679 * t1 * t444 * t446 * (15552 * t223 + 136 * t191
                        + 34992 * t63 + 243 * t623 - t670 - t671 - t672) * t77 * t80
                        / 4536;
                m[1][0] = t479 * t693 * t706 / 2160;
                m[1][1] = t479 * t710 * t723 / 51840;
                m[1][2] = t479 * t727 * t740 / 45360;
                m[1][3] = t479 * t744 * t757 / 48384;
                m[1][4] = t479 * t762 * t775 / 27216;
                m[1][5] = -t479 * t14 * B * t1 * t787 / 720;
                m[1][6] = -t1 * (51840 * t249 + 360 * t256 + 19440 * t208 + 151
                        * t263 + 8 * t214 + 24 * t221 - t186 - t195 - t188 - t201)
                        * t81 / 4320;
                m[1][7] = -t479 * t801 * t810 / 2520;
                m[1][8] = -t479 * t814 * t13 * t824 / 1008;
                m[1][9] = -t479 * t828 * t837 / 18144;
                m[1][10] = t479 * t841 * t2 * t857 / 8640;
                m[1][11] = t479 * t841 * t39 * t868 / 6480;
                m[1][12] = t479 * t841 * t13 * t879 / 6048;
                m[1][13] = t479 * t841 * t25 * t890 / 6048;
                m[1][14] = t479 * t18 * t1 * t32 * t904 / 31104;
                m[1][15] = -t479 * t693 * t914 / 270;
                m[1][16] = -t479 * t710 * t925 / 1728;
                m[1][17] = -t479 * t727 * t934 / 420;
                m[1][18] = -t479 * t744 * t945 / 3456;
                m[1][19] = -t479 * t949 * t955 / 567;
                m[1][20] = -t479 * t959 * t2 * (2160 * FastMath.PI + 11 * t962
                        - 2 * t964 - 6 * t967) * t77 * t80 / 120;
                m[1][21] = -t479 * t959 * t39 * t982 / 360;
                m[1][22] = -t479 * t959 * t13 * t992 / 84;
                m[1][23] = -t479 * t762 * t1002 / 672;
                m[1][24] = -t479 * t1006 * t1013 / 432;
                m[2][0] = -t39 * Gamma * t444 * t446 * mu * (37 * t197 + t1020
                        + 5184 * t2 + 36 * t49 - 12 * t1024) * t80 / 4320;
                m[2][1] = -t1032 * t90 * t13 * (120096 * t14 + 680 * t197 + 103680
                        * t2 + 720 * t49 - t1037) * t80 / 103680;
                m[2][2] = -t1045 * t446 * mu * (161 * t197 + 28560 * t14 - 56
                        * t1024 + t1049 + t1050) * t80 / 30240;
                m[2][3] = -t1057 * t444 * t446 * mu * (77184 * t14 + 434 * t197
                        + t1061 + t1062 - 153 * t1024) * t80 / 96768;
                m[2][4] = -t1071 * t446 * mu * (211 * t197 + 37584 * t14 + 36288
                        * t2 + 252 * t49 - t1076) * t80 / 54432;
                m[2][5] = -t10 * (-4320 * t1083 - 32 * t1086 - 3 * t291 * t1088
                        * t292 + 34560 * Icb + 240 * t1094) * t80 / 1440;
                m[2][6] = t1101 * t1106 / 1080;
                m[2][7] = (2 * t1112 + 3 * t1117 + 145152 * K + 1008 * t1121)
                        * t80 / 1008;
                m[2][8] = t1127 * t1106 / 1008;
                m[2][9] = 0.5e1 / 0.5184e4 * t259 * t1106;
                m[2][10] = -t1132 * t444 * t446 * t14 * (8736 - t1135 + t1136)
                        * t80 / 5760;
                m[2][11] = -t1045 * t446 * t14 * (5280 - t1135 + t1143) * t80
                        / 4320;
                m[2][12] = -t1151 * t518 * Gamma * (4128 - t1135 + t1152) * t80
                        / 4032;
                m[2][13] = -t1071 * t446 * t14 * (3552 - t1135 + t1159) * t80
                        / 4032;
                m[2][14] = -t22 * Gamma * t444 * t446 * t14 * (16032 - t1168 + t1169)
                        * t80 / 20736;
                m[2][15] = t1177 * t608 * Gamma * (-36 * t2 + t197 + 90 * t14)
                        * t80 / 180;
                m[2][16] = t1187 * t608 * Gamma * (-192 * t2 + t1189 + 432 * t14)
                        * t80 / 1152;
                m[2][17] = t1197 * t1199 * t536 * l * (-40 * t2 + t197 + 84 * t14)
                        * t80 / 280;
                m[2][18] = t663 * t1199 * t536 * l * (-288 * t2 + 7 * t197 + 576
                        * t14) * t80 / 2304;
                m[2][19] = t679 * t1199 * t536 * l * (-42 * t2 + t197 + 81 * t14)
                        * t80 / 378;
                m[2][20] = -t1101 * t1228 / 120;
                m[2][21] = -t217 * t1228 / 144;
                m[2][22] = -t1127 * t1228 / 168;
                m[2][23] = -t259 * t1228 / 192;
                m[2][24] = -t1238 * t1228 / 216;
                m[3][0] = t479 * t1242 * t706 / 3240;
                m[3][1] = t479 * t1247 * t723 / 77760;
                m[3][2] = t479 * t1251 * t740 / 68040;
                m[3][3] = t479 * t1255 * t757 / 72576;
                m[3][4] = t479 * t135 * t775 / 40824;
                m[3][5] = -t1032 * t1262 * t1 * t787 / 1080;
                m[3][6] = -t269 * (32 * t1268 - 19440 * t1270 - 87 * t1273 + 103680
                        * t8 * t204 * t13 * FastMath.PI + 720 * t291 * l * t35 * t58
                        + 16 * t1284 + 311040 * t313 + 2160 * t309 + 155520 * t315
                        + 1080 * t311) * t77 * t80 / 12960;
                m[3][7] = -t1032 * t1295 * t1 * t810 / 3780;
                m[3][8] = -t1 * (9072 * t237 + 63 * t243 + 3024 * t249 + 25 * t256
                        + 2 * t232 + 6 * t263 - 435456 * t185 - 3024 * t194 - 217728
                        * t187 - 1512 * t200) * t81 / 1512;
                m[3][9] = -t1032 * l * t1 * t25 * t837 / 27216;
                m[3][10] = t479 * t1319 * t857 / 12960;
                m[3][11] = t479 * t1323 * t868 / 9720;
                m[3][12] = t479 * t481 * t879 / 9072;
                m[3][13] = t479 * t1330 * t890 / 9072;
                m[3][14] = t479 * t515 * Gamma * t14 * t902 * t81 / 46656;
                m[3][15] = -t479 * t1242 * t914 / 405;
                m[3][16] = -t479 * t1247 * t925 / 2592;
                m[3][17] = -t479 * t1251 * t934 / 630;
                m[3][18] = -t479 * t1255 * t945 / 5184;
                m[3][19] = -0.2e1 / 0.1701e4 * t479 * t1241 * t25 * t955;
                m[3][20] = t1032 * t619 * (8 * t191 + 2160 * t63 + 27 * t623 + 4
                        * t630) * t77 * t80 / 360;
                m[3][21] = -t479 * t1367 * t982 / 540;
                m[3][22] = -t479 * t112 * t992 / 126;
                m[3][23] = -t479 * t135 * t1002 / 1008;
                m[3][24] = -t479 * t664 * t1013 / 648;
                m[4][0] = 0;
                m[4][1] = 0;
                m[4][2] = 0;
                m[4][3] = 0;
                m[4][4] = 0;
                m[4][5] = t1380;
                m[4][6] = 0;
                m[4][7] = t1383 - t1384;
                m[4][8] = 0;
                m[4][9] = K;
                m[4][10] = 0;
                m[4][11] = 0;
                m[4][12] = 0;
                m[4][13] = 0;
                m[4][14] = 0;
                m[4][15] = 0;
                m[4][16] = 0;
                m[4][17] = 0;
                m[4][18] = 0;
                m[4][19] = 0;
                m[4][20] = 0;
                m[4][21] = t1388;
                m[4][22] = 0;
                m[4][23] = 0;
                m[4][24] = 0;
                m[5][0] = -t1032 * t1390 * (t1391 + 6480 + 6 * t1135) * t80 * t1396
                        / 270;
                m[5][1] = t1187 * t608 * (62208 * t2 + 432 * t49 - 2098 * t197
                        - 279936 * t14 - t1037) * t80 * t1396 / 12960;
                m[5][2] = 0.2e1 / 0.405e3 * t1412 * t608 * (t1413 + 9 * t49 - 29
                        * t197 - 3888 * t14 - 3 * t1024) * t80 * t1396;
                m[5][3] = t1151 * t608 * (27648 * t2 + 192 * t49 - 514 * t197 - 69120
                        * t14 - t1428) * t80 * t1396 / 4032;
                m[5][4] = t89 * t536 * t202 * t40 * (46656 * t2 + 324 * t49 - 779
                        * t197 - 104976 * t14 - t1076) * t80 * t1396 / 6804;
                m[5][5] = -t1448 * t251 * t14 * t1451 * t1453 / 180;
                m[5][6] = -t1458 * t1460 / 135;
                m[5][7] = -t1465 * t1460 / 126;
                m[5][8] = -t1469 * t1460 / 126;
                m[5][9] = -0.5e1 / 0.648e3 * t1474 * t1460;
                m[5][10] = t1513 * t80 * t1396 / 1080;
                m[5][11] = t10 * (-62208 * t1518 - 440 * t1520 + 20736 * t1523
                        + 144 * t1527 - 58320 * t1531 - 417 * t1534 + 622080 * t1536
                        - 311040 * t1538 + 4320 * t1541 - 2160 * t1543) * t1547
                        / 6480;
                m[5][12] = t10 * (-204768 * t1551 - 1462 * t1553 + 103680 * t1557
                        + 720 * t1561 - 244944 * t1564 - 1761 * t1566 + 2177280
                        * t1536 - 1088640 * t1538 + 15120 * t1541 - 7560 * t1543)
                        * t1547 / 30240;
                m[5][13] = t10 * (-40176 * t1578 - 289 * t1580 + 25920 * t1583
                        + 180 * t1587 - 54432 * t1590 - 393 * t1592 + 435456 * t1536
                        - 217728 * t1538 + 3024 * t1541 - 1512 * t1543) * t1547
                        / 7560;
                m[5][14] = t10 * (-241056 * t1604 - 1744 * t1606 + 181440 * t1609
                        + 1260 * t1614 - 349920 * t1617 - 2535 * t1619 + 2612736
                        * t1536 - 1306368 * t1538 + 18144 * t1541 - 9072 * t1543)
                        * t1547 / 54432;
                m[5][15] = -t1177 * t608 * (t49 - 1080 * t14 - 6 * t197) * t80
                        * t1396 / 90;
                m[5][16] = -t13 * mu * t444 * t446 * (t1413 + 14 * t49 - 5832 * t14
                        - 33 * t197) * t80 * t1396 / 540;
                m[5][17] = -t1648 * t446 * (4032 * t2 + 38 * t49 - t1651 - 69 * t197)
                        * t80 * t1396 / 1260;
                m[5][18] = -t1659 * t446 * (6912 * t2 + 62 * t49 - 17280 * t14 - 99
                        * t197) * t80 * t1396 / 2016;
                m[5][19] = -t1670 * t446 * (15552 * t2 + 136 * t49 - 34992 * t14
                        - 201 * t197) * t80 * t1396 / 4536;
                m[5][20] = t479 * t1241 * t40 * t1453 / 15;
                m[5][21] = t479 * t1241 * t22 * t1453 / 18;
                m[5][22] = t1469 * t1689 / 21;
                m[5][23] = t1474 * t1689 / 24;
                m[5][24] = t1696 * t1689 / 27;
                m[6][0] = t1032 * t1699 * t1704 / 180;
                m[6][1] = t1032 * t1390 * t1712 / 2160;
                m[6][2] = t1032 * t1716 * t1704 / 270;
                m[6][3] = t1032 * t1720 * t1725 / 672;
                m[6][4] = t1032 * t1729 * t1734 / 4536;
                m[6][5] = t1032 * t1088 * Gamma * t1740 / 30;
                m[6][6] = 0.2e1 / 0.45e2 * t1151 * t1747;
                m[6][7] = t1101 * t1747 / 21;
                m[6][8] = t217 * t1747 / 21;
                m[6][9] = 0.5e1 / 0.108e3 * t1127 * t1747;
                m[6][10] = t1032 * t246 * B * t1763 / 180;
                m[6][11] = (58320 * t1508 + 421 * t1502 + 51840 * t1494 + 360 * t1500
                        + 1244160 * t1498 - 622080 * t1496 + 8640 * t1480 - 4320
                        * t1486) * t80 * t1396 / 4320;
                m[6][12] = t1032 * t1779 * t1784 / 2520;
                m[6][13] = t1032 * t1788 * t1795 / 1008;
                m[6][14] = t1032 * t1799 * t1804 / 9072;
                m[6][15] = -t1032 * t1699 * (2160 + 11 * t78) * t80 * t1396 / 120;
                m[6][16] = -t1032 * t1390 * t1818 / 180;
                m[6][17] = -t1032 * t1716 * t1825 / 84;
                m[6][18] = -t1032 * t1720 * t1832 / 336;
                m[6][19] = -t1032 * t1729 * t1839 / 432;
                m[6][20] = -0.2e1 / 0.5e1 * t1847;
                m[6][21] = -t1852 / 3;
                m[6][22] = -0.2e1 / 0.7e1 * t1854;
                m[6][23] = -t1856 / 4;
                m[6][24] = -0.2e1 / 0.9e1 * t259 * t1851;
                m[7][0] = t1032 * t1389 * t1860 / 45;
                m[7][1] = 0.77e2 / 0.4320e4 * t1866;
                m[7][2] = 0.2e1 / 0.135e3 * t1871;
                m[7][3] = 0.17e2 / 0.1344e4 * t1873;
                m[7][4] = 0.25e2 / 0.2268e4 * t1875;
                m[7][5] = t1032 * t246 * t1877 / 60;
                m[7][6] = t1883 / 45;
                m[7][7] = t1887 / 42;
                m[7][8] = t1889 / 42;
                m[7][9] = 0.5e1 / 0.216e3 * t1891;
                m[7][10] = -t10 * (-3240 * t1083 - 23 * t1086 + 8640 * Icb + 60
                        * t1094) * t80 / 360;
                m[7][11] = t1903 / 540;
                m[7][12] = (t1112 + 72576 * K + 504 * t1121) * t80 / 504;
                m[7][13] = t1912 / 504;
                m[7][14] = 0.5e1 / 0.2592e4 * t1914;
                m[7][15] = t1917 / 60;
                m[7][16] = t1921 / 72;
                m[7][17] = t1926 / 84;
                m[7][18] = t1928 / 96;
                m[7][19] = t1931 * t1925 / 108;
                m[7][20] = -t1935 / 5;
                m[7][21] = -t1939 / 6;
                m[7][22] = -t1943 / 7;
                m[7][23] = -t1945 / 8;
                m[7][24] = -t1127 * t1942 / 9;
                m[8][0] = -t1032 * t1389 * (t1949 + 6480) * t80 * t1396 / 540;
                m[8][1] = -t1032 * t1956 * t1712 / 3240;
                m[8][2] = -t1032 * t1960 * t1704 / 405;
                m[8][3] = -t1032 * t1964 * t1725 / 1008;
                m[8][4] = -t1032 * t1968 * t1734 / 6804;
                m[8][5] = -t1032 * t1459 * t1740 / 45;
                m[8][6] = -0.4e1 / 0.135e3 * t1976;
                m[8][7] = -0.2e1 / 0.63e2 * t1981;
                m[8][8] = -0.2e1 / 0.63e2 * t1986;
                m[8][9] = -0.5e1 / 0.162e3 * t1988;
                m[8][10] = -t1032 * t1262 * t1763 / 270;
                m[8][11] = -t10 * (32 * t1994 + 103680 * t8 * t13 * l * a + 720
                        * t8 * t204 * t35 * t20 + 58320 * t2004 + 405 * t2007 + 311040
                        * t1536 - 155520 * t1538 + 2160 * t1541 - 1080 * t1543)
                        * t1547 / 12960;
                m[8][12] = -t1032 * t1295 * t1784 / 3780;
                m[8][13] = -(9072 * t1494 + 67 * t1500 + 9072 * t1491 + 63 * t1511
                        - 435456 * t1498 + 217728 * t1496 - 3024 * t1480 + 1512
                        * t1486) * t80 * t1396 / 1512;
                m[8][14] = -t1032 * l * t25 * t1804 / 13608;
                m[8][15] = -t1032 * t89 * (8 * t49 - 2160 * t14 - 15 * t197) * t80
                        * t1396 / 360;
                m[8][16] = t1032 * t1956 * t1818 / 270;
                m[8][17] = t1032 * t1960 * t1825 / 126;
                m[8][18] = t1032 * t1964 * t1832 / 504;
                m[8][19] = t1032 * t1968 * t1839 / 648;
                m[8][20] = 0.4e1 / 0.15e2 * t1032 * t90 * t1845;
                m[8][21] = 0.2e1 / 0.9e1 * t1032 * t90 * t1979;
                m[8][22] = 0.4e1 / 0.21e2 * t1032 * t90 * t22 * t80 * t1396;
                m[8][23] = t1127 * t2070 / 6;
                m[8][24] = 0.4e1 / 0.27e2 * t259 * t2070;
                m[9][0] = 0;
                m[9][1] = t1387 / 20;
                m[9][2] = 0;
                m[9][3] = 0;
                m[9][4] = 0;
                m[9][5] = 0;
                m[9][6] = 0;
                m[9][7] = 0;
                m[9][8] = 0;
                m[9][9] = 0;
                m[9][10] = t1380;
                m[9][11] = 0;
                m[9][12] = 0.3e1 / 0.160e3 * t1382 - t1384;
                m[9][13] = 0;
                m[9][14] = K;
                m[9][15] = 0;
                m[9][16] = -t1388;
                m[9][17] = 0;
                m[9][18] = 0;
                m[9][19] = 0;
                m[9][20] = 0;
                m[9][21] = 0;
                m[9][22] = 0;
                m[9][23] = 0;
                m[9][24] = 0;
                m[10][0] = -t1032 * t246 * t2 * (t2079 + 142 * t49 + 173 * t197
                        + t1020) * t80 * t1396 / 360;
                m[10][1] = -t1177 * t518 * (49248 * t2 + 188 * t49 + t1781 + t1020)
                        * t80 * t1396 / 540;
                m[10][2] = -t1187 * t518 * (168480 * t2 + 658 * t49 + 647 * t197
                        + 19440 * t14) * t80 * t1396 / 2160;
                m[10][3] = -t1412 * t518 * (28512 * t2 + 113 * t49 + 106 * t197
                        + 3024 * t14) * t80 * t1396 / 420;
                m[10][4] = -t1151 * t518 * (272160 * t2 + 1090 * t49 + 989 * t197
                        + t1780) * t80 * t1396 / 4536;
                m[10][5] = 0.4e1 / 0.15e2 * t1458 * t2128;
                m[10][6] = 0.16e2 / 0.45e2 * t1465 * t2128;
                m[10][7] = 0.8e1 / 0.21e2 * t1469 * t2128;
                m[10][8] = 0.8e1 / 0.21e2 * t1474 * t2128;
                m[10][9] = 0.10e2 / 0.27e2 * t1696 * t2128;
                m[10][10] = -t1177 * t608 * (4320 * t14 + 31 * t197 + 432 * t2 + 2
                        * t49) * t80 * t1396 / 45;
                m[10][11] = -t1187 * t608 * (38880 * t14 + 286 * t197 + 6480 * t2
                        + 29 * t49) * t80 * t1396 / 540;
                m[10][12] = -t1648 * t446 * (18144 * t14 + 136 * t197 + 3888 * t2
                        + 17 * t49) * t80 * t1396 / 315;
                m[10][13] = -t1659 * t446 * (t1651 + 92 * t197 + 3024 * t2 + 13
                        * t49) * t80 * t1396 / 252;
                m[10][14] = -t1670 * t446 * (46656 * t14 + 359 * t197 + t2180 + 55
                        * t49) * t80 * t1396 / 1134;
                m[10][15] = t2202 * t80 * t1396 / 120;
                m[10][16] = t10 * (t2206 - t2207 + t2208 - t2209 + 8640 * t1518
                        + 20 * t1520 - 5184 * t1523 + 4 * t1527 + 2160 * t1531 + 15
                        * t1534) * t1547 / 180;
                m[10][17] = t10 * (14112 * t1551 + 34 * t1553 - 8064 * t1557 + 8
                        * t1561 + 3024 * t1564 + 21 * t1566 + t2226 - t2227 + t2228
                        - t2229) * t1547 / 336;
                m[10][18] = t10 * (15552 * t1578 + 38 * t1580 - 8640 * t1583 + 10
                        * t1587 + 3024 * t1590 + 21 * t1592 + t2226 - t2227 + t2228
                        - t2229) * t1547 / 420;
                m[10][19] = t10 * (14256 * t1604 + 35 * t1606 + 10 * t1614 - 7776
                        * t1609 + 2592 * t1617 + 18 * t1619 + 20736 * t1536 - 10368
                        * t1538 + 144 * t1541 - 72 * t1543) * t1547 / 432;
                m[10][20] = -0.16e2 / 0.5e1 * t217 * t2258;
                m[10][21] = -0.8e1 / 0.3e1 * t1127 * t2258;
                m[10][22] = -0.16e2 / 0.7e1 * t2263 * t160 * t1608 * t2127;
                m[10][23] = -2 * t1238 * t2258;
                m[10][24] = -0.16e2 / 0.9e1 * t253 * t2258;
                m[11][0] = 0;
                m[11][1] = 0;
                m[11][2] = 0;
                m[11][3] = 0;
                m[11][4] = 0;
                m[11][5] = 0;
                m[11][6] = 0;
                m[11][7] = 0;
                m[11][8] = 0;
                m[11][9] = 0;
                m[11][10] = 0;
                m[11][11] = 0;
                m[11][12] = 0;
                m[11][13] = 0;
                m[11][14] = 0;
                m[11][15] = 0;
                m[11][16] = K;
                m[11][17] = 0;
                m[11][18] = 0;
                m[11][19] = 0;
                m[11][20] = 0;
                m[11][21] = 0;
                m[11][22] = 0;
                m[11][23] = 0;
                m[11][24] = 0;
                m[12][0] = t1032 * t246 * (t2079 + 270 * t49 + 109 * t197 + t1020)
                        * t80 * t1396 / 1080;
                m[12][1] = t1032 * t1779 * t1712 / 1620;
                m[12][2] = 0.2e1 / 0.405e3 * t1032 * t1788 * t1704;
                m[12][3] = t1032 * t1799 * t1725 / 504;
                m[12][4] = t1032 * t2289 * t1734 / 3402;
                m[12][5] = 0.2e1 / 0.45e2 * t1032 * t1843 * t1740;
                m[12][6] = 0.8e1 / 0.135e3 * t1847;
                m[12][7] = 0.4e1 / 0.63e2 * t1852;
                m[12][8] = 0.4e1 / 0.63e2 * t1854;
                m[12][9] = 0.5e1 / 0.81e2 * t1856;
                m[12][10] = t1032 * t89 * B * t1763 / 135;
                m[12][11] = t1032 * t89 * t2 * (58320 * t14 + 421 * t197 + 51840
                        * t2 + 360 * t49) * t80 * t1396 / 3240;
                m[12][12] = t1032 * t1956 * t1784 / 1890;
                m[12][13] = t1032 * t1960 * t1795 / 756;
                m[12][14] = t1032 * t1964 * t1804 / 6804;
                m[12][15] = -t10 * (4320 * t8 * t2325 + 14 * t1994 + t2206 - t2207
                        + t2208 - t2209 + 2160 * t2004 + 15 * t2007) * t1547 / 360;
                m[12][16] = -t1032 * t1779 * t1818 / 135;
                m[12][17] = -(-18144 * t1498 + 9072 * t1496 - 126 * t1480 + 63
                        * t1486 + 1008 * t1494 + 5 * t1500) * t80 * t1396 / 63;
                m[12][18] = -t1032 * t1799 * t1832 / 252;
                m[12][19] = -t1032 * t2289 * t1839 / 324;
                m[12][20] = -0.8e1 / 0.15e2 * t1976;
                m[12][21] = -0.4e1 / 0.9e1 * t1981;
                m[12][22] = -0.8e1 / 0.21e2 * t1986;
                m[12][23] = -t1988 / 3;
                m[12][24] = -0.8e1 / 0.27e2 * t259 * t1985;
                m[13][0] = -0.2e1 / 0.45e2 * t1032 * t1612 * t1576 * t80;
                m[13][1] = -t203 * t204 * t14 * (3240 + t1949) * t80 / 1080;
                m[13][2] = -0.4e1 / 0.135e3 * t1903;
                m[13][3] = -0.17e2 / 0.672e3 * t2371;
                m[13][4] = -0.25e2 / 0.1134e4 * t1912;
                m[13][5] = -t2376;
                m[13][6] = -t2377;
                m[13][7] = -t2378;
                m[13][8] = -t2379;
                m[13][9] = -t2380;
                m[13][10] = -t2386;
                m[13][11] = -t2387;
                m[13][12] = -t2388;
                m[13][13] = -t2389;
                m[13][14] = -t2390;
                m[13][15] = -t2393;
                m[13][16] = -t2404;
                m[13][17] = -t2408;
                m[13][18] = -t2414;
                m[13][19] = -t2419;
                m[13][20] = t2422;
                m[13][21] = t2425;
                m[13][22] = t2429;
                m[13][23] = t2434;
                m[13][24] = t2438;
                m[14][0] = 0;
                m[14][1] = 0;
                m[14][2] = -t2439;
                m[14][3] = 0;
                m[14][4] = 0;
                m[14][5] = 0;
                m[14][6] = 0;
                m[14][7] = 0;
                m[14][8] = 0;
                m[14][9] = 0;
                m[14][10] = 0;
                m[14][11] = -t1388;
                m[14][12] = 0;
                m[14][13] = 0;
                m[14][14] = 0;
                m[14][15] = t1380;
                m[14][16] = 0;
                m[14][17] = t2440;
                m[14][18] = 0;
                m[14][19] = K;
                m[14][20] = 0;
                m[14][21] = 0;
                m[14][22] = 0;
                m[14][23] = 0;
                m[14][24] = 0;
                m[15][0] = t445 * t446 * t14 * t2456 * t81 / 2160;
                m[15][1] = t111 * Gamma * t444 * t446 * t14 * t2476 * t81 / 103680;
                m[15][2] = t479 * t1330 * t1 * t2494 * t81 / 45360;
                m[15][3] = t1448 * t251 * l * a * t1 * Gamma * t14 * t2516 * t81
                        / 96768;
                m[15][4] = t161 * t1522 * t14 * t2531 * t81 / 54432;
                m[15][5] = t134 * t10 * t8 * t608 * (13824 * FastMath.PI + 64 * t962
                        + t2543 - t2546 + t2548) * t77 * t80 / 1440;
                m[15][6] = t534 * t536 * t1968 * (12960 * FastMath.PI + 58 * t962
                        + t2543 - t2546 + t2548) * t77 * t80 / 1080;
                m[15][7] = t2566 * t608 * (62208 * FastMath.PI + 272 * t962 + 30
                        * t2542 - 45 * t2545 + 40 * t2547) * t77 * t80 / 5040;
                m[15][8] = t22 * t1 * t10 * t8 * t608 * (12096 * FastMath.PI + 52
                        * t962 + t2543 - t2546 + t2548) * t77 * t80 / 1008;
                m[15][9] = 0.5e1 / 0.36288e5 * t66 * t1 * t10 * t8 * t608 * (82944
                        * FastMath.PI + 352 * t962 + 42 * t2542 - 63 * t2545 + 56
                        * t2547) * t77 * t80;
                m[15][10] = -t2605 * t90 * t1 * (-t2606 + 51840 * t452 - 546 * t461
                        + t2609 - 77760 * t2449 + 360 * t456 + 69120 * t50 + 480 * t449
                        - t2614) * t81 / 17280;
                m[15][11] = -t136 * t12 * Gamma * (-t2606 + 31104 * t452 - 330 * t461
                        + t2609 - 46656 * t2449 + 216 * t456 + t519 + 288 * t449
                        - t2614) * t81 / 12960;
                m[15][12] = -t664 * t8 * t12 * Gamma * (-t2606 + 24192 * t452 - 258
                        * t461 + t2609 - 36288 * t2449 + 168 * t456 + 32256 * t50
                        + 224 * t449 - t2614) * t81 / 12096;
                m[15][13] = -t2566 * t608 * Gamma * (-t2606 + 20736 * t452 - 222
                        * t461 + t2609 - 31104 * t2449 + 144 * t456 + 27648 * t50
                        + 192 * t449 - t2614) * t81 / 12096;
                m[15][14] = -t1465 * t90 * t1 * (-23040 * t468 + 93312 * t452 - 1002
                        * t461 + 45 * t2446 - 139968 * t2449 + 648 * t456 + 124416
                        * t50 + 864 * t449 - 40 * t458) * t81 / 62208;
                m[15][15] = t88 * t2672 * (t601 + t2673 - t2674 + 540 * t46 - 810
                        * t2677 + 720 * t19 + t2680) * t77 * t80 / 540;
                m[15][16] = t2605 * t2672 * (9216 * t53 + 30 * t59 - 45 * t69 + 2592
                        * t46 - 3888 * t2677 + 3456 * t19 + 40 * t23) * t77 * t80
                        / 3456;
                m[15][17] = t2263 * t6 * t251 * t25 * t204 * t1 * (1920 * t53 + t2673
                        - t2674 + 504 * t46 - 756 * t2677 + 672 * t19 + t2680) * t77
                        * t80 / 840;
                m[15][18] = t8 * t1150 * a * t2672 * (13824 * t53 + 42 * t59 - 63
                        * t69 + 3456 * t46 - 5184 * t2677 + 4608 * t19 + t24) * t77
                        * t80 / 6912;
                m[15][19] = t1458 * t2672 * (2016 * t53 + t2673 - t2674 + 486 * t46
                        - 729 * t2677 + 648 * t19 + t2680) * t77 * t80 / 1134;
                m[15][20] = t1 * t2753 * t81 / 120;
                m[15][21] = t269 * t2768 * t77 * t80 / 720;
                m[15][22] = t269 * t2785 * t77 * t80 / 336;
                m[15][23] = t269 * t2802 * t77 * t80 / 6720;
                m[15][24] = t269 * t2819 * t77 * t80 / 432;
                m[16][0] = -t1032 * t1779 * (864 * t2 + 6 * t49 - 2 * t1024 + 720
                        * t14 + t1189) * Gamma * t80 / 480;
                m[16][1] = -t1032 * t1788 * (34560 * t2 + 240 * t49 - 77 * t1024
                        + 25920 * t14 + 180 * t197) * Gamma * t80 / 23040;
                m[16][2] = -t1032 * t1799 * (-28 * t1024 + t2180 + 90 * t49 + t1789
                        + 63 * t197) * Gamma * t80 / 10080;
                m[16][3] = -t1032 * t2289 * (24192 * t2 + 168 * t49 - t1428 + 16128
                        * t14 + 112 * t197) * Gamma * t80 / 21504;
                m[16][4] = -t1032 * t2864 * (12096 * t2 + 84 * t49 - 25 * t1024 + 7776
                        * t14 + 54 * t197) * Gamma * t80 / 12096;
                m[16][5] = t1032 * l * t50 * t292 * t80 / 320;
                m[16][6] = t1101 * t2882 / 240;
                m[16][7] = t217 * t2882 / 224;
                m[16][8] = t1127 * t2882 / 224;
                m[16][9] = 0.5e1 / 0.1152e4 * t259 * t2882;
                m[16][10] = -t1032 * t1716 * (-t1135 + 8640 + t1136) * Gamma * t80
                        / 3840;
                m[16][11] = -t1032 * t1720 * (-t1135 + 5184 + t1143) * Gamma * t80
                        / 2880;
                m[16][12] = -t1032 * t1729 * (-t1135 + 4032 + t1152) * Gamma * t80
                        / 2688;
                m[16][13] = -t1032 * t1389 * t40 * (-t1135 + 3456 + t1159) * Gamma
                        * t80 / 2688;
                m[16][14] = -t1032 * t1389 * t22 * (-t1168 + 15552 + t1169) * Gamma
                        * t80 / 13824;
                m[16][15] = t1032 * t1088 * t39 * (t78 + 90) * Gamma * t80 / 120;
                m[16][16] = t1032 * t1088 * t13 * (t1822 + 432) * Gamma * t80 / 768;
                m[16][17] = 0.3e1 / 0.560e3 * t1032 * t1088 * t25 * (t78 + 84)
                        * Gamma * t80;
                m[16][18] = t1032 * t1088 * t32 * (t2945 + 576) * Gamma * t80 / 1536;
                m[16][19] = t1032 * t1088 * t40 * (t78 + 81) * Gamma * t80 / 252;
                m[16][20] = -0.3e1 / 0.80e2 * t2960;
                m[16][21] = (-t1117 + 4608 * K + 32 * t1121) * t80 / 32;
                m[16][22] = -0.3e1 / 0.112e3 * t2969;
                m[16][23] = -0.3e1 / 0.128e3 * t2971;
                m[16][24] = -t2973 / 48;
                m[17][0] = t479 * t2975 * t706 / 1620;
                m[17][1] = t479 * t1319 * t723 / 38880;
                m[17][2] = t479 * t1323 * t740 / 34020;
                m[17][3] = t479 * t2985 * t757 / 36288;
                m[17][4] = t479 * t828 * t775 / 20412;
                m[17][5] = -t479 * mu * B * t1 * t787 / 540;
                m[17][6] = -t479 * t4 * (51840 * t223 + 360 * t191 + 19440 * t63
                        + 151 * t623 + 8 * t625 + 24 * t627) * t77 * t80 / 3240;
                m[17][7] = -t479 * t1367 * t810 / 1890;
                m[17][8] = -t479 * t112 * t824 / 756;
                m[17][9] = -t479 * t135 * t837 / 13608;
                m[17][10] = t479 * t710 * t857 / 6480;
                m[17][11] = t479 * t727 * t868 / 4860;
                m[17][12] = t479 * t692 * t13 * t879 / 4536;
                m[17][13] = t479 * t949 * t890 / 4536;
                m[17][14] = t479 * t1006 * t904 / 23328;
                m[17][15] = -0.2e1 / 0.405e3 * t479 * t2975 * t914;
                m[17][16] = -t479 * t1319 * t925 / 1296;
                m[17][17] = -t479 * t1323 * t934 / 315;
                m[17][18] = -t479 * t2985 * t945 / 2592;
                m[17][19] = -0.4e1 / 0.1701e4 * t479 * t1330 * t955;
                m[17][20] = -t269 * (4320 * t291 * t246 * t223 + 14 * t1268 - 8
                        * t1284 - 39 * t1273 - 2160 * t1270 + 17280 * t313 + 120
                        * t309 + 8640 * t315 + 60 * t311) * t77 * t80 / 360;
                m[17][21] = -t479 * t801 * t982 / 270;
                m[17][22] = -t1 * (1008 * t249 + 5 * t256 - t232 - 3 * t263
                        - 18144 * t185 - 126 * t194 - 9072 * t187 - 63 * t200) * t81
                        / 63;
                m[17][23] = -t479 * t828 * t1002 / 504;
                m[17][24] = -t479 * t814 * t32 * t1013 / 324;
                m[18][0] = -t1032 * t1779 * Gamma * (t2945 + 2160) * t80 / 2160;
                m[18][1] = -0.7e1 / 0.12960e5 * t1032 * t1788 * Gamma * (1512 + t1822)
                        * t80;
                m[18][2] = -t1032 * t1799 * Gamma * (t1822 + 1488) * t80 / 2160;
                m[18][3] = -t1032 * t2289 * Gamma * (14400 + t1391) * t80 / 24192;
                m[18][4] = -t1032 * t2864 * Gamma * (t1391 + 14256) * t80 / 27216;
                m[18][5] = t2386;
                m[18][6] = t2387;
                m[18][7] = t2388;
                m[18][8] = t2389;
                m[18][9] = t2390;
                m[18][10] = -t2376;
                m[18][11] = -t2377;
                m[18][12] = -t2378;
                m[18][13] = -t2379;
                m[18][14] = -t2380;
                m[18][15] = -t2422;
                m[18][16] = -t2425;
                m[18][17] = -t2429;
                m[18][18] = -t2434;
                m[18][19] = -t2438;
                m[18][20] = -t2393;
                m[18][21] = -t2404;
                m[18][22] = -t2408;
                m[18][23] = -t2414;
                m[18][24] = -t2419;
                m[19][0] = 0;
                m[19][1] = 0;
                m[19][2] = 0;
                m[19][3] = 0;
                m[19][4] = 0;
                m[19][5] = 0;
                m[19][6] = t1388;
                m[19][7] = 0;
                m[19][8] = 0;
                m[19][9] = 0;
                m[19][10] = 0;
                m[19][11] = 0;
                m[19][12] = 0;
                m[19][13] = 0;
                m[19][14] = 0;
                m[19][15] = 0;
                m[19][16] = 0;
                m[19][17] = 0;
                m[19][18] = 0;
                m[19][19] = 0;
                m[19][20] = t1380;
                m[19][21] = 0;
                m[19][22] = t2440;
                m[19][23] = 0;
                m[19][24] = K;
                m[20][0] = -(15 * t2411 + 15 * t1112 - 120 * t1121 - 8 * t1117
                        + 2160 * t8 * t2603 * t204 - 17280 * K + 2160 * t203 * t2325
                        - 8640 * t189 - 60 * t1093 * t10 * t20) * t80 / 120;
                m[20][1] = -t10 * (144 * t61 * rho * t6 * c * l * t1114 + 120
                        * t305 * t1085 - 77 * t297 * t1533 + 20736 * t8 * t210 * l
                        + 17280 * t8 * t204 * t451 - 69120 * Icb - 480 * t1094)
                        * t80 / 1440;
                m[20][2] = -t10 * (60 * t8 * t239 * a * t1612 + 45 * t364 * t1085
                        - 32 * t332 * t1533 + 8640 * t8 * t163 + 6480 * t8 * t1493
                        - 25920 * Icb - 180 * t1094) * t80 / 720;
                m[20][3] = -t10 * (160 * t1527 + 112 * t1520 - 85 * t1534 + 23040
                        * t1523 - 64512 * Icb + 16128 * t1518 - 448 * t1094) * t80
                        / 2240;
                m[20][4] = -t10 * (189 * t1561 + 126 * t1553 - 100 * t1566 + 27216
                        * t1557 + 18144 * t1551 - 72576 * Icb - 504 * t1094) * t80
                        / 3024;
                m[20][5] = t3196 / 20;
                m[20][6] = t3198 / 15;
                m[20][7] = t3200 / 14;
                m[20][8] = t3202 / 14;
                m[20][9] = 0.5e1 / 0.72e2 * t1127 * t3195;
                m[20][10] = -t1177 * t204 * mu * (40 * t78 - t1135 + 5760) * t80
                        / 240;
                m[20][11] = -t1187 * t204 * mu * (t1700 - 2 * t1135 + 6480) * t80
                        / 360;
                m[20][12] = -t1412 * t204 * mu * (84 * t78 - t1168 + 12096) * t80
                        / 840;
                m[20][13] = -t1151 * t204 * mu * (14 * t78 - t1135 + 2016) * t80
                        / 168;
                m[20][14] = -t1101 * t204 * mu * (t1708 - 35 * t1135 + 62208) * t80
                        / 6048;
                m[20][15] = 0.9e1 / 0.80e2 * t7 * t251 * t2 * t204 * t14 * (t78 + 80)
                        * t80;
                m[20][16] = t1177 * t204 * t14 * (t78 + 72) * t80 / 12;
                m[20][17] = 0.3e1 / 0.224e3 * t1187 * t204 * t14 * (t1822 + 336)
                        * t80;
                m[20][18] = 0.9e1 / 0.160e3 * t1412 * t204 * t14 * (t78 + 64) * t80;
                m[20][19] = t1151 * t204 * t14 * (t2945 + 432) * t80 / 144;
                m[20][20] = -0.3e1 / 0.5e1 * t1887;
                m[20][21] = -t1889 / 2;
                m[20][22] = -0.3e1 / 0.7e1 * t1891;
                m[20][23] = -0.3e1 / 0.8e1 * t3276;
                m[20][24] = -t259 * t1886 / 3;
                m[21][0] = -t2960 / 30;
                m[21][1] = (414720 * K + 2880 * t1121 - 77 * t1117) * t80 / 2880;
                m[21][2] = -t2969 / 45;
                m[21][3] = -0.17e2 / 0.896e3 * t2971;
                m[21][4] = -0.25e2 / 0.1512e4 * t2973;
                m[21][5] = -t1187 * t3195 / 40;
                m[21][6] = -t3196 / 30;
                m[21][7] = -t3198 / 28;
                m[21][8] = -t3200 / 28;
                m[21][9] = -0.5e1 / 0.144e3 * t3202;
                m[21][10] = -t1866 / 480;
                m[21][11] = -t1871 / 360;
                m[21][12] = -t1873 / 336;
                m[21][13] = -t1875 / 336;
                m[21][14] = -0.5e1 / 0.1728e4 * t253 * t1870;
                m[21][15] = -t1903 / 40;
                m[21][16] = -t2371 / 48;
                m[21][17] = -t1912 / 56;
                m[21][18] = -t1914 / 64;
                m[21][19] = -t1931 * t1911 / 72;
                m[21][20] = 0.3e1 / 0.10e2 * t1883;
                m[21][21] = t1887 / 4;
                m[21][22] = 0.3e1 / 0.14e2 * t1889;
                m[21][23] = 0.3e1 / 0.16e2 * t1891;
                m[21][24] = t3276 / 6;
                m[22][0] = -t3315 / 24;
                m[22][1] = 0;
                m[22][2] = K;
                m[22][3] = 0;
                m[22][4] = 0;
                m[22][5] = 0;
                m[22][6] = 0;
                m[22][7] = 0;
                m[22][8] = 0;
                m[22][9] = 0;
                m[22][10] = 0;
                m[22][11] = 0;
                m[22][12] = 0;
                m[22][13] = 0;
                m[22][14] = 0;
                m[22][15] = -t1382 / 48;
                m[22][16] = 0;
                m[22][17] = 0;
                m[22][18] = 0;
                m[22][19] = 0;
                m[22][20] = 0;
                m[22][21] = 0;
                m[22][22] = 0;
                m[22][23] = 0;
                m[22][24] = 0;
                m[23][0] = 0;
                m[23][1] = -t3315 / 48;
                m[23][2] = 0;
                m[23][3] = K;
                m[23][4] = 0;
                m[23][5] = 0;
                m[23][6] = 0;
                m[23][7] = 0;
                m[23][8] = 0;
                m[23][9] = 0;
                m[23][10] = t1387 / 24;
                m[23][11] = 0;
                m[23][12] = 0;
                m[23][13] = 0;
                m[23][14] = 0;
                m[23][15] = 0;
                m[23][16] = -t1382 / 96;
                m[23][17] = 0;
                m[23][18] = 0;
                m[23][19] = 0;
                m[23][20] = 0;
                m[23][21] = 0;
                m[23][22] = 0;
                m[23][23] = 0;
                m[23][24] = 0;
                m[24][0] = t1380;
                m[24][1] = 0;
                m[24][2] = -t3315 / 80;
                m[24][3] = 0;
                m[24][4] = K;
                m[24][5] = 0;
                m[24][6] = 0;
                m[24][7] = 0;
                m[24][8] = 0;
                m[24][9] = 0;
                m[24][10] = 0;
                m[24][11] = t1388;
                m[24][12] = 0;
                m[24][13] = 0;
                m[24][14] = 0;
                m[24][15] = 0;
                m[24][16] = 0;
                m[24][17] = -t1383;
                m[24][18] = 0;
                m[24][19] = 0;
                m[24][20] = 0;
                m[24][21] = 0;
                m[24][22] = 0;
                m[24][23] = 0;
                m[24][24] = 0;
            }
            {
                final double t1 = omega * omega;
                final double t2 = t1 * rho;
                final double t3 = R * R;
                final double t4 = t3 * R;
                final double t5 = t2 * t4;
                final double t6 = c * a;
                final double t8 = t5 * t6 * l;
                final double t9 = B * B;
                final double t10 = t9 * t9;
                final double t11 = Gamma * Gamma;
                final double t12 = t10 * t11;
                final double t13 = mu * mu;
                final double t14 = t13 * t13;
                final double t15 = t14 * t13;
                final double t17 = t15 * theta0 * FastMath.PI;
                final double t20 = t10 * t9;
                final double t21 = t20 * t11;
                final double t22 = t14 * theta0;
                final double t23 = FastMath.PI * t22;
                final double t26 = t14 * t14;
                final double t27 = t26 * t13;
                final double t28 = t11 * t11;
                final double t29 = t27 * t28;
                final double t30 = t10 * t10;
                final double t31 = t30 * FastMath.PI;
                final double t32 = t31 * theta0;
                final double t35 = t9 * B;
                final double t36 = t10 * t35;
                final double t37 = t36 * t11;
                final double t38 = t13 * mu;
                final double t40 = FastMath.PI * FastMath.PI;
                final double t44 = t30 * t36;
                final double t46 = t28 * t38;
                final double t50 = t14 * t38;
                final double t51 = t50 * t11;
                final double t53 = t35 * t40 * theta0;
                final double t56 = t30 * B;
                final double t57 = t56 * t11;
                final double t59 = theta0 * t40 * mu;
                final double t62 = t10 * B;
                final double t63 = t62 * t11;
                final double t64 = t14 * mu;
                final double t66 = t64 * theta0 * t40;
                final double t69 = t30 * t20;
                final double t70 = t69 * t28;
                final double t73 = t30 * t30;
                final double t78 = t30 * t10;
                final double t79 = t78 * t28;
                final double t82 = t30 * t9;
                final double t83 = t82 * t28;
                final double t84 = t26 * FastMath.PI;
                final double t85 = t84 * theta0;
                final double t88 = t30 * t11;
                final double t89 = t38 * thetaTW;
                final double t93 = t50 * t28;
                final double t94 = t78 * t40;
                final double t95 = t94 * thetaTW;
                final double t98 = t30 * t62;
                final double t99 = t98 * t28;
                final double t101 = t15 * thetaTW * FastMath.PI;
                final double t104 = t44 * t28;
                final double t105 = thetaTW * FastMath.PI;
                final double t106 = t105 * t14;
                final double t112 = thetaTW * t40 * mu;
                final double t118 = -725760 * t12 * t17 - 414720 * t21 * t23 + 1020
                        * t29 * t32 - 11520 * t37 * t38 * theta0 * t40 - 330 * t44
                        * t40 * t46 * theta0 + 29160 * t51 * t53 + 155520 * t57 * t59
                        - 352080 * t63 * t66 - 2880 * t70 * t23 + 1080 * t73 * B
                        * t28 * t59 - 5040 * t79 * t17 - 800 * t83 * t85 - 31104 * t88
                        * t89 * t40 + 432 * t93 * t95 - 3648 * t99 * t101 - 2304
                        * t104 * t106 + 864 * t73 * t9 * t28 * t112 + 124416 * t82
                        * t11 * t112;
                final double t121 = t64 * t40;
                final double t125 = t26 * t11;
                final double t127 = t35 * thetaTW * FastMath.PI;
                final double t131 = t40 * t11;
                final double t132 = t131 * thetaTW;
                final double t135 = t56 * FastMath.PI;
                final double t136 = t135 * thetaTW;
                final double t139 = t30 * t35;
                final double t140 = t139 * t28;
                final double t152 = t9 * t11;
                final double t156 = t139 * t40 * theta0;
                final double t159 = t82 * t40;
                final double t160 = t159 * lambda;
                final double t163 = t14 * t40;
                final double t164 = t163 * p;
                final double t167 = t11 * Gamma;
                final double t169 = t50 * FastMath.PI;
                final double t177 = t73 * t28;
                final double t179 = lambda * t40 * mu;
                final double t182 = t26 * mu;
                final double t185 = t62 * FastMath.PI * q;
                final double t189 = t36 * FastMath.PI * lambda;
                final double t192 = -331776 * t37 * t106 - 1620 * t70 * t121 * thetaTW
                        - 207360 * t125 * t127 - 233280 * t20 * t64 * t132 + 576
                        * t29 * t136 - 672 * t140 * t84 * thetaTW - 408 * t73 * t40
                        * t46 * thetaTW - 525312 * t63 * t101 - 2445 * t99 * t66 - 259200
                        * t152 * t85 + 765 * t93 * t156 + 450 * t93 * t160 - 1260
                        * t70 * t164 - 9600 * t36 * t167 * t169 * q + 800 * t56 * t28
                        * t84 * lambda + 1440 * t177 * t179 - 7200 * t182 * t167
                        * t185 + 600 * t29 * t189;
                final double t196 = t30 * t40 * q;
                final double t199 = t10 * t64;
                final double t203 = t14 * lambda;
                final double t204 = t203 * FastMath.PI;
                final double t208 = t64 * p * FastMath.PI;
                final double t213 = t169 * p;
                final double t216 = t15 * t28;
                final double t217 = t139 * FastMath.PI;
                final double t218 = t217 * lambda;
                final double t221 = t15 * t11;
                final double t223 = t40 * t10 * p;
                final double t228 = t94 * p;
                final double t232 = t35 * lambda * FastMath.PI;
                final double t238 = t82 * t167;
                final double t246 = t40 * Gamma;
                final double t250 = t78 * q;
                final double t251 = t40 * t167;
                final double t255 = t10 * q;
                final double t259 = t182 * t11;
                final double t261 = B * FastMath.PI * p;
                final double t264 = -5400 * t15 * t167 * t196 - 103680 * t199 * t131
                        * lambda - 397440 * t204 * t63 - 341280 * t63 * t208 - 181440
                        * t21 * t164 - 285 * t140 * t213 - 4500 * t216 * t218
                        + 103680 * t221 * t223 + 207360 * t88 * t179 + 270 * t216
                        * t228 - 648000 * t221 * t232 + 74160 * t35 * t11 * t213
                        + 5760 * t238 * t163 * q - 720 * t79 * t121 * lambda + 829440
                        * t9 * t14 * t246 * q + 11520 * t250 * t251 * t13 + 1313280
                        * t255 * t246 * t13 + 138240 * t259 * t261;
                final double t265 = t182 * t28;
                final double t266 = t135 * p;
                final double t274 = t13 * t40 * p;
                final double t280 = t11 * t38;
                final double t291 = t27 * t11;
                final double t297 = t50 * lambda;
                final double t300 = t26 * p;
                final double t310 = theta0 * FastMath.PI;
                final double t313 = p * t30;
                final double t316 = lambda * t30;
                final double t319 = 360 * t265 * t266 - 1880 * t69 * t40 * t46
                        * lambda + 720 * t177 * t274 + 696960 * t88 * t274 - 241920
                        * t40 * t20 * t280 * lambda - 2370 * t99 * t208 - 2760
                        * t99 * t204 + 119439360 * t9 * t40 * p + 129600 * t291 * p
                        + 89579520 * t274 - 311040 * t259 * lambda - 414720 * t152
                        * t297 + 1200 * t83 * t300 + 172800 * t152 * t300 - 2880
                        * t83 * t297 + 829440 * t159 * p * t11 + 38880 * t291 * t310
                        + 900 * t29 * t313 - 2160 * t265 * t316;
                final double t322 = 0.1e1 / B;
                final double t324 = 0.1e1 / t40;
                final double t326 = 2 * t9;
                final double t328 = 0.1e1 / (t326 + t13);
                final double t330 = 0.1e1 / (144 + t88);
                final double t332 = 0.1e1 / Gamma;
                final double t333 = t328 * t330 * t332;
                final double t337 = t104 * t59;
                final double t339 = t79 * t23;
                final double t343 = t280 * t20 * thetaTW * t40;
                final double t345 = t12 * t23;
                final double t347 = t221 * t127;
                final double t349 = t177 * t112;
                final double t351 = t63 * t106;
                final double t353 = t28 * t64;
                final double t354 = t353 * t95;
                final double t356 = t99 * t106;
                final double t358 = t40 * t28;
                final double t361 = t358 * t38 * t69 * thetaTW;
                final double t363 = t51 * t261;
                final double t365 = t93 * t266;
                final double t367 = 540 * t337 + 180 * t339 + 77760 * t343 + 25920
                        * t345 + 17280 * t347 + 432 * t349 + 20736 * t351 - 144
                        * t354 + 144 * t356 + 444 * t361 - 11520 * t363 - 30 * t365;
                final double t369 = t167 * t50 * t185;
                final double t370 = 600 * t369;
                final double t371 = t28 * t26;
                final double t372 = t371 * t189;
                final double t373 = 50 * t372;
                final double t374 = t11 * t14;
                final double t375 = t374 * t223;
                final double t376 = 34560 * t375;
                final double t377 = t28 * t14;
                final double t378 = t377 * t228;
                final double t379 = 90 * t378;
                final double t380 = t37 * t59;
                final double t381 = 77760 * t380;
                final double t382 = t88 * t112;
                final double t383 = 62208 * t382;
                final double t386 = t216 * t139 * thetaTW * FastMath.PI;
                final double t387 = 120 * t386;
                final double t388 = t371 * t136;
                final double t389 = 48 * t388;
                final double t390 = t377 * t218;
                final double t391 = 780 * t390;
                final double t394 = t251 * t13 * t82 * q;
                final double t398 = t131 * t38 * t10 * lambda;
                final double t399 = 34560 * t398;
                final double t402 = t353 * p * t139 * FastMath.PI;
                final double t403 = 105 * t402;
                final double t404 = t82 * theta0;
                final double t406 = t216 * t404 * FastMath.PI;
                final double t407 = 180 * t406;
                final double t408 = t370 - t373 - t376 - t379 + t381 + t383 + t387
                        - t389 + t391 - 720 * t394 + t399 - t403 + t407;
                final double t410 = t371 * t32;
                final double t411 = 85 * t410;
                final double t414 = t358 * t38 * t98 * theta0;
                final double t415 = 645 * t414;
                final double t416 = t353 * t156;
                final double t417 = 255 * t416;
                final double t418 = t11 * t13;
                final double t421 = t418 * t20 * p * t40;
                final double t422 = 37440 * t421;
                final double t423 = t21 * t179;
                final double t424 = 103680 * t423;
                final double t425 = t9 * theta0;
                final double t427 = t221 * t425 * FastMath.PI;
                final double t428 = 25920 * t427;
                final double t429 = t11 * t64;
                final double t432 = t429 * p * t35 * FastMath.PI;
                final double t433 = 15120 * t432;
                final double t434 = t374 * t232;
                final double t435 = 112320 * t434;
                final double t438 = t358 * t38 * t78 * lambda;
                final double t439 = 140 * t438;
                final double t440 = t167 * t14;
                final double t441 = t440 * t196;
                final double t442 = 1800 * t441;
                final double t443 = t353 * t160;
                final double t444 = 150 * t443;
                final double t447 = t246 * t13 * t9 * q;
                final double t449 = -t411 + t415 - t417 + t422 + t424 + t428 - t433
                        + t435 + t439 + t442 - t444 - 276480 * t447;
                final double t450 = t70 * t179;
                final double t451 = 720 * t450;
                final double t455 = t28 * t13 * t69 * p * t40;
                final double t456 = 360 * t455;
                final double t457 = t429 * t53;
                final double t458 = 9720 * t457;
                final double t461 = t280 * t62 * theta0 * t40;
                final double t462 = 110880 * t461;
                final double t463 = t51 * lambda;
                final double t464 = 25920 * t463;
                final double t465 = t40 * p;
                final double t466 = 7464960 * t465;
                final double t467 = t125 * p;
                final double t468 = 10800 * t467;
                final double t469 = t125 * t310;
                final double t470 = 3240 * t469;
                final double t471 = t251 * t250;
                final double t473 = t465 * t88;
                final double t474 = 51840 * t473;
                final double t475 = t246 * t255;
                final double t477 = t93 * t316;
                final double t478 = 180 * t477;
                final double t479 = t371 * t313;
                final double t480 = 75 * t479;
                final double t481 = t451 + t456 - t458 + t462 + t464 - t466 - t468
                        - t470 - 3240 * t471 - t474 - 466560 * t475 + t478 - t480;
                final double t485 = 0.1e1 / t9;
                final double t489 = t324 * t328 * t330 * t332;
                final double t493 = t4 * c;
                final double t495 = t2 * t493 * a;
                final double t496 = l * mu;
                final double t498 = t20 * FastMath.PI * q;
                final double t499 = t280 * t498;
                final double t501 = t38 * t167;
                final double t502 = t82 * FastMath.PI;
                final double t503 = t502 * p;
                final double t504 = t501 * t503;
                final double t506 = t13 * FastMath.PI;
                final double t510 = t38 * Gamma;
                final double t511 = t9 * FastMath.PI;
                final double t512 = t511 * p;
                final double t513 = t510 * t512;
                final double t515 = t9 * Gamma;
                final double t519 = mu * FastMath.PI;
                final double t520 = t88 * q;
                final double t521 = t519 * t520;
                final double t523 = Gamma * t10;
                final double t524 = t523 * p;
                final double t525 = t519 * t524;
                final double t527 = mu * t167;
                final double t529 = t78 * p * FastMath.PI;
                final double t530 = t527 * t529;
                final double t532 = t31 * lambda;
                final double t533 = t440 * t532;
                final double t535 = t502 * thetaTW;
                final double t536 = t440 * t535;
                final double t538 = t78 * t167;
                final double t540 = t506 * t538 * thetaTW;
                final double t543 = t506 * t523 * thetaTW;
                final double t545 = t14 * Gamma;
                final double t546 = B * theta0;
                final double t550 = 1800 * t499 - 90 * t504 + 140 * t506 * t238
                        * lambda - 34560 * t513 + 34560 * t506 * t515 * lambda - 720
                        * t521 + 37440 * t525 + 360 * t530 - 150 * t533 - 144 * t536
                        + 444 * t540 + 77760 * t543 - 9720 * t545 * t546 * FastMath.PI;
                final double t551 = t135 * theta0;
                final double t552 = t440 * t551;
                final double t554 = t139 * t167;
                final double t556 = t506 * t554 * theta0;
                final double t558 = t35 * Gamma;
                final double t560 = t506 * t558 * theta0;
                final double t562 = t519 * q;
                final double t567 = Gamma * t62;
                final double t568 = t567 * t310;
                final double t570 = t167 * t98;
                final double t571 = t570 * t310;
                final double t588 = lambda * FastMath.PI;
                final double t589 = t523 * t588;
                final double t591 = t538 * t588;
                final double t593 = -255 * t552 + 645 * t556 + 110880 * t560 - 276480
                        * t562 + 432 * t167 * t69 * t105 + 77760 * t568 + 540 * t571
                        - 225 * t440 * t56 * p + 77760 * t510 * B * lambda + 62208
                        * Gamma * t20 * t105 + 540 * t501 * t56 * lambda - 32400
                        * t545 * B * p + 103680 * t589 + 720 * t591;
                final double t596 = 0.1e1 / FastMath.PI;
                final double t603 = t370 - t373 - t376 - t379 + t381 + t383 + t387
                        - t389 + t391 + 900 * t394 + t399 - t403;
                final double t605 = t407 - t411 + t415 - t417 + t422 + t424 + t428
                        - t433 + t435 + t439 + t442 - t444;
                final double t607 = -43200 * t447 + t451 + t456 - t458 + t462 + t464
                        - t466 - t468 - t470 - t474 + t478 - t480;
                final double t615 = a * l;
                final double t616 = t615 * t38;
                final double t619 = FastMath.PI * t35 * Gamma * theta0;
                final double t622 = t167 * theta0;
                final double t623 = t217 * t622;
                final double t626 = t615 * mu;
                final double t629 = t615 * t64;
                final double t630 = t238 * t105;
                final double t635 = FastMath.PI * t78 * t167 * thetaTW;
                final double t640 = t615 * t14;
                final double t641 = FastMath.PI * p;
                final double t642 = t515 * t641;
                final double t648 = t167 * t56 * t310;
                final double t653 = q * FastMath.PI;
                final double t654 = t21 * t653;
                final double t657 = t238 * t641;
                final double t661 = t502 * t167 * lambda;
                final double t664 = Gamma * lambda;
                final double t665 = t511 * t664;
                final double t668 = t615 * t13;
                final double t669 = t88 * t653;
                final double t672 = t523 * t641;
                final double t675 = 311040 * t616 * t619 + 2410 * t616 * t623 + 207360
                        * t626 * t568 + 288 * t629 * t630 + 2352 * t616 * t635 + 622080
                        * t626 * t589 - 60480 * t640 * t642 + 1440 * t626 * t571 + 375
                        * t629 * t648 + 4320 * t626 * t591 - 3600 * t640 * t654 - 720
                        * t640 * t657 + 2360 * t616 * t661 + 311040 * t616 * t665
                        - 54240 * t668 * t669 + 201600 * t668 * t672;
                final double t676 = t615 * t15;
                final double t678 = t167 * p * t31;
                final double t681 = t538 * t641;
                final double t685 = t167 * t30 * t588;
                final double t691 = t9 * a;
                final double t702 = FastMath.PI * t10 * Gamma * thetaTW;
                final double t705 = t691 * l;
                final double t706 = t545 * theta0;
                final double t709 = Cm * c;
                final double t710 = t709 * t38;
                final double t715 = Gamma * p * FastMath.PI;
                final double t718 = t664 * FastMath.PI;
                final double t722 = t82 * a * l;
                final double t726 = t709 * mu;
                final double t739 = t622 * t30;
                final double t742 = 675 * t676 * t678 + 1200 * t668 * t681 - 1320
                        * t629 * t685 - 7464960 * t615 * t506 * q + 9953280 * t691
                        * l * q * FastMath.PI + 311040 * t615 * t15 * Gamma * theta0
                        + 311040 * t616 * t702 - 414720 * t705 * t706 - 6480 * t710
                        * t554 * FastMath.PI + 97200 * t676 * t715 - 233280 * t629
                        * t718 + 69120 * t722 * t653 * t11 + 1866240 * t726 * t567
                        * FastMath.PI - 933120 * t710 * t558 * FastMath.PI + 12960
                        * t726 * t570 * FastMath.PI - 2880 * t722 * t440 * theta0
                        + 2160 * t676 * t739;
                final double t746 = t322 * t330;
                final double t749 = 0.1e1 / (t326 - t13);
                final double t750 = t332 * t596 * t749;
                final double t755 = t64 * Gamma;
                final double t763 = t64 * t167;
                final double t764 = t641 * t30;
                final double t767 = theta0 * t30;
                final double t770 = FastMath.PI * Gamma;
                final double t786 = 32400 * t755 * t641 - 2488320 * t562 + 155520
                        * t525 - 17280 * t521 + 2160 * t540 + 2880 * t556 + 311040
                        * t543 + 414720 * t560 + 240 * t504 - 4800 * t499 + 4320
                        * t591;
                final double t792 = t332 * t485 * t596 * t749;
                final double t796 = t2 * t493;
                final double t797 = t88 * p;
                final double t800 = t523 * q;
                final double t823 = p * mu;
                final double t824 = t615 * t823;
                final double t831 = Gamma * mu;
                final double t832 = t9 * lambda;
                final double t835 = 311040 * t831 * t832 * FastMath.PI;
                final double t836 = t35 * theta0;
                final double t839 = 414720 * t831 * t836 * FastMath.PI;
                final double t840 = 2488320 * t653;
                final double t841 = 103680 * t706;
                final double t843 = Gamma * t13 * t512;
                final double t846 = t167 * t13 * t503;
                final double t849 = 4800 * t418 * t498;
                final double t851 = 140 * t501 * t532;
                final double t852 = t501 * t535;
                final double t855 = 500 * t501 * t551;
                final double t856 = t82 * lambda;
                final double t859 = 2160 * t527 * t856 * FastMath.PI;
                final double t860 = t139 * theta0;
                final double t863 = 2880 * t527 * t860 * FastMath.PI;
                final double t865 = 225 * t440 * t764;
                final double t866 = 17280 * t669;
                final double t868 = 720 * t440 * t767;
                final double t870 = 77760 * t510 * t588;
                final double t872 = 32400 * t545 * t641;
                final double t874 = t89 * t515 * FastMath.PI;
                final double t876 = t835 + t839 - t840 + t841 + 169920 * t843
                        + 780 * t846 - t849 - t851 + 1464 * t852 + t855 + t859 + t863
                        + t865 - t866 + t868 - t870 + t872 + 155520 * t874;
                final double t893 = B * t38;
                final double t895 = l * theta0;
                final double t917 = 233280 * t626 * t702 + 38880 * t705 * t89 * t770
                        + 1540 * t616 * t648 + 77760 * t893 * a * t895 * t770 + 1038
                        * t616 * t630 + 852 * t626 * t635 + 800 * t626 * t623 + 259200
                        * t626 * t619 + 260 * t616 * t685 - 9600 * t668 * t654 + 750
                        * t668 * t657 + 223200 * t668 * t642 + 311040 * t626 * t665;
                final double t927 = Cm * Gamma * FastMath.PI;
                final double t936 = t31 * t11;
                final double t946 = c * B;
                final double t955 = Cm * t167 * FastMath.PI;
                final double t964 = 225 * t640 * t678 + 1360 * t626 * t661 + 103680
                        * t615 * t706 - 155520 * c * t35 * mu * t927 - 106560 * t615
                        * Gamma * t10 * p * FastMath.PI - 7680 * t615 * q * t936
                        + 60 * t615 * t167 * t529 - 77760 * t616 * t718 + 32400 * t640
                        * t715 + 77760 * t946 * t38 * t927 + 720 * t640 * t739 + 540
                        * c * t56 * t38 * t955 - 1080 * c * t139 * mu * t955 - 2488320
                        * t615 * t653;
                final double t970 = c * c;
                final double t971 = t970 * t13;
                final double t975 = t615 * t13 * theta0;
                final double t981 = t78 * thetaTW;
                final double t986 = t10 * thetaTW;
                final double t992 = t841 - t840 + 131040 * t843 + 77760 * t672
                        + 654 * t852 + 1620 * t527 * t981 * FastMath.PI + 38880
                        * t874 + 233280 * t831 * t986 * FastMath.PI + 510 * t846
                        + 540 * t681 + t839;
                final double t993 = -t851 + t865 + t835 + t855 - t849 + t863 + t859 - t866 + t868 - t870 + t872;
                final double t1003 = mu * t11;
                final double t1034 = 600 * t337 - 1440 * t339 + 82944 * t343 - 207360 * t345 - 138240 * t347 + 480 * t349 - 165888 * t351 + 1368 * t354 - 1152 * t356 + 576 * t361 - 5040 * t363 - 435 * t365 - 4800 * t369 + 400 * t372 + 224640 * t375;
                final double t1050 = 1260 * t378 + 374400 * t380 + 290304 * t382 - 960 * t386 + 384 * t388 - 3000 * t390 - 6480 * t394 + 311040 * t398 - 510 * t402 - 1440 * t406 + 680 * t410 + 540 * t414 + 2130 * t416 - 103680 * t421 + 345600 * t423;
                final double t1077 = -207360 * t427 - 73440 * t432 - 432000 * t434 + 2160 * t438 - 3600 * t441 + 300 * t443 - 933120 * t447 + 800 * t450 - 720 * t455 + 252720 * t457 + 77760 * t461 - 432 * t82 * t50 * t358 * thetaTW + 155520 * t199 * t132 - 765 * t56 * t50 * t358 * theta0 - 29160 * B * t50 * t131 * theta0;
                final double t1087 = t11 * FastMath.PI;
                final double t1088 = t1087 * lambda;
                final double t1113 = 5400 * t20 * t15 * t251 * q - 270 * t82 * t15 * t358 * p + 233280 * B * t15 * t1088 + 1620 * t56 * t15 * t28 * FastMath.PI * lambda - 103680 * t9 * t15 * t131 * p - 450 * t30 * t50 * t358 * lambda - 207360 * t463 + 59719680 * t465 + 86400 * t467 + 25920 * t469 + 1440 * t471 + 184320 * t473 - 2557440 * t475 - 1440 * t477 + 600 * t479;
                final double t1123 = t14 * p;
                final double t1126 = t38 * lambda;
                final double t1149 = t1087 * thetaTW;
                final double t1155 = -75 * t1123 * t88 + 180 * t1126 * t88 + 240
                        * t139 * lambda * t1087 + 180 * t78 * theta0 * t1087 + 144
                        * t98 * thetaTW * t1087 - 11520 * t893 * t641 + 17280 * t35
                        * t13 * t105 + 17280 * t35 * mu * t641 + 25920 * t425 * t506
                        - 48 * t56 * t14 * t1149 - 50 * t36 * t14 * t1088;
                final double t1168 = t1087 * p;
                final double t1185 = 25920 * t1126 + 34560 * t232 + 25920 * t10
                        * theta0 * FastMath.PI + 20736 * t62 * thetaTW * FastMath.PI
                        - 3240 * t23 - 85 * t22 * t936 - 30 * t56 * t38 * t1168
                        + 600 * t62 * t38 * t770 * q + 180 * t404 * t506 * t11 + 120
                        * t139 * t13 * t1149 + 120 * t139 * mu * t1168 - 10800
                        * t1123;
                final double t1193 = t370 - t373 - t376 - t379 + t381 + t383 + t387
                        - t389 + t391 + 90 * t394 + t399 - t403 + t407;
                final double t1196 = -t411 + t415 - t417 + t422 + t424 + t428 - t433
                        + t435 + t439 + t442 - t444 - 159840 * t447;
                final double t1199 = t451 + t456 - t458 + t462 + t464 - t466 - t468
                        - t470 - 1620 * t471 - t474 - 233280 * t475 + t478 - t480;
                final double t1228 = t10 * rho;
                final double t1234 = t9 * rho;
                final double t1240 = t35 * rho;
                final double t1246 = rho * t4;
                final double t1247 = t1246 * c;
                final double t1248 = t38 * a;
                final double t1252 = 23040 * t1247 * t1248 * l * p;
                final double t1253 = t56 * rho;
                final double t1258 = 360 * t1253 * t4 * t971 * Cm * t11;
                final double t1262 = t11 * a;
                final double t1270 = Icb * thetaTW;
                final double t1276 = 51840 * t1246 * t946 * t975;
                final double t1277 = t139 * rho;
                final double t1287 = 51840 * t1246 * t970 * B * t13 * Cm;
                final double t1289 = t82 * rho * t493;
                final double t1290 = t13 * thetaTW;
                final double t1295 = 25920 * t1228 * t4 * c * thetaTW * t615 + 51840
                        * t1234 * t4 * t6 * l * lambda + 34560 * t1240 * t4 * c
                        * theta0 * t615 - t1252 + t1258 + 180 * t78 * rho * t493
                        * thetaTW * l * t1262 + 34560 * t1240 * t4 * t970 * Cm - 103680
                        * t1270 - 207360 * Icb * theta0 + t1276 + 240 * t1277 * t4
                        * t970 * Cm * t11 + t1287 + 180 * t1289 * t615 * t1290 * t11;
                final double t1304 = t1234 * t493;
                final double t1308 = t30 * rho * t493;
                final double t1309 = l * t11;
                final double t1313 = 60 * t1308 * t1248 * t1309 * p;
                final double t1316 = t14 * a;
                final double t1320 = 100 * t20 * rho * t493 * t1316 * t1309 * lambda;
                final double t1326 = 1200 * t1228 * t493 * t1248 * l * Gamma * q;
                final double t1333 = t1309 * theta0;
                final double t1336 = 125 * t36 * rho * t493 * t1316 * t1333;
                final double t1341 = 360 * t1253 * t493 * t13 * a * t1333;
                final double t1345 = 96 * t1308 * t1316 * t1309 * thetaTW;
                final double t1346 = t615 * t1290;
                final double t1349 = t30 * Icb;
                final double t1353 = theta0 * t11;
                final double t1356 = 180 * t1289 * t615 * t823 * t11 + 360 * t1289
                        * t615 * lambda * t11 + 25920 * t1304 * t824 - t1313 - t1320
                        + t1326 + 240 * t1277 * t493 * t895 * t1262 - t1336 + t1341
                        - t1345 + 25920 * t1304 * t1346 - 720 * t1349 * thetaTW
                        * t11 - 1440 * t1349 * t1353;
                final double t1367 = t1287 - t1252 - 1440 * t56 * Icb * t1353
                        - 207360 * B * Icb * theta0 + t1276 + t1258 - t1313 - t1345
                        - t1336 + t1341 - t1320 + t1326;
                n[0] = -t8 * (t118 + t192 + t264 + t319) * t322 * t324 * t333 / 155520;
                n[1] = -t8 * t13 * (t367 + t408 + t449 + t481) * t485 * t489 / 25920;
                n[2] = t495 * t496 * (t550 + t593) * t330 * t596 * t322 / 51840;
                n[3] = -t8 * (t367 + t603 + t605 + t607) * t485 * t324 * t333 / 38880;
                n[4] = 0;
                n[5] = t2 * t493 * (t675 + t742) * t746 * t750 / 12960;
                n[6] = -t8 * mu * (92160 * t513 + 103680 * t755 * theta0 + 622080
                        * t589 - 140 * t533 + 384 * t536 + 500 * t552 + 1080 * t530
                        + 225 * t763 * t764 + 720 * t763 * t767 - 77760 * t203 * t770
                        + t786) * t330 * t792 / 8640;
                n[7] = -t796 * mu * (60 * t626 * t797 - 1200 * t626 * t800 + 100
                        * t668 * t21 * lambda + 96 * t668 * t88 * thetaTW + 125 * t668
                        * t37 * theta0 + 103680 * t546 * t615 + 720 * t615 * theta0
                        * t56 * t11 + 103680 * B * Cm * c + 720 * t709 * t57 + 23040
                        * t824) * t746 / 4320;
                n[8] = t8 * t876 * t330 * t332 * t485 * t596 * t749 / 12960;
                n[9] = 0;
                n[10] = t2 * t493 * mu * (t917 + t964) * t330 * t750 / 2160;
                n[11] = t5 * t971 * Cm / 8 + t796 * t975 / 8;
                n[12] = -t8 * mu * (t992 + t993) * t330 * t792 / 6480;
                n[13] = t495 * t496 * (23040 * p + 60 * t797 - 1200 * t800 + 100
                        * t1003 * t20 * lambda + 96 * t1003 * t30 * thetaTW + 125
                        * t1003 * t36 * theta0) * t322 * t330 / 2160;
                n[14] = 0;
                n[15] = -t8 * mu * (t1034 + t1050 + t1077 + t1113) * t324 * t333
                        / 51840;
                n[16] = t495 * l * t13 * Gamma * (t1155 + t1185) * t330 * t596
                        / 11520;
                n[17] = -t8 * mu * (t367 + t1193 + t1196 + t1199) * t485 * t489
                        / 19440;
                n[18] = t495 * t496 * (34560 * t831 * t832 + 105 * t527 * t860
                        + 25920 * t831 * t986 + 33120 * t831 * t836 + 140 * t527
                        * t856 - 276480 * q - 14400 * t524 - 720 * t520 + 84 * t527
                        * t981) * t322 * t330 / 25920;
                n[19] = 0;
                n[20] = t1 * (t1295 + t1356) * t330 / 1440;
                n[21] = -t1 * t1367 * t746 / 2880;
                n[22] = (-2 * t1247 * t615 * lambda - t1247 * t1346 - t1247 * t824
                        + 4 * t1270) * t1 / 24;
                n[23] = -(t895 * a + t709) * c * t5 / 24;
                n[24] = -t1247 * t1 * thetaTW * t615 / 40;
            }
            // Solve
            RealMatrix coefficients = new Array2DRowRealMatrix(m, false);
            DecompositionSolver solver = new LUDecomposition(coefficients).getSolver();
            RealVector constants = new ArrayRealVector(n, false);
            RealVector solution = solver.solve(constants);
            u01 = solution.getEntry(0);
            u02 = solution.getEntry(1);
            u03 = solution.getEntry(2);
            u04 = solution.getEntry(3);
            u05 = solution.getEntry(4);
            u11 = solution.getEntry(5);
            u12 = solution.getEntry(6);
            u13 = solution.getEntry(7);
            u14 = solution.getEntry(8);
            u15 = solution.getEntry(9);
            v11 = solution.getEntry(10);
            v12 = solution.getEntry(11);
            v13 = solution.getEntry(12);
            v14 = solution.getEntry(13);
            v15 = solution.getEntry(14);
            u21 = solution.getEntry(15);
            u22 = solution.getEntry(16);
            u23 = solution.getEntry(17);
            u24 = solution.getEntry(18);
            u25 = solution.getEntry(19);
            v21 = solution.getEntry(20);
            v22 = solution.getEntry(21);
            v23 = solution.getEntry(22);
            v24 = solution.getEntry(23);
            v25 = solution.getEntry(24);
        }
    }

    // Compute flapping coefficients from rotor characteristics, pitch
    // deformation, mu and lambda
    private void computeFlappingCoefficient() {
        // Compute a2,b2
        { // Mapple generated code ---------------------------------------------
            final double t1 = B * B;
            final double t2 = t1 * t1;
            final double t3 = t2 * t1;
            final double t4 = t3 * Gamma;
            final double t7 = t2 * t2;
            final double t8 = t7 * t1;
            final double t9 = t8 * Gamma;
            final double t12 = t7 * Gamma;
            final double t15 = t7 * B;
            final double t16 = t15 * Gamma;
            final double t19 = t7 * t3;
            final double t20 = Gamma * Gamma;
            final double t24 = t1 * B;
            final double t25 = t7 * t24;
            final double t29 = t7 * t2;
            final double t36 = t2 * t24;
            final double t37 = t36 * Gamma;
            final double t40 = t2 * B;
            final double t41 = t7 * t40;
            final double t47 = Gamma * mu;
            final double t77 = 435456 * t4 * u21 + 241920 * t9 * u25 + 311040 * t12
                    * u23 + 272160 * t16 * u24 + 20160 * t19 * t20 * v25 + 30240
                    * t25 * t20 * v22 + 25920 * t29 * t20 * v23 + 36288 * t8 * t20
                    * v21 + 362880 * t37 * u22 + 22680 * t41 * t20 * v24 - 11612160
                    * mu * q + 51840 * t47 * t7 * v14 - 4032 * mu * t8 * t20 * u12
                    - 3024 * mu * t15 * t20 * u11 - 4320 * mu * t29 * t20 * u14 + 48384
                    * t47 * t3 * v12 - 4200 * mu * t41 * t20 * u15 + 50400 * t47
                    * t15 * v15 - 4320 * mu * t25 * t20 * u13;
            final double t91 = mu * mu;
            final double t92 = t91 * t29;
            final double t93 = t20 * Gamma;
            final double t101 = t91 * t2;
            final double t130 = t7 * t7;
            final double t157 = 36288 * t47 * t40 * v11 - 604800 * t47 * t2 * p
                    - 30240 * mu * t7 * t20 * q + 51840 * t47 * t36 * v13 + 3528
                    * t92 * t93 * u01 + 1451520 * t91 * t1 * Gamma * lambda + 1088640
                    * t101 * Gamma * u01 + 889056 * t91 * t40 * Gamma * u02 + 2520
                    * t91 * t19 * t93 * u03 + 2940 * t91 * t41 * t93 * u02 + 749952
                    * t91 * t3 * Gamma * u03 + 648000 * t91 * t36 * Gamma * u04
                    + 2205 * t91 * t7 * t36 * t93 * u04 + 1960 * t91 * t130 * t93
                    * u05 + 5880 * t91 * t8 * t93 * lambda + 570240 * t91 * t7 * Gamma
                    * u05 + 4410 * t91 * t25 * t93 * theta0 + 1391040 * t91 * t24
                    * Gamma * theta0 + 3528 * t92 * t93 * thetaTW + 1088640 * t101
                    * Gamma * thetaTW;
            final double t159 = 0.1e1 / B;
            final double t163 = 1 / (144 + t7 * t20);
            a2 = (t77 + t157) * t159 * t163 / 90720;
            final double t192 = mu * t20;
            final double t205 = 2520 * t25 * u22 * t20 + 3024 * t8 * u21 * t20
                    + 2160 * t29 * u23 * t20 + 1890 * t41 * u24 * t20 + 1680 * t19
                    * u25 * t20 - 22680 * t16 * v24 - 20160 * t9 * v25 - 36288 * t4
                    * v21 - 25920 * t12 * v23 - 30240 * t37 * v22 + 967680 * p * mu
                    + 360 * t192 * t25 * v13 + 336 * t192 * t8 * v12 + 2520 * t192
                    * t7 * p + 360 * t192 * t29 * v14;
            final double t230 = t91 * t20;
            final double t255 = 350 * t192 * t41 * v15 + 4200 * t47 * t15 * u15
                    + 4320 * t47 * t36 * u13 + 3024 * t47 * t40 * u11 + 252 * t192
                    * t15 * v11 - 50400 * t47 * t2 * q + 4032 * t47 * t3 * u12 + 4320
                    * t47 * t7 * u14 + 3234 * t230 * t15 * u02 + 2688 * t230 * t8
                    * u03 + 2295 * t230 * t25 * u04 + 2000 * t230 * t29 * u05 + 4200
                    * t230 * t3 * lambda + 4032 * t230 * t7 * u01 + 5250 * t230 * t36
                    * theta0 + 4032 * t230 * t7 * thetaTW;
            b2 = -(t205 + t255) * t159 * t163 / 7560;
        }
        // Compute a0, a1, b1
        { // Mapple generated code ---------------------------------------------
            final double t1 = mu * mu;
            final double t2 = t1 * mu;
            final double t5 = t1 * t1;
            final double t8 = B * B;
            final double t9 = t8 * theta0;
            final double t13 = t8 * B;
            final double t14 = t13 * t1;
            final double t15 = thetaTW * FastMath.PI;
            final double t22 = t8 * t8;
            final double t23 = t22 * t13;
            final double t24 = t23 * t1;
            final double t28 = t22 * B;
            final double t29 = t28 * t1;
            final double t33 = t22 * t8;
            final double t34 = t33 * t1;
            final double t38 = t28 * mu;
            final double t39 = v12 * FastMath.PI;
            final double t42 = u04 * FastMath.PI;
            final double t45 = t33 * mu;
            final double t46 = v13 * FastMath.PI;
            final double t49 = u05 * FastMath.PI;
            final double t52 = t22 * t22;
            final double t53 = t52 * mu;
            final double t54 = v15 * FastMath.PI;
            final double t57 = t23 * mu;
            final double t58 = v14 * FastMath.PI;
            final double t61 = t13 * mu;
            final double t62 = p * FastMath.PI;
            final double t65 = 2520 * t2 * lambda - 1050 * t5 * p + 2520 * t9 * t1
                    * FastMath.PI + 1680 * t14 * t15 + 1260 * t8 * t1 * b2 * FastMath.PI
                    - 360 * t24 * u25 * FastMath.PI - 504 * t29 * u23 * FastMath.PI
                    - 420 * t34 * u24 * FastMath.PI + 2016 * t38 * t39 + 840 * t34
                    * t42 + 1680 * t45 * t46 + 720 * t24 * t49 + 1260 * t53 * t54
                    + 1440 * t57 * t58 + 1680 * t61 * t62;
            final double t66 = t22 * t1;
            final double t73 = u01 * FastMath.PI;
            final double t76 = u02 * FastMath.PI;
            final double t79 = t22 * mu;
            final double t80 = v11 * FastMath.PI;
            final double t83 = u03 * FastMath.PI;
            final double t92 = t52 * B;
            final double t103 = t13 * lambda * FastMath.PI;
            final double t109 = t5 * theta0 * FastMath.PI;
            final double t114 = -630 * t66 * u22 * FastMath.PI - 840 * t14 * u21
                    * FastMath.PI + 1680 * t14 * t73 + 1260 * t66 * t76 + 2520
                    * t79 * t80 + 1008 * t29 * t83 + 2016 * t28 * u01 * FastMath.PI
                    + 1680 * t33 * u02 * FastMath.PI + 1120 * t92 * u05 * FastMath.PI
                    + 1260 * t52 * u04 * FastMath.PI + 1440 * t23 * u03 * FastMath.PI
                    + 3360 * t103 + 2520 * t22 * theta0 * FastMath.PI - 315 * t109
                    + 2016 * t28 * thetaTW * FastMath.PI;
            final double t117 = 0.1e1 / FastMath.PI;
            a0 = (t65 + t114) * Gamma * t117 / 20160;
            final double t121 = Gamma * mu;
            final double t130 = Gamma * t1;
            final double t175 = -40320 * q * FastMath.PI + 2880 * t121 * t23 * u04
                    * FastMath.PI + 2520 * t121 * t52 * u05 * FastMath.PI + 1080
                    * t130 * t23 * v15 * FastMath.PI + 4032 * t121 * t28 * u02 * FastMath.PI
                    + 3360 * t121 * t33 * u03 * FastMath.PI - 1680 * t121 * t13 * b2
                    * FastMath.PI + 2520 * t130 * t13 * v11 * FastMath.PI - 1440
                    * t121 * t23 * u24 * FastMath.PI - 1680 * t121 * t33 * u23 * FastMath.PI
                    - 2016 * t121 * t28 * u22 * FastMath.PI - 2520 * t121 * t22 * u21
                    * FastMath.PI + 1260 * t130 * t33 * v14 * FastMath.PI + 1512
                    * t130 * t28 * v13 * FastMath.PI;
            final double t210 = Gamma * t5;
            final double t227 = 1890 * t130 * t22 * v12 * FastMath.PI + 5040 * t121
                    * t8 * lambda * FastMath.PI - 1260 * t121 * t52 * u25 * FastMath.PI
                    + 5040 * t121 * t22 * u01 * FastMath.PI + 6720 * t121 * t13 * theta0
                    * FastMath.PI + 5040 * t121 * t22 * thetaTW * FastMath.PI + 2520
                    * Gamma * t22 * t62 + 1120 * Gamma * t92 * t54 - 1260 * Gamma * t2
                    * lambda * FastMath.PI + 525 * t210 * t62 + 2016 * Gamma * t28
                    * t80 + 1680 * Gamma * t33 * t39 + 1440 * Gamma * t23 * t46 + 1260
                    * Gamma * t52 * t58 + 1680 * t210 * theta0;
            final double t229 = 0.1e1 / Gamma;
            final double t231 = 0.1e1 / t8;
            final double t233 = 2 * t8;
            a1 = (t175 + t227) * t229 * t231 * t117 / (t233 - t1) / 1260;
            final double t238 = FastMath.PI * FastMath.PI;
            final double t245 = t238 * Gamma;
            final double t252 = Gamma * Gamma;
            final double t253 = t252 * t2;
            final double t259 = t252 * t5 * mu;
            final double t264 = t252 * t1;
            final double t265 = t52 * t13;
            final double t309 = t52 * t8;
            final double t315 = t252 * t5 * t1;
            final double t320 = t252 * t28;
            final double t324 = -725760 * t238 * p - 6480 * t245 * t24 * u15 - 22680
                    * t245 * t53 * v25 - 1260 * t253 * t92 * u24 * t238 + 1260 * t259
                    * t52 * v15 * FastMath.PI + 3780 * t264 * t265 * v15 * t238 + 5040
                    * t264 * t33 * p * t238 + 2520 * t259 * t22 * v11 * FastMath.PI
                    + 7560 * t264 * t23 * v11 * t238 + 2016 * t259 * t28 * v12
                    * FastMath.PI + 6048 * t264 * t52 * v12 * t238 + 1680 * t259 * t33
                    * v13 * FastMath.PI + 5040 * t264 * t92 * v13 * t238 + 10920
                    * t252 * t5 * t103 - 1470 * t259 * p * t13 * FastMath.PI + 1440
                    * t259 * t23 * v14 * FastMath.PI + 4320 * t264 * t309 * v14 * t238
                    - 840 * t315 * t13 * u21 * FastMath.PI + 2016 * t320 * t73 * t5;
            final double t325 = t252 * t52;
            final double t330 = t252 * t33;
            final double t334 = t252 * t92;
            final double t339 = t252 * t23;
            final double t398 = 6048 * t325 * u01 * t238 * mu + 1680 * t330 * t76
                    * t5 + 5040 * t334 * u02 * t238 * mu + 1440 * t339 * t83 * t5
                    + 4320 * t252 * t309 * u03 * t238 * mu + 1260 * t325 * t42 * t5
                    + 3780 * t252 * t265 * u04 * t238 * mu + 1120 * t334 * t49 * t5
                    + 3360 * t252 * t52 * t22 * u05 * t238 * mu + 10080 * t330 * lambda
                    * t238 * mu + 1260 * t315 * t8 * b2 * FastMath.PI + 3780 * t253
                    * t28 * b2 * t238 - 36288 * t245 * t38 * v22 - 30240 * t245
                    * t45 * v23 - 25920 * t245 * t57 * v24 + 30240 * t245 * t61 * a2
                    - 15120 * t245 * t14 * u11 - 11340 * t245 * t66 * u12 - 9072
                    * t245 * t29 * u13;
            final double t416 = theta0 * t238;
            final double t470 = -7560 * t245 * t34 * u14 - 45360 * t245 * t79 * v21
                    + 2520 * t315 * t9 * FastMath.PI + 7560 * t253 * t28 * theta0
                    * t238 + 2520 * t252 * t22 * t109 + 7560 * t339 * t416 * mu
                    - 945 * t259 * t416 * t13 + 1680 * t315 * t13 * thetaTW * FastMath.PI
                    + 5040 * t253 * t33 * thetaTW * t238 + 2016 * t320 * t15 * t5
                    + 6048 * t325 * thetaTW * t238 * mu - 2520 * t253 * t33 * u21
                    * t238 - 630 * t315 * t22 * u22 * FastMath.PI - 1890 * t253 * t23
                    * u22 * t238 - 504 * t315 * t28 * u23 * FastMath.PI - 1512 * t253
                    * t52 * u23 * t238 - 420 * t315 * t33 * u24 * FastMath.PI - 360
                    * t315 * t23 * u25 * FastMath.PI - 1080 * t253 * t309 * u25
                    * t238;
            final double t529 = t5 * t5;
            final double t530 = t252 * t529;
            final double t540 = 840 * t315 * t33 * u04 * FastMath.PI + 2520 * t253
                    * t92 * u04 * t238 + 720 * t315 * t23 * u05 * FastMath.PI + 2160
                    * t253 * t309 * u05 * t238 + 1260 * t315 * t22 * u02 * FastMath.PI
                    + 3780 * t253 * t23 * u02 * t238 + 1008 * t315 * t28 * u03
                    * FastMath.PI + 3024 * t253 * t52 * u03 * t238 + 1680 * t315 * t13
                    * u01 * FastMath.PI + 5040 * t253 * t33 * u01 * t238 - 20160
                    * t245 * t92 * u15 - 45360 * t245 * t22 * q - 36288 * t245 * t28
                    * u11 - 30240 * t245 * t33 * u12 - 25920 * t245 * t23 * u13
                    - 22680 * t245 * t52 * u14 - 315 * t530 * theta0 * FastMath.PI
                    + 2520 * t252 * t5 * t2 * lambda - 1050 * t530 * p;
            b1 = t231 / t238 / (t233 + t1) * (t324 + t398 + t470 + t540) * t229
                    / 22680;
        }
    }

    // Compute torque coefficient from rotor characteritics, pitch deformation
    // mu and lambda
    private void computeTorqueCoefficient() {
        // Airfoil section numbers
        final double a = bladeAirfoil.getLiftSlope();
        final double delta = bladeAirfoil.getProfileDrag();

        // Mapple generated code -----------------------------------------------
        final double t1 = mu * mu;
        final double t3 = t1 * t1;
        cqp = delta * (-8 - 8 * t1 + t3) / 64;
        final double t6 = B * B;
        final double t7 = t6 * B;
        final double t12 = a * t3;
        final double t13 = thetaTW * lambda;
        final double t16 = t6 * t6;
        final double t17 = a * t16;
        final double t20 = t16 * t16;
        final double t21 = a * t20;
        final double t25 = t16 * t7;
        final double t26 = a * t25;
        final double t30 = t16 * t6;
        final double t31 = a * t30;
        final double t35 = t16 * B;
        final double t36 = a * t35;
        final double t70 = a * t7 * theta0 * lambda / 6 + t12 * t13 / 64 + t17
                * t13 / 8 - t21 * v24 * a2 / 16 - t26 * v23 * a2 / 14 - t31 * v22
                * a2 / 12 - t36 * v21 * a2 / 10 + t21 * u24 * b2 / 16 + t26 * u23
                * b2 / 14 + t31 * u22 * b2 / 12 + t21 * v14 * p / 32 + t36 * u21
                * b2 / 10 - t21 * v14 * a1 / 32 - t26 * v13 * a1 / 28 + t26 * v13
                * p / 28 + t31 * v12 * p / 24 - t31 * v12 * a1 / 24;
        final double t102 = a * t20 * B;
        final double t121 = t1 * mu;
        final double t122 = a * t121;
        final double t126 = -t36 * v11 * a1 / 20 + t36 * v11 * p / 20 + t26 * u13
                * q / 28 + t21 * u14 * b1 / 32 + t21 * u14 * q / 32 + t26 * u13
                * b1 / 28 + t31 * u12 * q / 24 + t36 * u11 * b1 / 20 + t36 * u11
                * q / 20 + t31 * u12 * b1 / 24 + t102 * u25 * b2 / 18 - t102 * v25
                * a2 / 18 + t102 * u15 * b1 / 36 + t17 * q * b1 / 8 - t102 * v15
                * a1 / 36 - t17 * p * a1 / 8 - 0.3e1 / 0.16e2 * t122 * lambda * a1;
        final double t128 = a * t1;
        final double t129 = a1 * a1;
        final double t133 = u01 * lambda;
        final double t139 = a0 * a0;
        final double t163 = b1 * b1;
        final double t175 = mu * a;
        final double t176 = t30 * u13;
        final double t180 = t25 * u14;
        final double t188 = 0.3e1 / 0.32e2 * t128 * t6 * t129 + t12 * t133 / 64
                + t122 * lambda * p / 8 + t128 * t6 * t139 / 8 + t36 * u02 * lambda
                / 10 + t102 * u15 * q / 36 + t17 * t133 / 8 + t31 * u03 * lambda
                / 12 + t26 * u04 * lambda / 14 + t21 * u05 * lambda / 16 + t102 * v15
                * p / 36 + t128 * t6 * t163 / 32 + t128 * t35 * u13 * b1 / 80 + t128
                * t16 * u12 * b1 / 64 - t175 * t176 * a0 / 24 - t175 * t180 * a2
                / 56 + t128 * t30 * u14 * b1 / 96;
        final double t196 = a2 * a2;
        final double t199 = b2 * b2;
        final double t202 = q * q;
        final double t205 = p * p;
        final double t208 = lambda * lambda;
        final double t220 = t25 * v24;
        final double t231 = t30 * v23;
        final double t239 = -t175 * t176 * a2 / 48 + t17 * t129 / 16 + t17 * t163
                / 16 + t17 * t196 / 4 + t17 * t199 / 4 - t12 * t202 / 128 - 0.5e1
                / 0.128e3 * t12 * t205 - t128 * t208 / 8 - t12 * t139 / 32 + t17
                * t205 / 16 + a * t6 * t208 / 4 + t17 * t202 / 16 + t175 * t220 * q
                / 56 + t175 * t220 * b1 / 28 - t128 * t30 * v24 * a0 / 48 + t175
                * t231 * b1 / 24 - t128 * t35 * v23 * a0 / 40;
        final double t249 = t35 * v22;
        final double t253 = t16 * v21;
        final double t267 = t25 * u24;
        final double t274 = t30 * u23;
        final double t281 = t35 * u22;
        final double t288 = t30 * v14;
        final double t295 = t16 * u21;
        final double t302 = t175 * t231 * q / 48 - t128 * t16 * v22 * a0 / 32 + t175
                * t249 * q / 40 + t175 * t253 * q / 32 + t175 * t249 * b1 / 20 + t175
                * t253 * b1 / 16 - t128 * t7 * v21 * a0 / 24 + t175 * t267 * a1
                / 28 - t175 * t267 * p / 56 + t175 * t274 * a1 / 24 - t175 * t274
                * p / 48 - t175 * t281 * p / 40 + t175 * t281 * a1 / 20 + t128 * t288
                * a1 / 96 + t175 * t288 * lambda / 24 + t175 * t295 * a1 / 16 - t175
                * t295 * p / 32;
        final double t303 = t35 * v13;
        final double t314 = t16 * v12;
        final double t336 = t7 * v11;
        final double t343 = t35 * u12;
        final double t351 = t16 * u11;
        final double t365 = t175 * t303 * lambda / 20 - t175 * t25 * v14 * b2 / 56
                + t128 * t303 * a1 / 80 + t175 * t314 * lambda / 16 - t175 * t30
                * v13 * b2 / 48 - t175 * t35 * v12 * b2 / 40 + t128 * t314 * a1 / 64
                - t175 * t180 * a0 / 28 - t175 * t16 * v11 * b2 / 32 + t128 * t336
                * a1 / 48 + t175 * t336 * lambda / 12 - t175 * t343 * a0 / 20 + t128
                * t7 * u11 * b1 / 48 - t175 * t351 * a0 / 16 - t175 * t343 * a2
                / 40 + t175 * t7 * p * b2 / 12 - t175 * t351 * a2 / 32;
        final double t395 = t7 * q;
        final double t399 = t20 * u15;
        final double t407 = t20 * v25;
        final double t427 = t20 * u25;
        final double t431 = t25 * v15;
        final double t435 = -t175 * t20 * v15 * b2 / 64 + t175 * t7 * a1 * b2 / 12
                + t128 * t7 * u01 * b2 / 24 + t128 * t30 * u04 * b2 / 48 + t128
                * t25 * u05 * b2 / 56 + t128 * t16 * u02 * b2 / 32 + t128 * t35 * u03
                * b2 / 40 + t175 * t395 * a2 / 12 - t175 * t399 * a2 / 64 - t175
                * t7 * a0 * b1 / 6 + t175 * t407 * b1 / 32 - t175 * t7 * b1 * a2 / 12
                - t128 * t6 * a0 * a2 / 8 + t128 * t25 * u15 * b1 / 112 + t175 * t6
                * lambda * a1 / 4 + t175 * t427 * a1 / 32 + t128 * t431 * a1 / 112;
        final double t437 = a / FastMath.PI;
        final double t505 = 0.4e1 / 0.45e2 * t437 * t3 * q * a0 + t175 * t7 * theta0
                * p / 12 + t128 * t6 * theta0 * b2 / 16 - 0.2e1 / 0.45e2 * t437
                * t3 * theta0 * p + t437 * t121 * theta0 * lambda / 9 + t128 * t7
                * thetaTW * b2 / 24 + t175 * t16 * thetaTW * p / 16 + t175 * t407
                * q / 64 + t175 * t35 * u02 * p / 20 + t175 * t431 * lambda / 28
                + t175 * t30 * u03 * p / 24 - t175 * t399 * a0 / 32 - t175 * t395
                * a0 / 6 + t175 * t20 * u05 * p / 32 + t175 * t25 * u04 * p / 28
                - t175 * t427 * p / 64 + t175 * t16 * u01 * p / 16 - t128 * t25
                * v25 * a0 / 56;
        cqi = t70 + t126 + t188 + t239 + t302 + t365 + t435 + t505;
    }

    // Compute thrust coefficient from rotor characteritics, pitch deformation
    // mu and lambda
    private void computeThrustCoefficient() {
        // Airfoil section numbers
        final double a = bladeAirfoil.getLiftSlope();

        // Mapple generated code -----------------------------------------------
        final double t1 = B * B;
        final double t2 = t1 * t1;
        final double t3 = a * t2;
        final double t6 = mu * mu;
        final double t7 = t6 * t6;
        final double t8 = a * t7;
        final double t11 = t2 * t1;
        final double t15 = t2 * B;
        final double t21 = t2 * t2;
        final double t25 = t1 * B;
        final double t26 = t2 * t25;
        final double t33 = a * t6;
        final double t36 = t6 * mu;
        final double t37 = a * t36;
        final double t58 = t3 * thetaTW / 8 - t8 * thetaTW / 64 + a * t11 * u03
                / 12 + a * t15 * u02 / 10 + t3 * u01 / 8 + a * t21 * u05 / 16 + a
                * t26 * u04 / 14 + a * t1 * lambda / 4 + t33 * lambda / 8 - t37 * p
                / 32 - t8 * u01 / 64 + t37 * a1 / 16 + a * t25 * theta0 / 6 + t33
                * B * theta0 / 4 - 0.2e1 / 0.9e1 * a / FastMath.PI * t36 * theta0
                + t33 * t1 * thetaTW / 8;
        final double t59 = mu * a;
        final double t111 = t59 * t26 * v15 / 14 + t59 * t1 * p / 8 + t33 * t2
                * u03 / 16 - t33 * t11 * u25 / 48 + t33 * t25 * u02 / 12 + t33 * t15
                * u04 / 20 + t33 * t11 * u05 / 24 + t33 * t1 * u01 / 8 + t33 * B
                * b2 / 8 + t59 * t25 * v11 / 6 + t59 * t2 * v12 / 8 + t59 * t15 * v13
                / 10 + t59 * t11 * v14 / 12 - t33 * t1 * u21 / 16 - t33 * t25 * u22
                / 24 - t33 * t2 * u23 / 32 - t33 * t15 * u24 / 40;
        ct = t58 + t111;
    }

    // Compute H and Y coefficients
    private void computeRearAndLateralForcesCoefficients() {
        // Airfoil section numbers
        final double a = bladeAirfoil.getLiftSlope();
        final double delta = bladeAirfoil.getProfileDrag();

        // Mapple generated code -----------------------------------------------
        chp = delta * mu / 4;
        final double t2 = B * B;
        final double t3 = t2 * t2;
        final double t5 = t3 * B * a;
        final double t9 = a * t2;
        final double t10 = mu * theta0;
        final double t14 = B * a;
        final double t18 = t2 * B;
        final double t19 = t18 * a;
        final double t20 = mu * thetaTW;
        final double t28 = t3 * t18 * a;
        final double t29 = mu * v15;
        final double t33 = p * mu;
        final double t37 = mu * q;
        final double t41 = mu * v25;
        final double t45 = mu * a0;
        final double t50 = t3 * t2 * a;
        final double t51 = mu * u04;
        final double t55 = a * t3;
        final double t56 = mu * u02;
        final double t60 = t5 * u02 * a1 / 10 + 0.3e1 / 0.16e2 * t9 * t10 * b2
                - t14 * t10 * lambda / 4 + t19 * t20 * b2 / 8 - t9 * t20 * lambda
                / 8 + t28 * t29 * a1 / 28 - t9 * t33 * a1 / 32 - t9 * t37 * b1 / 32
                + t28 * t41 * a2 / 28 - t9 * t45 * a2 / 4 + t50 * t51 * b2 / 16
                + 0.3e1 / 0.32e2 * t55 * t56 * b2;
        final double t61 = mu * u25;
        final double t65 = mu * u03;
        final double t69 = mu * u05;
        final double t73 = mu * u01;
        final double t77 = mu * lambda;
        final double t81 = u11 * mu;
        final double t85 = mu * u12;
        final double t89 = u13 * mu;
        final double t93 = mu * u14;
        final double t97 = mu * v11;
        final double t104 = v12 * mu;
        final double t108 = -t28 * t61 * b2 / 28 + 0.3e1 / 0.40e2 * t5 * t65 * b2
                + 0.3e1 / 0.56e2 * t28 * t69 * b2 + t19 * t73 * b2 / 8 - t14 * t77
                * b2 / 8 - t19 * t81 * q / 48 - t55 * t85 * q / 64 - t5 * t89 * q
                / 80 - t50 * t93 * q / 96 + t19 * t97 * a1 / 12 - t19 * t97 * p
                / 16 + t55 * t104 * a1 / 16;
        final double t113 = mu * v13;
        final double t120 = v14 * mu;
        final double t127 = mu * u21;
        final double t131 = mu * u22;
        final double t135 = mu * u23;
        final double t139 = mu * u24;
        final double t143 = mu * v21;
        final double t147 = mu * v22;
        final double t151 = mu * v23;
        final double t155 = -0.3e1 / 0.64e2 * t55 * t104 * p + t5 * t113 * a1
                / 20 - 0.3e1 / 0.80e2 * t5 * t113 * p + t50 * t120 * a1 / 24 - t50
                * t120 * p / 32 + t9 * t127 * lambda / 16 + t19 * t131 * lambda
                / 24 + t55 * t135 * lambda / 32 + t5 * t139 * lambda / 40 - t19 * t143
                * a0 / 24 - t55 * t147 * a0 / 32 - t5 * t151 * a0 / 40;
        final double t156 = mu * v24;
        final double t169 = t3 * t3;
        final double t170 = t169 * a;
        final double t198 = -t50 * t156 * a0 / 48 + t55 * u01 * a1 / 8 + t50 * u03
                * a1 / 12 + t28 * u04 * a1 / 14 + t170 * u05 * a1 / 16 - t19 * a0
                * b1 / 12 + t19 * b1 * a2 / 8 + 0.3e1 / 0.64e2 * t170 * u15 * a2
                + 0.5e1 / 0.24e2 * t19 * q * a2 - t19 * a1 * b2 / 8 + 0.5e1 / 0.24e2
                * t19 * p * b2 + 0.3e1 / 0.64e2 * t170 * v15 * b2 + 0.3e1 / 0.32e2
                * t55 * u11 * a2;
        final double t237 = -t55 * u11 * a0 / 16 + 0.3e1 / 0.40e2 * t5 * u12 * a2
                - t5 * u12 * a0 / 20 + t50 * u13 * a2 / 16 - t50 * u13 * a0 / 24
                + 0.3e1 / 0.56e2 * t28 * u14 * a2 - t28 * u14 * a0 / 28 + 0.3e1
                / 0.32e2 * t55 * v11 * b2 - t19 * v11 * lambda / 12 + 0.3e1 / 0.40e2
                * t5 * v12 * b2 - t55 * v12 * lambda / 16 + t50 * v13 * b2 / 16;
        final double t274 = a0 * a0;
        final double t278 = -t5 * v13 * lambda / 20 + 0.3e1 / 0.56e2 * t28 * v14
                * b2 - t50 * v14 * lambda / 24 + t55 * u21 * p / 32 + t5 * u22 * p
                / 40 + t50 * u23 * p / 48 + t28 * u24 * p / 56 - t55 * v21 * q / 32
                - t5 * v22 * q / 40 - t50 * v23 * q / 48 - t28 * v24 * q / 56 + 0.3e1
                / 0.8e1 * t9 * lambda * a1 + t9 * mu * t274 / 8;
        final double t304 = a1 * a1;
        final double t317 = t170 * u25 * p / 64 - t50 * u03 * p / 24 - t9 * lambda
                * p / 4 - t28 * u04 * p / 28 - t170 * u05 * p / 32 - t19 * q * a0
                / 12 - t170 * v25 * q / 64 - t55 * u01 * p / 16 + t9 * mu * t304 / 8
                - t28 * v15 * lambda / 28 - t170 * u15 * a0 / 32 - t5 * u02 * p
                / 20;
        final double t348 = mu * u15;
        final double t358 = t19 * theta0 * a1 / 6 - t19 * theta0 * p / 12 + t55
                * thetaTW * a1 / 8 - t55 * thetaTW * p / 16 - t5 * t51 * lambda / 20
                - t9 * t73 * lambda / 8 - 0.3e1 / 0.112e3 * t28 * t29 * p - t19
                * t56 * lambda / 12 + t50 * t61 * lambda / 48 - t28 * t41 * a0 / 56
                - t28 * t348 * q / 112 - t55 * t65 * lambda / 16 - t50 * t69 * lambda
                / 24;
        chi = t60 + t108 + t155 + t198 + t237 + t278 + t317 + t358;
        final double t373 = mu * mu;
        final double t397 = t373 * v15;
        final double t411 = t28 * t348 * p / 112 + t28 * t29 * q / 112 + t50 * t41
                * lambda / 48 + t50 * t120 * q / 96 + t14 * t373 * theta0 * b1 / 4
                - 0.3e1 / 0.8e1 * t9 * t10 * a0 - 0.3e1 / 0.16e2 * t9 * t10 * a2
                - t19 * t20 * a2 / 8 + t9 * t373 * thetaTW * b1 / 8 - t19 * t20 * a0
                / 4 + t28 * t61 * a0 / 56 - t50 * t397 * a0 / 24 + t14 * t77 * a2
                / 8 - t50 * t397 * a2 / 48 + t50 * t373 * u15 * b2 / 48;
        final double t461 = -t9 * t373 * v11 * a0 / 8 + t19 * t97 * b1 / 6 - t19
                * t373 * v12 * a0 / 12 + t55 * t104 * b1 / 8 - t55 * t373 * v13
                * a0 / 16 + t5 * t113 * b1 / 10 - t5 * t373 * v14 * a0 / 20 + t50
                * t120 * b1 / 12 + t19 * t127 * a0 / 24 + t55 * t131 * a0 / 32 + t5
                * t135 * a0 / 40 + t50 * t139 * a0 / 48 - 0.3e1 / 0.4e1 * t14 * t77
                * a0 + 0.3e1 / 0.56e2 * t28 * t61 * a2 + t50 * t93 * p / 96;
        final double t509 = t5 * t156 * lambda / 40 - t50 * t51 * a0 / 8 + t5 * t89
                * a1 / 20 - 0.3e1 / 0.40e2 * t5 * t65 * a2 + t28 * t348 * a1 / 28
                - 0.3e1 / 0.20e2 * t5 * t65 * a0 + t9 * mu * a1 * b1 / 8 - t19 * t73
                * a2 / 8 + 0.3e1 / 0.56e2 * t28 * t41 * b2 + t50 * t93 * a1 / 24
                - t9 * t45 * b2 / 4 - 0.3e1 / 0.28e2 * t28 * t69 * a0 - 0.5e1 / 0.24e2
                * t19 * p * a2 + t19 * a0 * a1 / 12 + t9 * lambda * q / 4;
        final double t563 = -t19 * t73 * a0 / 4 - 0.3e1 / 0.56e2 * t28 * t69 * a2
                - t14 * t373 * a0 * a1 / 2 + t50 * t373 * v25 * a1 / 48 + t28 * t29
                * b1 / 14 + t50 * t373 * u05 * b1 / 24 + t5 * t373 * u04 * b1 / 20
                - t50 * t373 * u25 * b1 / 48 + t19 * t373 * u02 * b1 / 12 + t55
                * t373 * u03 * b1 / 16 + t9 * t373 * u01 * b1 / 8 + t5 * t113 * q
                / 80 + t5 * t89 * p / 80 + t55 * t151 * lambda / 32 + t55 * t85 * p
                / 64;
        final double t611 = t55 * t104 * q / 64 - t50 * t51 * a2 / 16 + t55 * t85
                * a1 / 16 + t19 * t81 * a1 / 12 - 0.3e1 / 0.16e2 * t55 * t56 * a0
                + 0.5e1 / 0.32e2 * t9 * t33 * b1 + 0.7e1 / 0.32e2 * t9 * t37 * a1
                - 0.3e1 / 0.32e2 * t55 * t56 * a2 + t19 * t147 * lambda / 24 + t19
                * t81 * p / 48 + t19 * t97 * q / 48 + t9 * t143 * lambda / 16 + t50
                * v23 * p / 48 + t170 * u25 * q / 64 + t55 * u21 * q / 32;
        final double t657 = t5 * u22 * q / 40 + t50 * u23 * q / 48 + t28 * u24 * q
                / 56 + t28 * u04 * b1 / 14 + t170 * u05 * b1 / 16 + t50 * u03 * b1
                / 12 + t5 * u02 * b1 / 10 + t55 * u01 * b1 / 8 + t19 * theta0 * b1
                / 6 + t19 * theta0 * q / 12 + t55 * thetaTW * b1 / 8 + t55 * thetaTW
                * q / 16 + t55 * u01 * q / 16 + t28 * u15 * lambda / 28 + 0.3e1
                / 0.40e2 * t5 * u12 * b2;
        final double t704 = t170 * u05 * q / 32 + t5 * u13 * lambda / 20 + t170
                * v25 * p / 64 + t5 * v22 * p / 40 + 0.3e1 / 0.8e1 * t9 * lambda * b1
                + 0.5e1 / 0.24e2 * t19 * q * b2 + 0.3e1 / 0.56e2 * t28 * u14 * b2
                + t28 * v24 * p / 56 + t50 * u03 * q / 24 + t50 * u14 * lambda / 24
                + t55 * v21 * p / 32 + t19 * u11 * lambda / 12 + t50 * u13 * b2 / 16
                - t50 * v13 * a2 / 16 - 0.3e1 / 0.56e2 * t28 * v14 * a2;
        final double t753 = 0.3e1 / 0.64e2 * t170 * u15 * b2 - 0.3e1 / 0.64e2 * t170
                * v15 * a2 - 0.3e1 / 0.32e2 * t55 * v11 * a2 + t55 * u12 * lambda
                / 16 + t5 * u02 * q / 20 - t28 * v14 * a0 / 28 - t5 * v12 * a0 / 20
                - t50 * v13 * a0 / 24 + 0.3e1 / 0.32e2 * t55 * u11 * b2 + t19 * b2
                * b1 / 8 + t19 * a1 * a2 / 8 - t55 * v11 * a0 / 16 - t170 * v15 * a0
                / 32 + t28 * u04 * q / 28 - t19 * p * a0 / 12 - 0.3e1 / 0.40e2 * t5
                * v12 * a2;
        cyi = t411 + t461 + t509 + t563 + t611 + t657 + t704 + t753;
    }

    // Compute harmonic forces and torque coefficients
    private void computeHarmonicCoefficients() {
        // Airfoil section numbers
        final double a = bladeAirfoil.getLiftSlope();
        final double delta = bladeAirfoil.getProfileDrag();

        // Mapple generated code -----------------------------------------------
        {
            final double t1 = B * B;
            final double t2 = t1 * t1;
            final double t3 = t2 * a;
            final double t8 = t2 * B * a;
            final double t12 = t1 * B;
            final double t13 = t12 * a;
            final double t18 = t2 * t1 * a;
            final double t23 = t2 * t12 * a;
            final double t27 = mu * mu;
            final double t52 = t3 * mu * u02 / 4 + t8 * mu * u03 / 5 + t13 * mu
                    * u01 / 3 + t18 * mu * u04 / 6 + t23 * mu * u05 / 7 + t13 * t27
                    * v12 / 8 + 0.3e1 / 0.32e2 * t3 * t27 * v13 + t18 * t27 * v15
                    / 16 + 0.3e1 / 0.40e2 * t8 * t27 * v14 - t18 * mu * u24 / 12
                    - t23 * mu * u25 / 14 - t8 * mu * u23 / 10 - t3 * u22 * mu / 8;
            final double t56 = t1 * a;
            final double t66 = B * a;
            final double t88 = t2 * t2;
            final double t89 = t88 * a;
            final double t92 = -t13 * mu * u21 / 6 + 0.3e1 / 0.16e2 * t56 * t27
                    * v11 + t56 * theta0 * mu / 2 + t13 * mu * thetaTW / 3 + t66
                    * mu * lambda / 2 - t56 * mu * b2 / 8 + t66 * t27 * a1 / 8 + t23
                    * v14 / 14 - t13 * a1 / 6 + t8 * v12 / 10 + t3 * v11 / 8 + t18
                    * v13 / 12 + t13 * p / 6 + t89 * v15 / 16;
            tp1S = t52 + t92;
            tp1C = t3 * t27 * u13 / 32 + t8 * t27 * u14 / 40 - t56 * mu * a2 / 8
                    + t66 * t27 * b1 / 8 + t13 * t27 * u12 / 24 + t56 * t27 * u11
                    / 16 + t18 * t27 * u15 / 48 + t18 * v24 * mu / 12 + t23 * mu
                    * v25 / 14 + t3 * mu * v22 / 8 + t8 * mu * v23 / 10 + t13 * v21
                    * mu / 6 - t56 * mu * a0 / 4 + t13 * b1 / 6 + t8 * u12 / 10 + t23
                    * u14 / 14 + t3 * u11 / 8 + t13 * q / 6 + t89 * u15 / 16 + t18
                    * u13 / 12;
            tp2S = t3 * t27 * v23 / 16 + t13 * t27 * v22 / 12 + t18 * t27 * v25
                    / 24 + t56 * mu * b1 / 4 - t66 * t27 * a0 / 4 + t56 * t27 * v21
                    / 8 + t8 * t27 * v24 / 20 + t56 * mu * q / 8 + t18 * mu * u14
                    / 12 + t23 * mu * u15 / 14 + t8 * mu * u13 / 10 + t3 * mu * u12
                    / 8 + t13 * mu * u11 / 6 - t13 * a2 / 3 + t3 * v21 / 8 + t8 * v22
                    / 10 + t23 * v24 / 14 + t89 * v25 / 16 + t18 * v23 / 12;
            final double t233 = t56 * mu * a1 / 4 - t56 * t27 * u01 / 8 + t18 * t27
                    * u25 / 24 - t56 * mu * p / 8 - t3 * t27 * u03 / 16 - t8 * mu * v13
                    / 10 - t3 * mu * v12 / 8 - t8 * t27 * u04 / 20 - t18 * t27 * u05
                    / 24 - t23 * mu * v15 / 14 - t18 * mu * v14 / 12 - t13 * t27 * u02
                    / 12;
            final double t267 = -t13 * mu * v11 / 6 + t8 * u24 * t27 / 20 + t3 * u23
                    * t27 / 16 + t13 * u22 * t27 / 12 + t56 * t27 * u21 / 8 - t66 * theta0
                    * t27 / 4 - t56 * t27 * thetaTW / 8 + t13 * b2 / 3 + t3 * u21 / 8 + t89
                    * u25 / 16 + t8 * u22 / 10 + t23 * u24 / 14 + u23 * t18 / 12;
            tp2C = t233 + t267;
        }
        {
            final double t2 = mu * mu;
            hpP1S = delta / 6 + 0.3e1 / 0.8e1 * delta * t2;
            hpP2C = -delta * mu / 4;
        }
        {
            final double t1 = B * B;
            final double t2 = t1 * B;
            final double t3 = t2 * a;
            final double t7 = t1 * a;
            final double t11 = t1 * t1;
            final double t12 = t11 * a;
            final double t17 = t11 * t2 * a;
            final double t18 = mu * v25;
            final double t23 = t11 * B * a;
            final double t24 = mu * u23;
            final double t28 = mu * u22;
            final double t32 = mu * v22;
            final double t36 = mu * v21;
            final double t41 = t11 * t1 * a;
            final double t42 = mu * v24;
            final double t46 = mu * u11;
            final double t50 = mu * u21;
            final double t54 = mu * u24;
            final double t58 = mu * u15;
            final double t62 = mu * v15;
            final double t66 = t3 * theta0 * b2 / 4 - t7 * theta0 * lambda / 4
                    + 0.3e1 / 0.16e2 * t12 * thetaTW * b2 - t17 * t18 * q / 56 + t23
                    * t24 * p / 20 + t12 * t28 * p / 16 - t12 * t32 * q / 32 - t3
                    * t36 * q / 24 - t41 * t42 * q / 48 + t3 * t46 * a2 / 12 - t3
                    * t50 * a1 / 12 - t41 * t54 * a1 / 24 - t17 * t58 * a0 / 56 + 0.5e1
                    / 0.56e2 * t17 * t62 * b2;
            final double t67 = mu * u25;
            final double t74 = mu * v14;
            final double t78 = mu * u03;
            final double t82 = mu * u05;
            final double t92 = mu * u14;
            final double t99 = mu * q;
            final double t106 = mu * u12;
            final double t110 = B * a;
            final double t111 = mu * lambda;
            final double t115 = mu * v23;
            final double t119 = -t17 * t67 * a1 / 28 + t17 * t58 * a2 / 28 - 0.3e1
                    / 0.40e2 * t23 * t74 * lambda - 0.3e1 / 0.40e2 * t23 * t78 * p
                    - 0.3e1 / 0.56e2 * t17 * t82 * p + t41 * t54 * p / 24 + t17 * t67 * p
                    / 28 - t41 * t92 * a0 / 48 + 0.5e1 / 0.48e2 * t41 * t74 * b2 + t7
                    * t99 * a0 / 16 + t17 * t82 * a1 / 14 + t12 * t106 * a2 / 16 - t110
                    * t111 * a1 / 8 - t23 * t115 * q / 40;
            final double t121 = mu * v12;
            final double t125 = mu * v13;
            final double t132 = mu * u04;
            final double t139 = mu * p;
            final double t143 = mu * u02;
            final double t153 = mu * u01;
            final double t163 = mu * u13;
            final double t170 = 0.5e1 / 0.32e2 * t12 * t121 * b2 - 0.3e1 / 0.32e2 * t12
                    * t125 * lambda - t41 * t62 * lambda / 16 - t41 * t132 * p / 16 - t12
                    * t106 * a0 / 32 - t7 * t139 * b2 / 16 + t12 * t143 * a1 / 8 - t12
                    * t28 * a1 / 16 - t3 * t46 * a0 / 24 + t3 * t153 * a1 / 6 + t3 * t50
                    * p / 12 + t41 * t132 * a1 / 12 - t23 * t163 * a0 / 40 + t23 * t125
                    * b2 / 8;
            final double t171 = mu * v11;
            final double t193 = mu * a0;
            final double t197 = mu * b1;
            final double t201 = mu * b2;
            final double t211 = mu * thetaTW;
            final double t218 = theta0 * mu;
            final double t222 = 0.5e1 / 0.24e2 * t3 * t171 * b2 + t41 * t92 * a2
                    / 24 + t23 * t163 * a2 / 20 + t23 * t78 * a1 / 10 - 0.3e1 / 0.32e2
                    * t12 * t143 * p - 0.3e1 / 0.16e2 * t7 * t171 * lambda - t23 * t24
                    * a1 / 20 - t7 * t193 * b1 / 16 + t7 * t197 * a2 / 8 + 0.3e1
                    / 0.16e2 * t7 * t201 * a1 - t3 * t121 * lambda / 8 - t3 * t153 * p
                    / 8 + t3 * t211 * a1 / 6 - t3 * t211 * p / 8 + t7 * t218 * a1 / 4;
            final double t240 = t11 * t11;
            final double t241 = t240 * a;
            final double t269 = -0.3e1 / 0.16e2 * t7 * t218 * p - t3 * thetaTW
                    * lambda / 6 + t17 * v14 * a1 / 14 + 0.7e1 / 0.24e2 * t3 * p * a1
                    - 0.3e1 / 0.56e2 * t17 * v14 * p - 0.3e1 / 0.64e2 * t241 * v15 * p
                    + 0.3e1 / 0.32e2 * t241 * u05 * b2 + 0.3e1 / 0.28e2 * t17 * u04
                    * b2 - t3 * q * b1 / 24 - t23 * u22 * b2 / 10 + t23 * v22 * a2 / 10
                    - t41 * v13 * p / 16 - t17 * u24 * b2 / 14 - 0.3e1 / 0.40e2 * t23
                    * v12 * p;
            final double t312 = -0.3e1 / 0.32e2 * t12 * v11 * p - t41 * u13 * q / 48
                    + t23 * v12 * a1 / 10 - t12 * u11 * q / 32 - t12 * v21 * a0 / 16
                    - t241 * v25 * a0 / 32 - t23 * u12 * q / 40 - t17 * u05 * lambda
                    / 14 - t41 * u04 * lambda / 12 - t12 * u02 * lambda / 8 - t23
                    * u03 * lambda / 10 - t3 * u01 * lambda / 6 + t3 * a0 * a2 / 6
                    + t17 * v24 * a2 / 14;
            final double t356 = t241 * v15 * a1 / 16 + t41 * v13 * a1 / 12 + t41
                    * u03 * b2 / 8 - t23 * v22 * a0 / 20 - t17 * u14 * q / 56 - t241
                    * u15 * q / 64 + t12 * u22 * lambda / 16 + t17 * u25 * lambda / 28
                    + t41 * u24 * lambda / 24 + t3 * u21 * lambda / 12 + t12 * v21 * a2
                    / 8 - t12 * u21 * b2 / 8 - t241 * u25 * b2 / 16 + t12 * v11 * a1 / 8;
            final double t384 = a1 * a1;
            final double t387 = a2 * a2;
            final double t390 = b2 * b2;
            final double t393 = q * q;
            final double t396 = p * p;
            final double t399 = lambda * lambda;
            final double t402 = -t41 * u23 * b2 / 12 + t41 * v23 * a2 / 12 + 0.5e1 / 0.8e1
                    * t7 * lambda * b2 + t241 * v25 * a2 / 16 - t41 * v23 * a0 / 24 + 0.3e1
                    / 0.20e2 * t23 * u02 * b2 + 0.3e1 / 0.16e2 * t12 * u01 * b2 + t23 * u23
                    * lambda / 20 - t17 * v24 * a0 / 28 - t3 * t384 / 6 - t3 * t387 / 3
                    - t3 * t390 / 3 - t3 * t393 / 24 - t3 * t396 / 8 - t110 * t399 / 2;
            hiP1S = t66 + t119 + t170 + t222 + t269 + t312 + t356 + t402;
            final double t441 = -t41 * t58 * lambda / 48 + t17 * t58 * b2 / 56 + t17
                    * t62 * a2 / 28 - t17 * t62 * a0 / 56 - t12 * t163 * lambda / 32
                    - t41 * t42 * p / 48 - t17 * t18 * p / 56 + t12 * t121 * a2 / 16
                    - t12 * t121 * a0 / 32 - t41 * t132 * q / 48 - t110 * t111 * b1
                    / 8 + t12 * t106 * b2 / 32;
            final double t478 = t7 * t139 * a0 / 16 - t7 * t99 * b2 / 16 + t3 * t36
                    * a1 / 12 + t3 * t46 * b2 / 24 - t3 * t106 * lambda / 24 - t3
                    * t153 * q / 24 - t17 * t82 * q / 56 - t23 * t92 * lambda / 40
                    - t23 * t115 * p / 40 - t12 * t32 * p / 32 + t3 * t171 * a2 / 12
                    - t3 * t171 * a0 / 24;
            final double t513 = mu * a1;
            final double t517 = -t23 * t78 * q / 40 + t12 * t32 * a1 / 16 + t23
                    * t163 * b2 / 40 + t17 * t18 * a1 / 28 + t41 * t42 * a1 / 24
                    - t12 * t143 * q / 32 + t41 * t74 * a2 / 24 - t41 * t74 * a0
                    / 48 - t3 * t36 * p / 24 + t23 * t125 * a2 / 20 - t7 * t46 * lambda
                    / 16 + 0.3e1 / 0.8e1 * t7 * t513 * a2;
            final double t557 = -0.7e1 / 0.16e2 * t7 * t193 * a1 + t23 * t115 * a1
                    / 20 + t7 * t197 * b2 / 16 - t23 * t125 * a0 / 40 + t41 * t92
                    * b2 / 48 - t3 * t211 * q / 24 - t7 * t218 * q / 16 - t41 * v24
                    * lambda / 24 - t3 * p * b1 / 24 + t17 * u24 * a2 / 28 + t241
                    * u15 * a1 / 16 + t17 * u14 * a1 / 14 + 0.3e1 / 0.28e2 * t17
                    * u04 * a2;
            final double t596 = -t12 * u21 * a0 / 16 - t23 * u12 * p / 40 + t12
                    * v21 * b2 / 16 + 0.5e1 / 0.24e2 * t3 * q * a1 - t17 * v14 * q
                    / 56 - t17 * u14 * p / 56 - t23 * v23 * lambda / 20 - t17 * u24
                    * a0 / 28 - t23 * v12 * q / 40 - t41 * u23 * a0 / 24 - t3 * a0
                    * b2 / 6 - t241 * v15 * q / 64;
            final double t636 = t23 * v22 * b2 / 20 + t23 * u22 * a2 / 20 + 0.5e1
                    / 0.8e1 * t7 * lambda * a2 - t241 * u25 * a0 / 32 + t241 * u25
                    * a2 / 32 + t241 * v25 * b2 / 32 + 0.3e1 / 0.32e2 * t241 * u05
                    * a2 + 0.3e1 / 0.20e2 * t23 * u02 * a2 + t23 * u12 * a1 / 10
                    + 0.3e1 / 0.16e2 * t12 * u01 * a2 + t12 * u11 * a1 / 8 + t12
                    * u21 * a2 / 16 + t3 * a1 * b1 / 6;
            final double t674 = -t41 * v13 * q / 48 - t41 * u13 * p / 48 - t12
                    * u11 * p / 32 - t12 * v11 * q / 32 - t7 * lambda * a0 / 4 - t241
                    * u05 * a0 / 16 - t17 * u04 * a0 / 14 - t41 * u03 * a0 / 12
                    - t23 * u02 * a0 / 10 - t12 * u01 * a0 / 8 + t41 * u03 * a2 / 8
                    + t41 * u13 * a1 / 12;
            final double t714 = -t23 * u22 * a0 / 20 - t3 * q * p / 12 - t241 * u15
                    * p / 64 - t12 * v22 * lambda / 16 - t3 * v21 * lambda / 12 - t17
                    * v25 * lambda / 28 + t41 * u23 * a2 / 24 + t41 * v23 * b2 / 24
                    + t17 * v24 * b2 / 28 + t3 * theta0 * a2 / 4 - t3 * theta0 * a0
                    / 6 + 0.3e1 / 0.16e2 * t12 * thetaTW * a2 - t12 * thetaTW * a0 / 8;
            hiP1C = t441 + t478 + t517 + t557 + t596 + t636 + t674 + t714;
            final double t750 = -t17 * t62 * q / 56 + t17 * t58 * a1 / 28 - t23
                    * t125 * q / 40 + 0.5e1 / 0.112e3 * t17 * t18 * b2 - t7 * t99 * a1
                    / 16 - t41 * t74 * q / 48 - t41 * t92 * p / 48 - t41 * t54 * a2
                    / 96 + t23 * t115 * b2 / 16 + t110 * t111 * a0 / 4 - t17 * t58
                    * p / 56;
            final double t787 = -t12 * t115 * lambda / 16 - t17 * t67 * a2 / 112
                    + t41 * t92 * a1 / 24 - t41 * t132 * a0 / 24 + t12 * t106 * a1
                    / 16 - t3 * t50 * a2 / 48 + 0.5e1 / 0.48e2 * t3 * t36 * b2 + t12
                    * t143 * a2 / 8 - t23 * t24 * a2 / 80 - t7 * t139 * b1 / 16 - t3
                    * t153 * a0 / 12 - t7 * t36 * lambda / 8;
            final double t825 = -t23 * t78 * a0 / 20 + t23 * t163 * a1 / 20 + 0.5e1
                    / 0.96e2 * t41 * t42 * b2 - t12 * t28 * a2 / 64 + t3 * t153 * a2
                    / 6 - t23 * t163 * p / 40 - t3 * t171 * q / 24 - t12 * t143 * a0
                    / 16 - t17 * t82 * a0 / 28 + t17 * t82 * a2 / 14 + t23 * t78 * a2
                    / 10 + t41 * t132 * a2 / 12;
            final double t862 = -t3 * t46 * p / 24 + 0.5e1 / 0.64e2 * t12 * t32 * b2
                    + t3 * t46 * a1 / 12 - t7 * t218 * a0 / 8 + t7 * t218 * a2 / 4
                    + t3 * t211 * a2 / 6 - t3 * t211 * a0 / 12 - t3 * t32 * lambda / 12
                    - t23 * t42 * lambda / 20 - t12 * t121 * q / 32 - t12 * t106 * p
                    / 32 + 0.3e1 / 0.16e2 * t7 * t201 * a2;
            final double t901 = -t7 * t193 * b2 / 4 + t7 * t513 * b1 / 8 - t41 * t18
                    * lambda / 24 + t23 * v12 * a2 / 10 - t12 * v11 * a0 / 16 - t23
                    * v12 * a0 / 20 + t3 * a0 * a1 / 12 + t17 * v24 * a1 / 14 + t241
                    * v15 * a2 / 16 - t17 * v24 * p / 28 + t41 * v13 * a2 / 12 + t41
                    * u13 * b2 / 24;
            final double t938 = -t12 * u12 * lambda / 16 - t3 * u11 * lambda / 12
                    - t241 * u05 * q / 32 - t41 * u14 * lambda / 24 + t3 * b1 * b2 / 12
                    - t12 * u01 * q / 16 + t41 * v23 * a1 / 12 + t12 * v21 * a1 / 8
                    + t12 * v11 * a2 / 8 - t41 * v23 * p / 24 + t241 * v25 * a1 / 16
                    - t41 * v13 * a0 / 24;
            final double t976 = t23 * u12 * b2 / 20 + t12 * u11 * b2 / 16 - t3 * a1
                    * a2 / 2 - t12 * v21 * p / 16 - t3 * p * a0 / 12 - t17 * v14
                    * a0 / 28 - t23 * u13 * lambda / 20 - t23 * u02 * q / 20 - t41
                    * u03 * q / 24 - t241 * v15 * a0 / 32 + t17 * v14 * a2 / 14 + t3
                    * p * a2 / 3;
            final double t1013 = -t7 * lambda * b1 / 8 - t17 * u04 * q / 28 - t17
                    * u15 * lambda / 28 - t7 * lambda * q / 4 + t241 * u15 * b2 / 32
                    + t17 * u14 * b2 / 28 - t23 * v22 * p / 20 + t3 * q * b2 / 12
                    + t23 * v22 * a1 / 10 - t241 * v25 * p / 32 - t3 * theta0 * q
                    / 12 - t12 * thetaTW * q / 16;
            hiP2S = t750 + t787 + t825 + t862 + t901 + t938 + t976 + t1013;
            final double t1049 = -t23 * t125 * a1 / 20 + t3 * theta0 * p / 12 + t12
                    * thetaTW * p / 16 - t7 * mu * t390 / 32 - t12 * u21 * p / 16 + t3
                    * b1 * a2 / 12 - t41 * u23 * p / 24 - t12 * v11 * b2 / 8 - t17 * v14
                    * b2 / 14 - t241 * u25 * p / 32 + t3 * a1 * b2 / 2;
            final double t1077 = a0 * a0;
            final double t1084 = -t23 * v12 * b2 / 10 + t241 * u15 * a2 / 32 + t41
                    * u03 * p / 24 - t241 * u15 * a0 / 32 + t17 * u04 * p / 28 + t241
                    * u05 * p / 32 + t17 * v15 * lambda / 28 + t41 * v14 * lambda / 24
                    + t7 * lambda * p / 4 + t7 * mu * t1077 / 8 - t12 * u11 * a0 / 16;
            final double t1119 = -t3 * a0 * b1 / 12 + t12 * v12 * lambda / 16 + t12
                    * u01 * p / 16 + t23 * v13 * lambda / 20 + t7 * mu * t384 / 8
                    - t7 * lambda * a1 / 8 + 0.5e1 / 0.32e2 * t7 * mu * t387 + t23
                    * u12 * a2 / 20 - t17 * u24 * p / 28 - t23 * u12 * a0 / 20 - t41
                    * v13 * b2 / 12;
            final double t1153 = -t241 * v15 * b2 / 16 - t3 * p * b2 / 3 + t3 * q
                    * a2 / 12 + t3 * v11 * lambda / 12 - t17 * u14 * a0 / 28 - t41
                    * u13 * a0 / 24 + t23 * u02 * p / 20 - t3 * q * a0 / 12 - t23
                    * u22 * p / 20 + t17 * u14 * a2 / 28 + t41 * u13 * a2 / 24;
            final double t1189 = t12 * u11 * a2 / 16 + t17 * u24 * a1 / 14 + t23
                    * u22 * a1 / 10 + t41 * u23 * a1 / 12 + t12 * u21 * a1 / 8 + t241
                    * u25 * a1 / 16 - t12 * t143 * b2 / 8 - t17 * t18 * a2 / 112
                    - t17 * t82 * b2 / 14 - t12 * t32 * a2 / 64 + 0.7e1 / 0.64e2
                    * t12 * t28 * b2;
            final double t1223 = -t3 * t153 * b2 / 6 - t23 * t115 * a2 / 80 + 0.7e1
                    / 0.48e2 * t3 * t50 * b2 - t23 * t78 * b2 / 10 + t41 * t82
                    * lambda / 24 + t23 * t125 * p / 20 + t12 * t78 * lambda / 16
                    + t23 * t132 * lambda / 20 + t41 * t74 * p / 24 + t12 * t121 * p
                    / 16 - t7 * t50 * lambda / 8;
            final double t1258 = -t17 * t62 * a1 / 28 - t12 * t121 * a1 / 16 - t41
                    * t74 * a1 / 24 + t17 * t62 * p / 28 + t110 * t218 * lambda / 4
                    - t7 * t218 * b2 / 4 + t7 * t211 * lambda / 8 - t3 * t211 * b2
                    / 6 + t7 * t153 * lambda / 8 + t3 * t171 * p / 12 - t3 * t171 * a1 / 12;
            final double t1295 = t3 * t143 * lambda / 12 - t7 * t193 * a2 / 4 - t41
                    * t132 * b2 / 12 - t3 * t28 * lambda / 12 + 0.7e1 / 0.80e2 * t23
                    * t24 * b2 - t41 * t67 * lambda / 24 - t41 * t42 * a2 / 96 + t17
                    * t67 * b2 / 16 + 0.7e1 / 0.96e2 * t41 * t54 * b2 - t12 * t24
                    * lambda / 16 - t23 * t54 * lambda / 20 - t3 * t36 * a2 / 48;
            hiP2C = t1049 + t1084 + t1119 + t1153 + t1189 + t1223 + t1258 + t1295;
        }
        {
            final double t2 = mu * mu;
            ypP1C = -delta / 6 - delta * t2 / 8;
            ypP2S = -delta * mu / 4;
        }
        {
            final double t1 = B * B;
            final double t2 = t1 * t1;
            final double t4 = t2 * B * a;
            final double t5 = mu * mu;
            final double t6 = u24 * t5;
            final double t13 = t5 * v14;
            final double t17 = t2 * a;
            final double t18 = mu * u02;
            final double t22 = t1 * B;
            final double t23 = t22 * a;
            final double t24 = mu * u01;
            final double t28 = t1 * a;
            final double t29 = mu * p;
            final double t33 = t5 * v12;
            final double t37 = t5 * u01;
            final double t41 = t5 * u04;
            final double t45 = B * a;
            final double t46 = t5 * a1;
            final double t51 = t2 * t22 * a;
            final double t52 = mu * u05;
            final double t57 = t2 * t1 * a;
            final double t58 = t5 * u05;
            final double t62 = t5 * v15;
            final double t66 = t5 * u02;
            final double t70 = mu * u04;
            final double t74 = t5 * u15;
            final double t78 = t5 * u03;
            final double t82 = mu * u24;
            final double t86 = t5 * a0;
            final double t90 = t4 * t6 * a2 / 20 + t4 * t6 * a0 / 20 + 0.3e1
                    / 0.40e2 * t4 * t13 * b1 + t17 * t18 * b1 / 4 + t23 * t24 * b1
                    / 3 - t28 * t29 * a2 / 8 + t23 * t33 * b1 / 8 - t28 * t37 * a0
                    / 4 - t4 * t41 * a2 / 20 + t45 * t46 * b1 / 4 + t51 * t52 * b1
                    / 7 - t57 * t58 * a2 / 24 + t57 * t62 * b1 / 16 - t23 * t66 * a2
                    / 12 + t57 * t70 * b1 / 6 + t57 * t74 * a1 / 48 - t17 * t78 * a0
                    / 8 - t57 * t82 * b1 / 12 - t45 * t86 * b2 / 2;
            final double t91 = mu * v24;
            final double t95 = mu * v25;
            final double t99 = mu * v12;
            final double t106 = mu * u15;
            final double t113 = mu * v15;
            final double t120 = mu * u13;
            final double t124 = mu * u03;
            final double t128 = mu * v22;
            final double t132 = mu * u12;
            final double t136 = mu * lambda;
            final double t143 = mu * v23;
            final double t150 = mu * v11;
            final double t160 = t57 * t91 * p / 48 + t51 * t95 * p / 56 - 0.7e1
                    / 0.32e2 * t17 * t99 * a0 + t57 * t70 * q / 48 + t57 * t106
                    * lambda / 48 + 0.3e1 / 0.56e2 * t51 * t106 * b2 - 0.3e1 / 0.28e2
                    * t51 * t113 * a2 - t51 * t113 * a0 / 8 + t17 * t120 * lambda
                    / 32 + t4 * t124 * q / 40 + t17 * t128 * a1 / 16 + 0.3e1 / 0.32e2
                    * t17 * t132 * b2 + 0.5e1 / 0.8e1 * t45 * t136 * b1 - 0.3e1
                    / 0.16e2 * t17 * t99 * a2 + t4 * t143 * p / 40 + t17 * t128 * p
                    / 32 - t23 * t150 * a2 / 4 - 0.7e1 / 0.24e2 * t23 * t150 * a0
                    + t51 * t52 * q / 56;
            final double t162 = mu * u14;
            final double t178 = mu * q;
            final double t182 = mu * v21;
            final double t186 = mu * u11;
            final double t199 = mu * b1;
            final double t203 = mu * v13;
            final double t210 = mu * v14;
            final double t226 = t4 * t162 * lambda / 40 + t57 * t91 * a1 / 24 + 0.3e1
                    / 0.40e2 * t4 * t120 * b2 + t51 * t95 * a1 / 28 - 0.5e1 / 0.16e2
                    * t28 * t29 * a0 + 0.3e1 / 0.16e2 * t28 * t178 * b2 + t23 * t182
                    * a1 / 12 + t23 * t186 * b2 / 8 + t23 * t132 * lambda / 24 + t23
                    * t24 * q / 24 + t4 * t143 * a1 / 20 + t28 * t199 * b2 / 16
                    - 0.7e1 / 0.40e2 * t4 * t203 * a0 + t57 * t162 * b2 / 16 - t57
                    * t210 * a2 / 8 - 0.7e1 / 0.48e2 * t57 * t210 * a0 + t17 * t18
                    * q / 32 - 0.3e1 / 0.20e2 * t4 * t203 * a2 + t28 * t186 * lambda
                    / 16;
            final double t227 = mu * a1;
            final double t231 = mu * a0;
            final double t238 = t5 * thetaTW;
            final double t245 = mu * thetaTW;
            final double t252 = u23 * t5;
            final double t256 = t5 * v24;
            final double t260 = t5 * u14;
            final double t267 = v21 * t5;
            final double t271 = v23 * t5;
            final double t278 = t5 * u21;
            final double t282 = mu * u25;
            final double t289 = mu * u23;
            final double t293 = t5 * u13;
            final double t297 = -t28 * t227 * a2 / 4 + 0.3e1 / 0.16e2 * t28 * t231
                    * a1 + t23 * t182 * p / 24 - t28 * t238 * a0 / 4 - t28 * t238
                    * a2 / 8 + t23 * t245 * b1 / 3 - t57 * t58 * a0 / 12 + t17 * t252
                    * a0 / 16 + t4 * t256 * b2 / 20 + t4 * t260 * a1 / 40 + t17
                    * t252 * a2 / 16 + t28 * t267 * b2 / 8 + t17 * t271 * b2 / 16
                    - t17 * t78 * a2 / 16 + t28 * t278 * a2 / 8 - t51 * t282 * b1
                    / 14 + t4 * t124 * b1 / 5 - t4 * t289 * b1 / 10 + t17 * t293 * a1
                    / 32;
            final double t309 = theta0 * mu;
            final double t316 = theta0 * t5;
            final double t326 = u22 * t5;
            final double t330 = mu * u21;
            final double t340 = v22 * t5;
            final double t344 = mu * u22;
            final double t348 = t5 * u12;
            final double t352 = t5 * v13;
            final double t356 = t5 * u11;
            final double t360 = t5 * v25;
            final double t364 = t5 * u25;
            final double t368 = -t4 * t41 * a0 / 10 + t28 * t278 * a0 / 8 - t23
                    * t66 * a0 / 6 + t28 * t309 * q / 16 + t23 * t245 * q / 24 - t45
                    * t316 * a2 / 4 + t28 * t309 * b1 / 2 - t45 * t316 * a0 / 2
                    + t23 * t326 * a2 / 12 - t23 * t330 * b1 / 6 - t28 * t37 * a2
                    / 8 + t23 * t326 * a0 / 12 + t23 * t340 * b2 / 12 - t17 * t344
                    * b1 / 8 + t23 * t348 * a1 / 24 + 0.3e1 / 0.32e2 * t17 * t352
                    * b1 + t28 * t356 * a1 / 16 + t57 * t360 * b2 / 24 + t57 * t364
                    * a2 / 24;
            final double t372 = t5 * v11;
            final double t400 = t2 * t2;
            final double t401 = t400 * a;
            final double t429 = t57 * t364 * a0 / 24 + 0.3e1 / 0.16e2 * t28 * t372
                    * b1 - t17 * thetaTW * a0 / 8 - 0.3e1 / 0.16e2 * t17 * thetaTW
                    * a2 + t51 * v24 * b2 / 28 - t23 * theta0 * a2 / 4 - t23 * theta0
                    * a0 / 6 + t57 * v23 * b2 / 24 + t57 * u23 * a2 / 24 + t51
                    * v25 * lambda / 28 + t401 * u15 * p / 64 + t17 * v22 * lambda
                    / 16 + t23 * v21 * lambda / 12 + t23 * q * p / 12 + t4 * u22 * a0
                    / 20 - t4 * u02 * a0 / 10 - t17 * u01 * a0 / 8 - t57 * u03 * a2
                    / 8 - t57 * u03 * a0 / 12;
            final double t488 = -t51 * u04 * a0 / 14 - t401 * u05 * a0 / 16 - t28
                    * lambda * a0 / 4 + t17 * v11 * q / 32 + t17 * u11 * p / 32
                    + t57 * u13 * p / 48 + t57 * v13 * q / 48 - t23 * a1 * b1 / 6
                    + t17 * u21 * a2 / 16 - 0.3e1 / 0.20e2 * t4 * u02 * a2 - 0.3e1
                    / 0.16e2 * t17 * u01 * a2 + t401 * v25 * b2 / 32 - 0.3e1 / 0.32e2
                    * t401 * u05 * a2 + t401 * u25 * a2 / 32 + t51 * u14 * p / 56
                    + t4 * v23 * lambda / 20 + t51 * u24 * a0 / 28 + t4 * v12 * q / 40
                    + t57 * u23 * a0 / 24;
            final double t549 = t23 * a0 * b2 / 6 + t401 * v15 * q / 64 + t4 * v22
                    * b2 / 20 + t4 * u22 * a2 / 20 - 0.5e1 / 0.8e1 * t28 * lambda
                    * a2 + t401 * u25 * a0 / 32 + t51 * v14 * q / 56 - t23 * q * a1
                    / 24 + t17 * v21 * b2 / 16 + t4 * u12 * p / 40 + t17 * u21 * a0
                    / 16 - 0.3e1 / 0.28e2 * t51 * u04 * a2 + t51 * u24 * a2 / 28 + 0.5e1
                    / 0.24e2 * t23 * p * b1 + t57 * v24 * lambda / 24 + t57 * v13
                    * b1 / 12 + t4 * v12 * b1 / 10 + t401 * v15 * b1 / 16 + t17 * v11
                    * b1 / 8 + t51 * v14 * b1 / 14;
            yiP1S = t90 + t160 + t226 + t297 + t368 + t429 + t488 + t549;
            final double t603 = t57 * u04 * lambda / 12 + t51 * u05 * lambda / 14
                    + t23 * u01 * lambda / 6 + t4 * u03 * lambda / 10 + t17 * u02
                    * lambda / 8 + 0.3e1 / 0.56e2 * t51 * u14 * q - t4 * v22 * a0
                    / 20 + t57 * u03 * b2 / 8 - t51 * v24 * a2 / 14 + t23 * a0 * a2
                    / 6 + t17 * u22 * lambda / 16 + 0.3e1 / 0.64e2 * t401 * u15
                    * q + t51 * u25 * lambda / 28 + t57 * u24 * lambda / 24 + t23
                    * u21 * lambda / 12 - t57 * v23 * a0 / 24 - t401 * v25 * a2 / 16;
            final double t655 = 0.5e1 / 0.8e1 * t28 * lambda * b2 - t57 * v23 * a2 / 12
                    + t57 * u23 * b2 / 12 + t401 * u25 * b2 / 16 + t17 * u21 * b2 / 8
                    - t17 * v21 * a2 / 8 + t4 * u23 * lambda / 20 + 0.3e1 / 0.16e2
                    * t17 * u01 * b2 + 0.3e1 / 0.20e2 * t4 * u02 * b2 - t51 * v24
                    * a0 / 28 + 0.3e1 / 0.32e2 * t17 * u11 * q + t57 * v13 * p / 48
                    + t51 * u24 * b2 / 14 + t4 * v12 * p / 40 + t17 * v11 * p / 32
                    + t57 * u13 * q / 16 - t4 * v22 * a2 / 10;
            final double t696 = mu * b2;
            final double t709 = t4 * u22 * b2 / 10 + 0.7e1 / 0.24e2 * t23 * q * b1
                    + 0.3e1 / 0.32e2 * t401 * u05 * b2 + 0.3e1 / 0.28e2 * t51 * u04
                    * b2 + t51 * v14 * p / 56 + t401 * v15 * p / 64 + t23 * thetaTW
                    * lambda / 6 - t23 * p * a1 / 24 + 0.3e1 / 0.16e2 * t17 * thetaTW
                    * b2 + t28 * theta0 * lambda / 4 + t23 * theta0 * b2 / 4 + t51
                    * u14 * b1 / 14 - t28 * t199 * a2 / 8 + 0.7e1 / 0.16e2 * t28 * t696
                    * a1 + t23 * t99 * lambda / 24 + t23 * t24 * p / 24 + t4 * t289
                    * a1 / 20;
            final double t764 = t4 * t124 * a1 / 10 + t17 * t18 * p / 32 + t28 * t150
                    * lambda / 16 + t57 * t70 * a1 / 12 - t4 * t120 * a0 / 8 + 0.3e1
                    / 0.40e2 * t4 * t203 * b2 + t23 * t150 * b2 / 8 - 0.5e1 / 0.24e2
                    * t23 * t186 * a0 + t23 * t24 * a1 / 6 + 0.3e1 / 0.16e2 * t28
                    * t29 * b2 + t17 * t18 * a1 / 8 + t17 * t344 * a1 / 16 + 0.7e1
                    / 0.8e1 * t45 * t136 * a1 + t4 * t143 * q / 40 + 0.3e1 / 0.32e2
                    * t17 * t99 * b2 + t17 * t203 * lambda / 32 + t57 * t113 * lambda
                    / 48 + t57 * t70 * p / 48;
            final double t818 = -0.5e1 / 0.32e2 * t17 * t132 * a0 - 0.7e1 / 0.16e2
                    * t28 * t178 * a0 + t51 * t52 * a1 / 14 + t51 * t282 * a1 / 28 + t4
                    * t210 * lambda / 40 + t4 * t124 * p / 40 + t51 * t52 * p / 56
                    - 0.5e1 / 0.48e2 * t57 * t162 * a0 + t57 * t210 * b2 / 16 + t57
                    * t82 * a1 / 24 - 0.5e1 / 0.56e2 * t51 * t106 * a0 + 0.3e1 / 0.56e2
                    * t51 * t113 * b2 + t23 * t330 * a1 / 12 + t57 * t91 * q / 48
                    + t17 * t128 * q / 32 + t23 * t182 * q / 24 + t51 * t95 * q / 56;
            final double t870 = t45 * t316 * b2 / 4 - t45 * t86 * a2 / 2 + t4 * t143
                    * b1 / 10 + t4 * t13 * a1 / 40 + t57 * t91 * b1 / 12 + t23 * t66
                    * b2 / 12 + t51 * t95 * b1 / 14 + t57 * t62 * a1 / 48 + t57 * t74
                    * b1 / 48 + t17 * t78 * b2 / 16 + t23 * t33 * a1 / 24 + t17 * t128
                    * b1 / 8 + t28 * t37 * b2 / 8 + t4 * t41 * b2 / 20 + t23 * t182
                    * b1 / 6 + t4 * t260 * b1 / 40 + t57 * t58 * b2 / 24;
            final double t899 = b1 * b1;
            final double t909 = b2 * b2;
            final double t916 = a1 * a1;
            final double t926 = -t4 * t256 * a0 / 20 - t23 * t340 * a0 / 12 + t23
                    * t348 * b1 / 24 + t17 * t352 * a1 / 32 + t28 * t238 * b2 / 8
                    - 0.9e1 / 0.16e2 * t28 * t231 * b1 + t23 * t245 * p / 24 + t57
                    * u13 * b1 / 12 + t401 * u15 * b1 / 16 + t45 * t5 * t899 / 8
                    + t4 * u12 * b1 / 10 + t17 * u11 * b1 / 8 + t45 * t5 * t909 / 4
                    + t23 * t245 * a1 / 6 + 0.3e1 / 0.8e1 * t45 * t5 * t916 + t28
                    * t309 * p / 16 - t401 * v25 * a0 / 32;
            final double t930 = a0 * a0;
            final double t934 = a2 * a2;
            final double t938 = p * p;
            final double t941 = lambda * lambda;
            final double t950 = q * q;
            final double t980 = t28 * t309 * a1 / 4 + t45 * t5 * t930 / 2 + t45
                    * t5 * t934 / 4 + t23 * t938 / 24 + t45 * t941 / 2 + t23 * t899
                    / 6 + t23 * t934 / 3 + t23 * t909 / 3 + t23 * t950 / 8 - t28
                    * t267 * a0 / 8 - t17 * t271 * a0 / 16 + t28 * t372 * a1 / 16
                    + t28 * t178 * a2 / 4 - t57 * t360 * a0 / 24 + t17 * t293 * b1
                    / 32 + t28 * t356 * b1 / 16 - t17 * v21 * a0 / 16 + 0.3e1
                    / 0.40e2 * t4 * u12 * q;
            yiP1C = t603 + t655 + t709 + t764 + t818 + t870 + t926 + t980;
            final double t1028 = t51 * v15 * lambda / 28 + t401 * u05 * p / 32
                    + t51 * u04 * p / 28 - t401 * u15 * a0 / 32 + t57 * u03 * p
                    / 24 - t401 * u15 * a2 / 16 + t4 * v12 * b2 / 20 - t23 * a1
                    * b2 / 12 + t51 * v14 * b2 / 28 + t17 * v11 * b2 / 16 - t23
                    * b1 * a2 / 2 + 0.5e1 / 0.32e2 * t28 * mu * t909 + t17 * thetaTW
                    * p / 16 + t17 * v12 * lambda / 16 - t23 * a0 * b1 / 12;
            final double t1077 = -t17 * u11 * a0 / 16 + t28 * mu * t930 / 8 + t28
                    * lambda * p / 4 + t57 * v14 * lambda / 24 + t17 * u01 * p / 16
                    - t28 * mu * t916 / 8 + t4 * v13 * lambda / 20 - t28 * lambda
                    * a1 / 8 - t4 * u12 * a2 / 10 - t28 * mu * t934 / 32 + t401 * v15
                    * b2 / 32 + t23 * theta0 * p / 12 + t57 * v13 * b2 / 24 - t4
                    * u12 * a0 / 20 + t23 * p * b2 / 12 + t23 * v11 * lambda / 12;
            final double t1127 = -t23 * q * a2 / 3 - t51 * u14 * a0 / 28 - t57 * u13
                    * a0 / 24 - t23 * q * a0 / 12 + t4 * u02 * p / 20 - t57 * u13
                    * a2 / 12 - t51 * u14 * a2 / 14 - t17 * u11 * a2 / 8 + t17 * v21
                    * q / 16 + t57 * v23 * q / 24 + t51 * v24 * q / 28 + t401 * v25
                    * q / 32 + t57 * v23 * b1 / 12 + t4 * v22 * q / 20 + t17 * v21
                    * b1 / 8 + t401 * v25 * b1 / 16;
            final double t1176 = t28 * mu * t899 / 4 + t4 * v22 * b1 / 10 + t45
                    * t316 * a1 / 4 + t28 * t238 * a1 / 8 + t4 * t203 * a1 / 20 + t17
                    * t132 * q / 32 - 0.3e1 / 0.20e2 * t4 * t143 * a0 - t23 * t182
                    * a0 / 4 - 0.3e1 / 0.16e2 * t17 * t128 * a0 + 0.3e1 / 0.4e1 * t45
                    * t136 * b2 + t57 * t162 * q / 48 + t23 * t150 * a1 / 12 + t23 * t18
                    * lambda / 12 + t51 * t106 * q / 56 + 0.3e1 / 0.16e2 * t28 * t29
                    * a1 - t57 * t91 * a0 / 8;
            final double t1227 = -0.3e1 / 0.28e2 * t51 * t95 * a0 + t28 * t24 * lambda
                    / 8 + t23 * t150 * p / 24 + t23 * t186 * b1 / 6 + t28 * t37 * a1
                    / 8 + t4 * t41 * a1 / 20 + t57 * t62 * b2 / 24 + t28 * t267 * b1 / 8
                    - t28 * t356 * a0 / 8 - t4 * t260 * a0 / 20 + t57 * t58 * a1 / 24
                    + t17 * t271 * b1 / 16 + t4 * t256 * b1 / 20 - t45 * t86 * b1 / 2
                    + t57 * t162 * b1 / 12 + t4 * t120 * b1 / 10;
            final double t1277 = t17 * t132 * b1 / 8 + t23 * t33 * b2 / 12 + t17 * t78
                    * a1 / 16 + t45 * t5 * b2 * a1 / 2 - t17 * t293 * a0 / 16 - t57
                    * t74 * a0 / 24 + t57 * t360 * b1 / 24 + t28 * t372 * b2 / 8 + t23
                    * t66 * a1 / 12 + t28 * t245 * lambda / 8 + t23 * t245 * b2 / 4 + t4
                    * t13 * b2 / 20 + t51 * t106 * b1 / 14 + t23 * t340 * b1 / 12
                    - t23 * t348 * a0 / 12 + t17 * t352 * b2 / 16;
            final double t1327 = t45 * t309 * lambda / 4 + 0.3e1 / 0.8e1 * t28 * t309
                    * b2 + 0.3e1 / 0.16e2 * t28 * t178 * b1 + t51 * t113 * p / 56
                    - 0.7e1 / 0.80e2 * t4 * t143 * a2 + t23 * t330 * b2 / 48 + 0.3e1
                    / 0.20e2 * t4 * t124 * b2 + t57 * t52 * lambda / 24 + t4 * t203 * p
                    / 40 + t17 * t124 * lambda / 16 + t4 * t70 * lambda / 20 + t57
                    * t210 * p / 48 + t17 * t99 * p / 32 + t51 * t113 * a1 / 28 + t17
                    * t99 * a1 / 16 + t57 * t210 * a1 / 24;
            final double t1376 = t23 * t186 * q / 24 + t4 * t120 * q / 40 - 0.7e1
                    / 0.64e2 * t17 * t128 * a2 + t17 * t344 * b2 / 64 + t23 * t24 * b2
                    / 4 - 0.7e1 / 0.48e2 * t23 * t182 * a2 + t4 * t289 * b2 / 80
                    - 0.7e1 / 0.96e2 * t57 * t91 * a2 + t51 * t282 * b2 / 112 + t57
                    * t82 * b2 / 96 + 0.3e1 / 0.4e1 * t28 * t231 * a2 + t57 * t70 * b2
                    / 8 + 0.3e1 / 0.16e2 * t17 * t18 * b2 - t51 * t95 * a2 / 16 + 0.3e1
                    / 0.28e2 * t51 * t52 * b2 + t51 * v24 * b1 / 14;
            yiP2S = t1028 + t1077 + t1127 + t1176 + t1227 + t1277 + t1327 + t1376;
            final double t1421 = t17 * v11 * a0 / 16 + t4 * v12 * a2 / 20 + t4 * v12
                    * a0 / 20 + t17 * u12 * lambda / 16 + t57 * u13 * b2 / 12 + t57
                    * v13 * a2 / 24 + t401 * v15 * a2 / 32 + t23 * u11 * lambda / 12
                    + t401 * u05 * q / 32 + t57 * u14 * lambda / 24 + t23 * b1 * b2 / 2
                    + t17 * u01 * q / 16 + t17 * v11 * a2 / 16 + t17 * u11 * b2 / 8;
            final double t1464 = t4 * u12 * b2 / 10 + t57 * v13 * a0 / 24 + t57 * u03
                    * q / 24 + t4 * u02 * q / 20 + t4 * u13 * lambda / 20 + t51 * v14
                    * a0 / 28 + t23 * p * a0 / 12 - t23 * a1 * a2 / 12 + t401 * v15 * a0
                    / 32 + t23 * p * a2 / 12 + t51 * v14 * a2 / 28 + t28 * lambda * b1
                    / 8 + t57 * t364 * b1 / 24 + t51 * u14 * b2 / 14;
            final double t1508 = t401 * u15 * b2 / 16 + t28 * lambda * q / 4 + t51
                    * u15 * lambda / 28 + t51 * u04 * q / 28 + t17 * u21 * b1 / 8 + t401
                    * u25 * b1 / 16 + t4 * u22 * b1 / 10 + t23 * q * b2 / 3 + t51
                    * u24 * b1 / 14 + t23 * theta0 * q / 12 + t57 * u23 * b1 / 12
                    + t57 * u23 * q / 24 + t17 * u21 * q / 16 + t4 * u22 * q / 20;
            final double t1554 = t17 * thetaTW * q / 16 + t51 * u24 * q / 28 + t401
                    * u25 * q / 32 - t45 * t316 * b1 / 4 - t28 * t238 * b1 / 8
                    - 0.5e1 / 0.112e3 * t51 * t282 * a2 + t57 * t162 * a1 / 24 + t57
                    * t70 * a0 / 24 + t17 * t132 * a1 / 16 - 0.5e1 / 0.48e2 * t23
                    * t330 * a2 + t23 * t182 * b2 / 48 + t51 * t106 * a1 / 28 + t51
                    * t95 * b2 / 112 + t28 * t178 * a1 / 4 - t23 * t150 * b1 / 6;
            final double t1599 = -t51 * t113 * b1 / 14 - 0.5e1 / 0.96e2 * t57 * t82
                    * a2 + t4 * t143 * b2 / 80 - t45 * t136 * a0 / 4 - t23 * t66 * b1
                    / 12 + t57 * t62 * a0 / 24 + t23 * t245 * a0 / 12 + 0.3e1 / 0.16e2
                    * t28 * t696 * a2 - 0.3e1 / 0.4e1 * t28 * t231 * b2 + 0.3e1 / 0.8e1
                    * t28 * t227 * b1 + t23 * t186 * a1 / 12 + t17 * t18 * a0 / 16
                    + t51 * t52 * a0 / 28 + 0.3e1 / 0.28e2 * t51 * t52 * a2;
            final double t1645 = 0.3e1 / 0.20e2 * t4 * t124 * a2 + t57 * t70 * a2 / 8
                    + t17 * t128 * b2 / 64 - 0.5e1 / 0.64e2 * t17 * t344 * a2 + t23
                    * t24 * a2 / 4 + t57 * t91 * b2 / 96 + t4 * t120 * a1 / 20 + t4
                    * t124 * a0 / 20 + t23 * t24 * a0 / 12 - t4 * t289 * a2 / 16 - t28
                    * t29 * b1 / 8 + 0.3e1 / 0.16e2 * t17 * t18 * a2 + t23 * t326 * b1
                    / 12 + t17 * t252 * b1 / 16 - t45 * t46 * a0 / 2;
            final double t1689 = -t23 * t330 * a0 / 4 - 0.3e1 / 0.28e2 * t51 * t282
                    * a0 - t57 * t82 * a0 / 8 - 0.3e1 / 0.20e2 * t4 * t289 * a0 - 0.3e1
                    / 0.16e2 * t17 * t344 * a0 - t4 * t203 * b1 / 10 + t57 * t62 * a2
                    / 24 + t4 * t13 * a0 / 20 - t4 * t41 * b1 / 20 + t28 * t372 * a2
                    / 8 - t57 * t58 * b1 / 24 + 0.3e1 / 0.4e1 * t45 * t136 * a2 - t17
                    * t99 * b1 / 8 + t28 * t309 * a0 / 8;
            final double t1735 = 0.3e1 / 0.8e1 * t28 * t309 * a2 + t23 * t245 * a2 / 4
                    + t4 * t13 * a2 / 20 + t23 * t33 * a2 / 12 - t57 * t210 * b1 / 12
                    - t28 * t37 * b1 / 8 + t17 * t352 * a2 / 16 + t28 * t278 * b1 / 8
                    - t17 * t78 * b1 / 16 + t28 * t372 * a0 / 8 + t17 * t352 * a0 / 16
                    + t45 * t46 * a2 / 2 + t4 * t6 * b1 / 20 + t23 * t33 * a0 / 12
                    - t23 * a0 * a1 / 12;
            yiP2C = t1421 + t1464 + t1508 + t1554 + t1599 + t1645 + t1689 + t1735;
        }
        {
            qpP1S = -delta * mu / 3;
            final double t3 = mu * mu;
            qpP2C = delta * t3 / 8;
        }
        {
            final double t1 = B * B;
            final double t2 = t1 * t1;
            final double t4 = t2 * B * a;
            final double t5 = mu * u22;
            final double t10 = t2 * t1 * a;
            final double t11 = mu * mu;
            final double t12 = t11 * u14;
            final double t16 = mu * v22;
            final double t20 = t1 * B;
            final double t21 = t20 * a;
            final double t22 = t11 * u11;
            final double t26 = v23 * t11;
            final double t30 = mu * v13;
            final double t34 = mu * v12;
            final double t38 = mu * u13;
            final double t42 = mu * u02;
            final double t47 = t2 * t20 * a;
            final double t48 = mu * v14;
            final double t52 = mu * u04;
            final double t56 = mu * u14;
            final double t60 = mu * u03;
            final double t67 = t2 * a;
            final double t68 = v22 * t11;
            final double t72 = mu * v11;
            final double t79 = t4 * t5 * b2 / 10 - t10 * t12 * a0 / 48 - t4 * t16
                    * a0 / 20 - t21 * t22 * a0 / 24 + t4 * t26 * b1 / 40 + t10 * t30
                    * p / 16 + 0.3e1 / 0.40e2 * t4 * t34 * p + t10 * t38 * q / 48
                    - t4 * t42 * b2 / 20 - t47 * t48 * a1 / 28 - t47 * t52 * b2 / 28
                    + t47 * t56 * b1 / 28 - t10 * t60 * b2 / 24 + 0.3e1 / 0.56e2
                    * t47 * t48 * p + t67 * t68 * b1 / 32 - t67 * t72 * a1 / 16
                    + t67 * t42 * lambda / 8;
            final double t83 = mu * u01;
            final double t90 = mu * u05;
            final double t94 = t1 * a;
            final double t95 = mu * lambda;
            final double t99 = t11 * v24;
            final double t103 = v21 * t11;
            final double t107 = mu * v21;
            final double t114 = t11 * v11;
            final double t118 = t11 * u01;
            final double t122 = mu * u24;
            final double t129 = mu * u21;
            final double t133 = mu * v23;
            final double t137 = mu * v24;
            final double t141 = t11 * v14;
            final double t145 = t4 * t60 * lambda / 10 + t21 * t83 * lambda / 6
                    + t10 * t52 * lambda / 12 + t47 * t90 * lambda / 14 + t94
                    * t95 * b2 / 4 + t10 * t99 * b1 / 48 + t21 * t103 * b1 / 24
                    - t67 * t107 * a2 / 8 - t67 * t107 * a0 / 16 + t21 * t114 * b2
                    / 24 + t21 * t118 * a1 / 24 + t47 * t122 * b2 / 14 - t67
                    * t5 * lambda / 16 - t21 * t129 * lambda / 12 - t10 * t133
                    * a2 / 12 - t47 * t137 * a0 / 28 + t10 * t141 * b2 / 48;
            final double t147 = t11 * v12;
            final double t151 = t11 * u12;
            final double t155 = t11 * v15;
            final double t159 = t11 * u04;
            final double t163 = t11 * u02;
            final double t167 = t2 * t2;
            final double t168 = t167 * a;
            final double t169 = mu * v15;
            final double t176 = t11 * v25;
            final double t180 = t11 * u03;
            final double t193 = t11 * a0;
            final double t197 = mu * v25;
            final double t201 = t11 * u13;
            final double t205 = mu * u15;
            final double t212 = t67 * t147 * b2 / 32 - t67 * t151 * a0 / 32 + t47
                    * t155 * b2 / 56 + t10 * t159 * a1 / 48 + t67 * t163 * a1 / 32
                    + 0.3e1 / 0.64e2 * t168 * t169 * p - t10 * t122 * lambda / 24
                    + t47 * t176 * b1 / 56 + t4 * t180 * a1 / 40 - t4 * t16 * a2
                    / 10 + t10 * t38 * b1 / 24 - t10 * t30 * a1 / 24 - t94 * t193
                    * b1 / 8 - t168 * t197 * a2 / 16 - t4 * t201 * a0 / 40 + t168
                    * t205 * b1 / 32 - t168 * t169 * a1 / 32;
            final double t213 = mu * a0;
            final double t217 = mu * q;
            final double t224 = mu * u23;
            final double t228 = t11 * v13;
            final double t241 = mu * u25;
            final double t248 = mu * u12;
            final double t256 = mu * u11;
            final double t272 = mu * p;
            final double t276 = t21 * t213 * a2 / 3 + t21 * t217 * b1 / 12 + 0.3e1
                    / 0.32e2 * t67 * t72 * p - t4 * t224 * lambda / 20 + t4 * t228
                    * b2 / 40 - t10 * t133 * a0 / 24 + t47 * t56 * q / 56 + t168 * t205
                    * q / 64 - t47 * t241 * lambda / 28 - t4 * t34 * a1 / 20 + t4
                    * t248 * b1 / 20 + t94 * t11 * b2 * a1 / 8 + t67 * t256 * b1
                    / 16 - t67 * t83 * b2 / 16 + t67 * t256 * q / 32 - t47 * t137
                    * a2 / 14 + t4 * t248 * q / 40 + t21 * t272 * a1 / 12;
            final double t279 = t11 * u15;
            final double t286 = t11 * u05;
            final double t302 = theta0 * mu;
            final double t306 = theta0 * t11;
            final double t310 = t11 * thetaTW;
            final double t314 = mu * thetaTW;
            final double t336 = -t47 * t279 * a0 / 56 + t67 * t129 * b2 / 8 + t47
                    * t286 * a1 / 56 + t168 * t241 * b2 / 16 + t10 * t224 * b2 / 12
                    - t168 * t90 * b2 / 32 - t168 * t197 * a0 / 32 + t94 * t302 * lambda
                    / 4 + t94 * t306 * a1 / 16 + t21 * t310 * a1 / 24 - t67 * t314
                    * b2 / 16 - t21 * t302 * b2 / 12 + t21 * t314 * lambda / 6 - t10
                    * u12 * a2 / 12 - t10 * v12 * b2 / 12 + t10 * u22 * a1 / 24 + t10
                    * v22 * q / 24;
            final double t374 = t167 * B * a;
            final double t393 = -t10 * u22 * p / 24 - t4 * u21 * p / 20 + t4 * v21
                    * q / 20 - t168 * u14 * a2 / 16 - t67 * theta0 * a1 / 8 + t67
                    * theta0 * p / 8 - t4 * thetaTW * a1 / 10 + t4 * thetaTW * p / 10
                    + t47 * u23 * a1 / 28 + t47 * v23 * b1 / 28 + t10 * v22 * b1 / 24
                    + t47 * v23 * q / 28 + t374 * v25 * q / 36 - t4 * u11 * a2 / 10
                    + t4 * u21 * a1 / 20 + t4 * v21 * b1 / 20 - t67 * p * b2 / 4
                    - t4 * v11 * b2 / 10;
            final double t446 = t47 * v14 * lambda / 14 - t47 * u03 * a1 / 14
                    - t10 * u02 * a1 / 12 - t21 * lambda * a1 / 3 - t374 * u05 * a1
                    / 18 - t168 * u04 * a1 / 16 - t4 * u01 * a1 / 10 + t374 * u25
                    * a1 / 36 + t374 * v25 * b1 / 36 - t168 * v14 * b2 / 16 + t168
                    * u24 * a1 / 32 - t374 * u25 * p / 36 + t10 * v13 * lambda / 12
                    + t10 * u02 * p / 12 + t4 * u01 * p / 10 + t4 * v12 * lambda
                    / 10 + t67 * v11 * lambda / 8;
            final double t456 = a1 * a1;
            final double t469 = b1 * b1;
            final double t503 = -t67 * q * a2 / 4 - t374 * v15 * b2 / 18 + t168
                    * v24 * b1 / 32 - t21 * mu * t456 / 12 - t47 * v13 * b2 / 14
                    - t374 * u15 * a2 / 18 - t47 * u13 * a2 / 14 + t21 * mu * t469
                    / 12 + t67 * b2 * a1 / 4 - t67 * b1 * a2 / 4 + t168 * v24 * q
                    / 32 - t47 * u23 * p / 28 - t168 * u24 * p / 32 + t47 * u03 * p
                    / 14 + t168 * v15 * lambda / 16 + t168 * u04 * p / 16 + t374
                    * u05 * p / 18 + t21 * lambda * p / 3;
            qiP1S = t79 + t145 + t212 + t276 + t336 + t393 + t446 + t503;
            final double t557 = t67 * t151 * b2 / 32 - t67 * t129 * a0 / 16 + t67 * t256
                    * p / 32 + t10 * t12 * b2 / 48 + t47 * t122 * a2 / 28 + t47 * t137
                    * b2 / 28 - t67 * t147 * a0 / 32 - t10 * t141 * a0 / 48 + t10
                    * t133 * b2 / 24 + t10 * t224 * a2 / 24 + t67 * t163 * b1 / 32
                    + t10 * t30 * q / 48 + t10 * t137 * lambda / 24 + t47 * t197
                    * lambda / 28 + t47 * t176 * a1 / 56 - t4 * t42 * a2 / 20 + t10
                    * t159 * b1 / 48;
            final double t613 = -t10 * t224 * a0 / 24 - t168 * t241 * a0 / 32 + t47
                    * t286 * b1 / 56 + t94 * t11 * b1 * b2 / 8 - t47 * t155 * a0 / 56
                    + t47 * t279 * b2 / 56 + t21 * t272 * b1 / 12 + t67 * t256 * a1
                    / 16 - t67 * t83 * a2 / 16 + t10 * t38 * p / 48 - t47 * t122 * a0
                    / 28 + t4 * t133 * lambda / 20 - t4 * t228 * a0 / 40 + t21 * t22
                    * b2 / 24 + t4 * t26 * a1 / 40 - t4 * t5 * a0 / 20 + t4 * t180
                    * b1 / 40 + t168 * t169 * q / 64;
            final double t666 = t67 * t129 * a2 / 16 + t67 * theta0 * q / 8 + t4
                    * thetaTW * b1 / 10 + t4 * thetaTW * q / 10 + t10 * t30 * b1 / 24
                    + t10 * t38 * a1 / 24 + t4 * t34 * b1 / 20 + t67 * t16 * lambda
                    / 16 + t21 * t217 * a1 / 4 - t10 * t60 * a2 / 24 + t67 * t68
                    * a1 / 32 + t67 * t72 * b1 / 16 + t94 * t95 * a2 / 4 + t10 * t99
                    * a1 / 48 + t21 * t103 * a1 / 24 + t67 * t107 * b2 / 16 - t21
                    * t114 * a0 / 24;
            final double t682 = mu * a1;
            final double t722 = t21 * t118 * b1 / 24 + t21 * t107 * lambda / 12
                    + t47 * t48 * q / 56 + t67 * t72 * q / 32 + t168 * t169 * b1 / 32
                    + t21 * t682 * b1 / 6 - t21 * t213 * b2 / 3 + t47 * t56 * a1
                    / 28 - t94 * t95 * a0 / 2 - t47 * t52 * a2 / 28 + t47 * t48 * b1
                    / 28 + t374 * v25 * p / 36 - t47 * v13 * a2 / 14 + t10 * u12 * b2
                    / 12 - t10 * v12 * a2 / 12 + t10 * v22 * p / 24 + t10 * u22 * q
                    / 24 + t4 * v21 * p / 20;
            final double t776 = t168 * u14 * b2 / 16 + t374 * u15 * b2 / 18 + t47
                    * u13 * b2 / 14 + t4 * u21 * q / 20 + t47 * u23 * q / 28 + t67
                    * a1 * a2 / 4 + t67 * b1 * b2 / 4 + t168 * v24 * p / 32 + t168
                    * u24 * q / 32 + t47 * u03 * q / 14 + t374 * u05 * q / 18 + t168
                    * u15 * lambda / 16 + t168 * u04 * q / 16 + t21 * lambda * q
                    / 3 + t47 * u23 * b1 / 28 - t47 * v23 * a1 / 28 + t10 * u22 * b1
                    / 24;
            final double t831 = -t10 * v22 * a1 / 24 + t47 * v23 * p / 28 + t4 * u11
                    * b2 / 10 + t4 * u21 * b1 / 20 - t4 * v21 * a1 / 20 - t4 * v11
                    * a2 / 10 - t67 * p * a2 / 4 + t67 * theta0 * b1 / 8 + t168
                    * t197 * b2 / 32 + t168 * t241 * a2 / 32 - t67 * t83 * a0 / 8
                    - t4 * t42 * a0 / 10 - t47 * t52 * a0 / 14 - t10 * t60 * a0 / 12
                    - t168 * t90 * a0 / 16 + t4 * t16 * b2 / 20 + t4 * t5 * a2 / 20
                    + t4 * t201 * b2 / 40;
            final double t845 = t11 * a1;
            final double t885 = -t168 * t90 * a2 / 32 + t168 * t205 * a1 / 32 + t4
                    * t34 * q / 40 + t4 * t248 * p / 40 + t94 * t845 * a2 / 4
                    - 0.3e1 / 0.8e1 * t94 * t193 * a1 + t4 * t248 * a1 / 20 + t47
                    * t56 * p / 56 - t21 * t302 * a2 / 12 + t168 * t205 * p / 64
                    - t67 * t314 * a2 / 16 + t21 * t310 * b1 / 24 - t67 * t314 * a0
                    / 8 - t21 * t302 * a0 / 6 + t94 * t306 * b1 / 16 + t47 * u03
                    * b1 / 14 + t10 * u02 * b1 / 12;
            final double t940 = t21 * lambda * b1 / 3 + t374 * u05 * b1 / 18 + t168
                    * u04 * b1 / 16 + t4 * u01 * b1 / 10 + t374 * u25 * b1 / 36
                    - t374 * v25 * a1 / 36 - t168 * v14 * a2 / 16 + t168 * u24 * b1
                    / 32 + t374 * u25 * q / 36 + t47 * u14 * lambda / 14 + t10
                    * u13 * lambda / 12 + t10 * u02 * q / 12 + t4 * u01 * q / 10
                    + t4 * u12 * lambda / 10 + t67 * u11 * lambda / 8 + t67 * q
                    * b2 / 4 - t374 * v15 * a2 / 18 - t168 * v24 * a1 / 32;
            qiP1C = t557 + t613 + t666 + t722 + t776 + t831 + t885 + t940;
            final double t985 = t10 * v12 * q / 24 + t47 * u13 * p / 28 + t4 * v11
                    * q / 20 - t168 * u14 * a1 / 32 - t374 * u15 * a1 / 36 - t47
                    * u03 * a2 / 7 - t67 * theta0 * a2 / 4 - t4 * thetaTW * a2 / 5
                    - t47 * u13 * a1 / 28 + t4 * u11 * p / 20 - t67 * a1 * b1 / 8
                    + t168 * u14 * p / 32 + t168 * v14 * q / 32 + t168 * v25 * lambda
                    / 16;
            final double t1028 = t374 * u15 * p / 36 + t374 * v15 * q / 36 + t47
                    * v13 * q / 28 - t4 * u11 * a1 / 20 + t4 * v11 * b1 / 20 + t67
                    * p * b1 / 8 - t10 * u02 * a2 / 6 - 0.2e1 / 0.3e1 * t21 * lambda
                    * a2 - t374 * u05 * a2 / 9 - t168 * u04 * a2 / 8 - t4 * u01
                    * a2 / 5 + t67 * q * p / 8 + t168 * v14 * b1 / 32 + t47 * v24
                    * lambda / 14;
            final double t1072 = t10 * v23 * lambda / 12 + t4 * v22 * lambda / 10
                    + t67 * v21 * lambda / 8 - t67 * q * a1 / 8 + t374 * v15 * b1
                    / 36 + t47 * v13 * b1 / 28 - t10 * u12 * a1 / 24 + t10 * v12
                    * b1 / 24 + t10 * u12 * p / 24 + t10 * t133 * p / 24 - t47 * t48
                    * a0 / 28 + t4 * t42 * q / 20 + t4 * t38 * lambda / 20 + t4
                    * t228 * b1 / 40;
            final double t1091 = u23 * t11;
            final double t1116 = -t67 * t163 * a0 / 16 - t10 * t159 * a0 / 24
                    - t10 * t30 * a0 / 24 + t168 * t197 * p / 32 + t47 * t137 * p
                    / 28 + t4 * t248 * b2 / 20 + t4 * t1091 * a2 / 80 + 0.3e1 / 0.64e2
                    * t67 * t68 * b2 + t67 * t83 * q / 16 + t67 * t248 * lambda / 16
                    + t21 * t256 * lambda / 12 + t67 * t107 * p / 16 - t4 * t34 * a0
                    / 20 + t10 * t141 * b1 / 48;
            final double t1137 = u24 * t11;
            final double t1144 = t11 * a2;
            final double t1163 = -t21 * t118 * a0 / 12 + t67 * t147 * b1 / 32
                    + t67 * t151 * a1 / 32 + 0.3e1 / 0.80e2 * t4 * t26 * b2 + t10
                    * t60 * q / 24 + t10 * t56 * lambda / 24 + t10 * t1137 * a2
                    / 96 + 0.3e1 / 0.112e3 * t47 * t176 * b2 + t94 * t1144 * b2 / 16
                    + t47 * t155 * b1 / 56 + t47 * t279 * a1 / 56 + t4 * t16 * p
                    / 20 + t67 * t256 * b2 / 16 + t67 * t83 * b1 / 8;
            final double t1167 = t11 * u25;
            final double t1189 = t11 * u21;
            final double t1193 = u22 * t11;
            final double t1209 = t10 * t60 * b1 / 12 + t47 * t1167 * a2 / 112
                    + t21 * t103 * b2 / 16 - t168 * t169 * a0 / 32 - t21 * t272
                    * a0 / 6 - t47 * t286 * a0 / 28 - t67 * t72 * a2 / 8 + t94
                    * t95 * b1 / 4 + t21 * t1189 * a2 / 48 + t67 * t1193 * a2 / 64
                    + t10 * t99 * b2 / 32 - t67 * t72 * a0 / 16 + t21 * t114 * b1
                    / 24 - t21 * t682 * a2 / 3;
            final double t1254 = t47 * t56 * b2 / 28 + t21 * mu * b2 * b1 / 6
                    - t47 * t48 * a2 / 14 + t47 * t52 * b1 / 14 + t4 * t42 * b1 / 10
                    + t10 * t12 * a1 / 48 - t21 * t310 * a0 / 12 + t67 * t314 * b1
                    / 8 + t21 * t302 * q / 12 - t94 * t306 * a0 / 8 + t21 * t302
                    * b1 / 6 + t67 * t314 * q / 16 - t4 * t34 * a2 / 10 + t47 * t205
                    * lambda / 28;
            final double t1297 = t47 * t52 * q / 28 + t168 * t90 * q / 32 + t21
                    * t22 * a1 / 24 + t10 * t38 * b2 / 24 - t10 * t30 * a2 / 12 + t21
                    * t213 * a1 / 6 + t21 * t217 * b2 / 6 + t94 * t845 * b1 / 8
                    - t94 * t193 * b2 / 4 + t4 * t201 * a1 / 40 - t4 * t180 * a0 / 20
                    + t168 * t90 * b1 / 16 + t168 * t205 * b2 / 32 - t168 * t169
                    * a2 / 16;
            qiP2S = t985 + t1028 + t1072 + t1116 + t1163 + t1209 + t1254 + t1297;
            final double t1300 = p * p;
            final double t1305 = q * q;
            final double t1331 = a0 * a0;
            final double t1335 = -t67 * t1300 / 16 - t67 * t456 / 16 + t67 * t1305
                    / 16 + t67 * t469 / 16 + t67 * theta0 * b2 / 4 + t4 * thetaTW * b2
                    / 5 + t47 * u13 * q / 28 + t67 * q * b1 / 8 + t168 * u14 * q / 32
                    + t10 * u12 * q / 24 + t4 * u11 * q / 20 + t94 * t11 * t1331 / 8;
            final double t1339 = a2 * a2;
            final double t1343 = b2 * b2;
            final double t1374 = t94 * t11 * t456 / 8 + 0.3e1 / 0.32e2 * t94 * t11
                    * t1339 + t94 * t11 * t1343 / 32 - t374 * v15 * p / 36 - t168
                    * v14 * p / 32 + t168 * v14 * a1 / 32 + t374 * v15 * a1 / 36 - t47
                    * v13 * p / 28 + t47 * v13 * a1 / 28 - t10 * v12 * p / 24 + t4
                    * v11 * a1 / 20 + t67 * p * a1 / 8;
            final double t1412 = t10 * u12 * b1 / 24 + t47 * u13 * b1 / 28 + t374
                    * u15 * b1 / 36 + t168 * u14 * b1 / 32 - t4 * v11 * p / 20 + t4
                    * u11 * b1 / 20 + t374 * u15 * q / 36 + t10 * v12 * a1 / 24 + t47
                    * u24 * lambda / 14 + t47 * u03 * b2 / 7 + t10 * u02 * b2 / 6
                    + 0.2e1 / 0.3e1 * t21 * lambda * b2;
            final double t1452 = t374 * u05 * b2 / 9 + t168 * u04 * b2 / 8 + t4
                    * u01 * b2 / 5 + t4 * u22 * lambda / 10 + t67 * u21 * lambda / 8
                    + t10 * u23 * lambda / 12 + t168 * u25 * lambda / 16 + t21
                    * t302 * a1 / 6 - t21 * t302 * p / 12 + t67 * t314 * a1 / 8
                    - t67 * t314 * p / 16 + t4 * t26 * a2 / 80 + t4 * t1091 * b2 / 80;
            final double t1491 = t10 * t1137 * b2 / 96 + t47 * t176 * a2 / 112
                    + t47 * t1167 * b2 / 112 + t168 * t90 * a1 / 16 + t67 * t83 * a1
                    / 8 + t4 * t42 * a1 / 10 + t47 * t52 * a1 / 14 + t10 * t60 * a1
                    / 12 + t67 * t68 * a2 / 64 + t10 * t99 * a2 / 96 + t21 * t103
                    * a2 / 48 + t21 * t1189 * b2 / 48;
            final double t1532 = t67 * t1193 * b2 / 64 + t94 * t95 * a1 / 4 + t168
                    * t241 * p / 32 + t168 * t169 * b2 / 16 + t67 * t129 * p / 16
                    + t21 * t217 * a2 / 6 + t47 * t122 * p / 28 - t21 * t72 * lambda
                    / 12 - t67 * t83 * p / 16 + t21 * mu * a2 * b1 / 6 - t10 * t48
                    * lambda / 24 - t10 * t60 * p / 24 - t67 * t34 * lambda / 16;
            final double t1570 = t21 * t682 * b2 / 3 + t67 * t72 * b2 / 8 + t10
                    * t224 * p / 24 + t67 * t256 * a2 / 16 + t10 * t38 * a2 / 24
                    + t4 * t248 * a2 / 20 + t4 * t5 * p / 20 - t94 * t1144 * a0 / 4
                    + t4 * t34 * b2 / 10 + t47 * t48 * b2 / 14 - t4 * t30 * lambda
                    / 20 - t47 * t169 * lambda / 28;
            final double t1610 = -t47 * t52 * p / 28 - t168 * t90 * p / 32 - t4
                    * t42 * p / 20 - t67 * t256 * a0 / 16 - t47 * t56 * a0 / 28
                    - t21 * t213 * b1 / 6 - t21 * t217 * a0 / 6 - t168 * t205 * a0
                    / 32 - t10 * t38 * a0 / 24 - t4 * t248 * a0 / 20 + t10 * t30
                    * b2 / 12 + t168 * t205 * a2 / 32 + t47 * t56 * a2 / 28;
            qiP2C = t1335 + t1374 + t1412 + t1452 + t1491 + t1532 + t1570 + t1610;
        }
    }

    /**
     * Compute aerodynamic feathering moment.
     * 
     * @param radialPosition : radial position (ratio versus blade radius)
     * @param azimuth : azimuth of the blade (rad)
     * @return feathering moment (N.m)
     */
    public double computeAerodynamicFeatheringMoment(double radialPosition,
            double azimuth) {
        final double x = radialPosition;
        final double psi = azimuth;

        // Airfoil section numbers
        final double a = bladeAirfoil.getLiftSlope();
        final double Cm = bladeAirfoil.getMomentCoefficient();
        final double l = (xcg - bladeAirfoil.getAerodynamicCenterLocation()) * c;

        // Mapple generated code -----------------------------------------------
        final double t1 = R * R;
        final double t5 = omega * omega;
        final double t8 = B * x;
        final double t9 = a * l;
        final double t10 = t8 * t9;
        final double t11 = FastMath.sin(psi);
        final double t12 = mu * t11;
        final double t13 = FastMath.cos(psi);
        final double t14 = u11 * t13;
        final double t15 = t12 * t14;
        final double t18 = 2 * psi;
        final double t19 = FastMath.cos(t18);
        final double t20 = u21 * t19;
        final double t21 = t12 * t20;
        final double t24 = FastMath.sin(t18);
        final double t25 = v21 * t24;
        final double t26 = t12 * t25;
        final double t29 = mu * mu;
        final double t30 = t11 * t11;
        final double t31 = t29 * t30;
        final double t32 = v22 * t24;
        final double t33 = t31 * t32;
        final double t36 = u22 * t19;
        final double t37 = t31 * t36;
        final double t40 = u12 * t13;
        final double t41 = t31 * t40;
        final double t44 = x * x;
        final double t45 = B * t44;
        final double t46 = t45 * t9;
        final double t47 = t12 * t32;
        final double t50 = B * B;
        final double t51 = t44 * x;
        final double t52 = t50 * t51;
        final double t53 = t52 * a;
        final double t54 = l * mu;
        final double t56 = t54 * t30 * v14;
        final double t60 = t54 * t11 * u04;
        final double t63 = t44 * t44;
        final double t64 = t50 * t63;
        final double t65 = t64 * a;
        final double t67 = t54 * t11 * u05;
        final double t71 = t54 * t30 * v15;
        final double t74 = t50 * t44;
        final double t75 = t74 * a;
        final double t76 = l * t29;
        final double t77 = t30 * t11;
        final double t79 = t76 * t77 * v14;
        final double t83 = t76 * t30 * u04;
        final double t86 = 560 * t10 * t15 + 560 * t10 * t21 + 560 * t10 * t26
                + 280 * t10 * t33 + 280 * t10 * t37 + 280 * t10 * t41 + 420 * t46
                * t47 + 280 * t53 * t56 + 280 * t53 * t60 + 240 * t65 * t67 + 240
                * t65 * t71 + 168 * t75 * t79 + 168 * t75 * t83;
        final double t87 = t50 * x;
        final double t88 = t87 * a;
        final double t90 = t54 * t11 * u02;
        final double t94 = t54 * t30 * v12;
        final double t98 = t76 * t77 * v13;
        final double t102 = t76 * t30 * u03;
        final double t105 = t50 * B;
        final double t106 = t105 * a;
        final double t107 = t106 * l;
        final double t108 = u13 * t13;
        final double t109 = t31 * t108;
        final double t112 = t105 * x;
        final double t113 = t112 * a;
        final double t115 = t54 * t30 * v13;
        final double t119 = t54 * t11 * u03;
        final double t122 = t105 * t44;
        final double t123 = t122 * a;
        final double t125 = t76 * t77 * v15;
        final double t129 = t76 * t30 * u05;
        final double t136 = t105 * t51;
        final double t137 = t136 * a;
        final double t142 = 420 * t88 * t90 + 420 * t88 * t94 + 210 * t88 * t98
                + 210 * t88 * t102 + 210 * t107 * t109 + 336 * t113 * t115 + 336
                * t113 * t119 + 140 * t123 * t125 + 140 * t123 * t129 + 280 * t123
                * t56 + 280 * t123 * t60 + 240 * t137 * t67 + 240 * t137 * t71;
        final double t148 = t50 * a;
        final double t149 = t148 * l;
        final double t170 = t50 * t50;
        final double t171 = t170 * B;
        final double t172 = t171 * a;
        final double t173 = t172 * l;
        final double t174 = u25 * t19;
        final double t175 = t31 * t174;
        final double t178 = 168 * t113 * t79 + 168 * t113 * t83 + 560 * t149 * t15
                + 560 * t149 * t21 + 560 * t149 * t26 + 280 * t149 * t33 + 280
                * t149 * t37 + 280 * t149 * t41 + 336 * t75 * t115 + 336 * t75 * t119
                + 140 * t53 * t125 + 140 * t53 * t129 + 140 * t173 * t175;
        final double t179 = v25 * t24;
        final double t180 = t31 * t179;
        final double t183 = t171 * x;
        final double t184 = t183 * a;
        final double t189 = t170 * a;
        final double t190 = t189 * l;
        final double t191 = u23 * t19;
        final double t192 = t12 * t191;
        final double t195 = v23 * t24;
        final double t196 = t12 * t195;
        final double t199 = t12 * t108;
        final double t202 = u24 * t19;
        final double t203 = t31 * t202;
        final double t206 = u14 * t13;
        final double t207 = t31 * t206;
        final double t210 = v24 * t24;
        final double t211 = t31 * t210;
        final double t214 = t170 * x;
        final double t215 = t214 * a;
        final double t224 = 140 * t173 * t180 + 240 * t184 * t67 + 240 * t184 * t71
                + 336 * t190 * t192 + 336 * t190 * t196 + 336 * t190 * t199 + 168
                * t190 * t203 + 168 * t190 * t207 + 168 * t190 * t211 + 140 * t215
                * t125 + 140 * t215 * t129 + 280 * t215 * t56 + 280 * t215 * t60;
        final double t227 = t170 * t44;
        final double t228 = t227 * a;
        final double t235 = t12 * t36;
        final double t238 = t12 * t40;
        final double t241 = t31 * t191;
        final double t244 = t31 * t195;
        final double t247 = t63 * a;
        final double t248 = t247 * l;
        final double t257 = t63 * x;
        final double t258 = t257 * a;
        final double t259 = t258 * l;
        final double t260 = t12 * t202;
        final double t263 = t12 * t206;
        final double t266 = 240 * t228 * t67 + 240 * t228 * t71 + 420 * t107 * t47
                + 420 * t107 * t235 + 420 * t107 * t238 + 210 * t107 * t241 + 210
                * t107 * t244 + 336 * t248 * t199 + 168 * t248 * t203 + 168 * t248
                * t207 + 168 * t248 * t211 + 280 * t259 * t260 + 280 * t259 * t263;
        final double t267 = t12 * t210;
        final double t270 = u15 * t13;
        final double t271 = t31 * t270;
        final double t278 = t63 * t44;
        final double t279 = t278 * a;
        final double t280 = t279 * l;
        final double t281 = t12 * t270;
        final double t284 = t12 * t174;
        final double t287 = t12 * t179;
        final double t291 = B * a * l;
        final double t292 = t31 * t14;
        final double t295 = t31 * t20;
        final double t298 = t31 * t25;
        final double t302 = t12 * q * t13;
        final double t305 = t8 * a;
        final double t314 = 280 * t259 * t267 + 140 * t259 * t271 + 140 * t259
                * t175 + 140 * t259 * t180 + 240 * t280 * t281 + 240 * t280 * t284
                + 240 * t280 * t287 + 420 * t291 * t292 + 420 * t291 * t295 + 420
                * t291 * t298 + 420 * t291 * t302 + 560 * t305 * t54 * t11 * u01
                + 560 * t305 * t54 * t30 * v11;
        final double t324 = B * t51;
        final double t325 = t324 * a;
        final double t330 = B * t63;
        final double t331 = t330 * a;
        final double t340 = B * t257;
        final double t341 = t340 * a;
        final double t350 = t45 * a;
        final double t353 = 280 * t305 * t76 * t30 * u02 + 280 * t305 * t76 * t77
                * v12 + 336 * t325 * t115 + 336 * t325 * t119 + 140 * t331 * t125
                + 140 * t331 * t129 + 280 * t331 * t56 + 280 * t331 * t60 + 240
                * t341 * t67 + 240 * t341 * t71 + 168 * t325 * t79 + 168 * t325
                * t83 + 420 * t350 * t90;
        final double t360 = t170 * t50;
        final double t361 = t360 * a;
        final double t362 = t361 * l;
        final double t378 = x * a * l;
        final double t385 = 420 * t350 * t94 + 210 * t350 * t98 + 210 * t350 * t102
                + 240 * t362 * t281 + 240 * t362 * t284 + 240 * t362 * t287 + 280
                * t173 * t260 + 280 * t173 * t263 + 280 * t173 * t267 + 140 * t173
                * t271 + 420 * t378 * t292 + 420 * t378 * t295 + 420 * t378 * t298;
        final double t391 = t44 * a;
        final double t392 = t391 * l;
        final double t405 = t51 * a;
        final double t406 = t405 * l;
        final double t419 = 420 * t378 * t302 + 560 * t392 * t15 + 560 * t392 * t21
                + 560 * t392 * t26 + 280 * t392 * t33 + 280 * t392 * t37 + 280
                * t392 * t41 + 420 * t406 * t47 + 420 * t406 * t235 + 420 * t406
                * t238 + 210 * t406 * t241 + 210 * t406 * t244 + 210 * t406 * t109;
        final double t424 = t31 * u01;
        final double t428 = t77 * v11 * t29;
        final double t432 = t30 * p * mu;
        final double t435 = t12 * u01;
        final double t438 = mu * t30;
        final double t439 = t438 * v11;
        final double t442 = t31 * u02;
        final double t445 = t29 * t77;
        final double t446 = t445 * v12;
        final double t449 = t438 * v13;
        final double t452 = t12 * u03;
        final double t455 = t445 * v15;
        final double t458 = t31 * u05;
        final double t461 = 336 * t248 * t192 + 336 * t248 * t196 + 420 * t378
                * t424 + 420 * t378 * t428 + 420 * t378 * t432 + 560 * t392 * t435
                + 560 * t392 * t439 + 280 * t392 * t442 + 280 * t392 * t446 + 336
                * t248 * t449 + 336 * t248 * t452 + 140 * t259 * t455 + 140 * t259
                * t458;
        final double t463 = t438 * v14;
        final double t466 = t12 * u04;
        final double t469 = t12 * u05;
        final double t472 = t438 * v15;
        final double t475 = t445 * v14;
        final double t478 = t31 * u04;
        final double t481 = t12 * u02;
        final double t484 = t438 * v12;
        final double t487 = t445 * v13;
        final double t490 = t31 * u03;
        final double t494 = l * p * t11;
        final double t501 = 280 * t259 * t463 + 280 * t259 * t466 + 240 * t280
                * t469 + 240 * t280 * t472 + 168 * t248 * t475 + 168 * t248 * t478
                + 420 * t406 * t481 + 420 * t406 * t484 + 210 * t406 * t487 + 210
                * t406 * t490 + 280 * t305 * t494 + 420 * t291 * t424 + 420 * t291
                * t428;
        final double t505 = l * q * t13;
        final double t510 = u11 * l * a;
        final double t515 = u21 * l * a;
        final double t520 = v21 * l * a;
        final double t524 = l * v11 * t11;
        final double t529 = v22 * l * a;
        final double t534 = u22 * l * a;
        final double t539 = u12 * l * a;
        final double t543 = l * v12 * t11;
        final double t548 = u23 * l * a;
        final double t553 = v23 * l * a;
        final double t558 = u13 * l * a;
        final double t561 = 420 * t291 * t432 + 280 * t305 * t505 + 210 * t45 * t13
                * t510 + 210 * t45 * t19 * t515 + 210 * t45 * t24 * t520 + 210 * t350
                * t524 + 168 * t324 * t24 * t529 + 168 * t324 * t19 * t534 + 168
                * t324 * t13 * t539 + 168 * t325 * t543 + 140 * t330 * t19 * t548
                + 140 * t330 * t24 * t553 + 140 * t330 * t13 * t558;
        final double t565 = l * v13 * t11;
        final double t569 = l * v14 * t11;
        final double t574 = u24 * l * a;
        final double t579 = u14 * l * a;
        final double t584 = v24 * l * a;
        final double t587 = B * t278;
        final double t588 = t587 * a;
        final double t590 = l * v15 * t11;
        final double t594 = l * u15 * t13;
        final double t598 = l * u25 * t19;
        final double t602 = l * v25 * t24;
        final double t609 = t360 * x;
        final double t610 = t609 * a;
        final double t615 = 140 * t331 * t565 + 120 * t341 * t569 + 120 * t340
                * t19 * t574 + 120 * t340 * t13 * t579 + 120 * t340 * t24 * t584
                + 105 * t588 * t590 + 105 * t588 * t594 + 105 * t588 * t598 + 105
                * t588 * t602 + 240 * t362 * t469 + 240 * t362 * t472 + 105 * t610
                * t590 + 105 * t610 * t594;
        final double t639 = t171 * t44;
        final double t640 = t639 * a;
        final double t647 = 105 * t610 * t598 + 105 * t610 * t602 + 140 * t173
                * t455 + 140 * t173 * t458 + 280 * t173 * t463 + 280 * t173 * t466
                + 120 * t184 * t569 + 120 * t183 * t19 * t574 + 120 * t183 * t13
                * t579 + 120 * t183 * t24 * t584 + 105 * t640 * t590 + 105 * t640
                * t594 + 105 * t640 * t598;
        final double t681 = 105 * t640 * t602 + 336 * t190 * t449 + 336 * t190
                * t452 + 168 * t190 * t475 + 168 * t190 * t478 + 140 * t214 * t19
                * t548 + 140 * t214 * t24 * t553 + 140 * t214 * t13 * t558 + 140
                * t215 * t565 + 120 * t228 * t569 + 120 * t227 * t19 * t574 + 120
                * t227 * t13 * t579 + 120 * t227 * t24 * t584;
        final double t682 = t170 * t51;
        final double t683 = t682 * a;
        final double t714 = 105 * t683 * t590 + 105 * t683 * t594 + 105 * t683
                * t598 + 105 * t683 * t602 + 420 * t107 * t481 + 420 * t107 * t484
                + 210 * t107 * t487 + 210 * t107 * t490 + 168 * t112 * t24 * t529
                + 168 * t112 * t19 * t534 + 168 * t112 * t13 * t539 + 168 * t113
                * t543 + 140 * t122 * t19 * t548;
        final double t738 = t105 * t63;
        final double t739 = t738 * a;
        final double t752 = 140 * t122 * t24 * t553 + 140 * t122 * t13 * t558 + 140
                * t123 * t565 + 120 * t137 * t569 + 120 * t136 * t19 * t574 + 120
                * t136 * t13 * t579 + 120 * t136 * t24 * t584 + 105 * t739 * t590
                + 105 * t739 * t594 + 105 * t739 * t598 + 105 * t739 * t602 + 560
                * t149 * t435 + 560 * t149 * t439;
        final double t788 = 280 * t149 * t442 + 280 * t149 * t446 + 210 * t87 * t13
                * t510 + 210 * t87 * t19 * t515 + 210 * t87 * t24 * t520 + 210 * t88
                * t524 + 168 * t74 * t24 * t529 + 168 * t74 * t19 * t534 + 168 * t74
                * t13 * t539 + 168 * t75 * t543 + 140 * t52 * t19 * t548 + 140 * t52
                * t24 * t553 + 140 * t52 * t13 * t558;
        final double t800 = t324 * t9;
        final double t813 = t330 * t9;
        final double t818 = 420 * t46 * t235 + 420 * t46 * t238 + 210 * t46 * t241
                + 210 * t46 * t244 + 210 * t46 * t109 + 336 * t800 * t192 + 336 * t800
                * t196 + 336 * t800 * t199 + 168 * t800 * t203 + 168 * t800 * t207
                + 168 * t800 * t211 + 280 * t813 * t260 + 280 * t813 * t263;
        final double t827 = t340 * t9;
        final double t834 = t183 * t9;
        final double t839 = t214 * t9;
        final double t848 = 280 * t813 * t267 + 140 * t813 * t271 + 140 * t813 * t175
                + 140 * t813 * t180 + 240 * t827 * t281 + 240 * t827 * t284 + 240
                * t827 * t287 + 240 * t834 * t284 + 240 * t834 * t287 + 280 * t839
                * t260 + 280 * t839 * t263 + 280 * t839 * t267 + 140 * t839 * t271;
        final double t855 = t227 * t9;
        final double t862 = t112 * t9;
        final double t873 = t122 * t9;
        final double t880 = 140 * t839 * t175 + 140 * t839 * t180 + 240 * t855 * t281
                + 240 * t855 * t284 + 240 * t855 * t287 + 336 * t862 * t192 + 336
                * t862 * t196 + 336 * t862 * t199 + 168 * t862 * t203 + 168 * t862
                * t207 + 140 * t873 * t271 + 140 * t873 * t175 + 140 * t873 * t180;
        final double t881 = t136 * t9;
        final double t888 = t87 * t9;
        final double t901 = t74 * t9;
        final double t910 = 240 * t881 * t281 + 240 * t881 * t284 + 240 * t881 * t287
                + 420 * t888 * t47 + 420 * t888 * t235 + 420 * t888 * t238 + 210
                * t888 * t241 + 210 * t888 * t244 + 210 * t888 * t109 + 336 * t901
                * t192 + 336 * t901 * t196 + 336 * t901 * t199 + 168 * t901 * t203;
        final double t916 = t52 * t9;
        final double t929 = t64 * t9;
        final double t940 = 168 * t901 * t207 + 168 * t901 * t211 + 280 * t916 * t260
                + 280 * t916 * t263 + 280 * t916 * t267 + 140 * t916 * t271 + 140
                * t916 * t175 + 140 * t916 * t180 + 240 * t929 * t281 + 240 * t929
                * t284 + 240 * t929 * t287 + 140 * t53 * t565 + 120 * t65 * t569;
        final double t950 = t50 * t257;
        final double t951 = t950 * a;
        final double t961 = theta0 * mu * t11;
        final double t966 = t31 * thetaTW;
        final double t969 = t12 * thetaTW;
        final double t976 = 120 * t64 * t19 * t574 + 120 * t64 * t13 * t579 + 120
                * t64 * t24 * t584 + 105 * t951 * t590 + 105 * t951 * t594 + 105 * t951
                * t598 + 105 * t951 * t602 + 840 * t378 * t961 + 840 * t291 * t961
                + 420 * t378 * t966 + 560 * t392 * t969 + 420 * t291 * t966 + 560
                * t149 * t969;
        final double t980 = t9 * mu;
        final double t985 = a0 - a1 * t13 - b1 * t11 - a2 * t19 - b2 * t24;
        final double t986 = t985 * t13;
        final double t1005 = a1 * t11 - b1 * t13 + 2 * a2 * t24 - 2 * b2 * t19;
        final double t1006 = t12 * t1005;
        final double t1031 = -420 * t980 * t986 * B - 420 * t980 * t986 * x - 840 * t29 * t11 * a * l * t985 * t13 - 420 * t378 * t1006 - 420 * t291 * t1006 + 280 * t50 * Cm * c + 280 * t44 * Cm * c + 240 * t834 * t281 + 560 * t305 * t54 * t11 * thetaTW + 168 * t862 * t211 + 280 * t873 * t260 + 280 * t873 * t263 + 280 * t873 * t267;
        final double t1032 = l * t1005;
        final double t1056 = t63 * t51 * a;
        final double t1057 = l * u05;
        final double t1061 = t170 * t105 * a;
        final double t1073 = -280 * t391 * t1032 - 280 * t148 * t1032 + 420 * t9 * lambda * B + 420 * t9 * lambda * x + 210 * t51 * u01 * t9 + 168 * t63 * u02 * t9 + 140 * t257 * u03 * t9 + 120 * t278 * u04 * t9 + 105 * t1056 * t1057 + 105 * t1061 * t1057 + 120 * t360 * u04 * t9 + 140 * t171 * u03 * t9 + 168 * t170 * u02 * t9;
        final double t1078 = Cm * c;
        final double t1096 = u01 * l * a;
        final double t1100 = u02 * l * a;
        final double t1104 = u03 * l * a;
        final double t1108 = u04 * l * a;
        final double t1111 = t9 * u05;
        final double t1116 = 210 * t105 * u01 * t9 + 840 * t31 * t1078 + 280 * t8
                * t1078 + 280 * t44 * theta0 * t9 + 280 * t50 * theta0 * t9 + 210
                * t51 * thetaTW * t9 + 210 * t105 * thetaTW * t9 + 210 * t45 * t1096
                + 168 * t324 * t1100 + 140 * t330 * t1104 + 120 * t340 * t1108
                + 105 * t587 * t1111 + 105 * t609 * t1111;
        final double t1143 = 120 * t183 * t1108 + 105 * t639 * t1111 + 140 * t214
                * t1104 + 120 * t227 * t1108 + 105 * t682 * t1111 + 168 * t112
                * t1100 + 140 * t122 * t1104 + 120 * t136 * t1108 + 105 * t738 * t1111
                + 210 * t87 * t1096 + 168 * t74 * t1100 + 140 * t52 * t1104 + 120
                * t64 * t1108;
        final double t1148 = Cm * mu;
        final double t1149 = t11 * c;
        final double t1183 = 105 * t950 * t1111 + 840 * t1148 * t1149 * B + 840
                * t1148 * t1149 * x + 140 * t171 * t19 * t548 + 840 * t12 * t9 * lambda
                + 280 * t391 * t494 + 280 * t391 * t505 + 210 * t51 * t13 * t510
                + 210 * t51 * t19 * t515 + 210 * t51 * t24 * t520 + 210 * t405 * t524
                + 168 * t63 * t24 * t529 + 168 * t63 * t19 * t534;
        final double t1217 = 168 * t63 * t13 * t539 + 168 * t247 * t543 + 140 * t257
                * t19 * t548 + 140 * t257 * t24 * t553 + 140 * t257 * t13 * t558
                + 140 * t258 * t565 + 120 * t279 * t569 + 120 * t278 * t19 * t574
                + 120 * t278 * t13 * t579 + 120 * t278 * t24 * t584 + 105 * t1056
                * t590 + 105 * t1056 * t594 + 105 * t1056 * t598;
        final double t1251 = 105 * t1056 * t602 + 105 * t1061 * t590 + 105 * t1061
                * t594 + 105 * t1061 * t598 + 105 * t1061 * t602 + 120 * t361 * t569
                + 120 * t360 * t19 * t574 + 120 * t360 * t13 * t579 + 120 * t360
                * t24 * t584 + 140 * t171 * t24 * t553 + 140 * t171 * t13 * t558 + 140
                * t172 * t565 + 168 * t170 * t24 * t529;
        final double t1276 = theta0 * l * a;
        final double t1282 = thetaTW * l * a;
        final double t1290 = 168 * t170 * t19 * t534 + 168 * t170 * t13 * t539 + 168
                * t189 * t543 + 210 * t105 * t13 * t510 + 210 * t105 * t19 * t515
                + 210 * t105 * t24 * t520 + 210 * t106 * t524 + 280 * t148 * t505 + 280
                * t148 * t494 + 280 * t8 * t1276 + 840 * t31 * t1276 + 210 * t45
                * t1282 + 210 * t87 * t1282 - 280 * t8 * t9 * t1005;
        return rho * t1 * R * c * t5 * (B - x) * (t561 + t714 + t353 + t880 + t910
                + t940 + t752 + t1143 + t314 + t501 + t419 + t461 + t224 + t266 + t385
                + t1251 + t1290 + t1031 + t1073 + t142 + t86 + t178 + t788 + t818
                + t848 + t615 + t647 + t681 + t1183 + t1217 + t976 + t1116) / 1680;
    }

    /**
     * Compute propeller feathering moment.
     * 
     * @param radialPosition : radial position (ratio versus blade radius)
     * @param azimuth : azimuth of the blade (rad)
     * @return feathering moment (N.m)
     */
    public double computePropellerFeatheringMoment(double radialPosition,
            double azimuth) {
        final double x = radialPosition;
        final double psi = azimuth;

        // Airfoil section numbers
        final double Icb = Ic - Ib;

        // Mapple generated code -----------------------------------------------
        final double t1 = omega * omega;
        final double t5 = FastMath.sin(psi);
        final double t9 = 2 * psi;
        final double t10 = FastMath.cos(t9);
        final double t26 = FastMath.sin(t9);
        final double t41 = x * x;
        final double t42 = t41 * x;
        final double t46 = 10 * x * v15 * t5 + 30 * x * u21 * t10 + 20 * x * u22
                * t10 + 15 * x * u23 * t10 + 12 * x * u24 * t10 + 10 * x * u25 * t10
                + 30 * x * v21 * t26 + 20 * x * v22 * t26 + 15 * x * v23 * t26 + 12
                * x * v24 * t26 + 10 * x * v25 * t26 + 12 * t42 * v24 * t26;
        final double t51 = FastMath.cos(psi);
        final double t63 = t41 * t41;
        final double t64 = t63 * x;
        final double t89 = 12 * t42 * u24 * t10 + 12 * t42 * u14 * t51 + 12 * t42
                * v14 * t5 + 10 * t42 * v25 * t26 + 10 * t42 * v15 * t5 + 10 * t64
                * v25 * t26 + 10 * t64 * v15 * t5 + 10 * t64 * u15 * t51 + 10 * t64
                * u25 * t10 + 12 * t63 * v24 * t26 + 12 * t63 * u24 * t10 + 12
                * t63 * u14 * t51 + 12 * t63 * v14 * t5;
        final double t130 = 10 * t63 * v25 * t26 + 10 * t63 * v15 * t5 + 10 * t63
                * u15 * t51 + 10 * t63 * u25 * t10 + 15 * t42 * v23 * t26 + 15
                * t42 * u13 * t51 + 15 * t42 * v13 * t5 + 15 * t42 * u23 * t10 + 10
                * t42 * u15 * t51 + 10 * t42 * u25 * t10 + 20 * t41 * v22 * t26
                + 20 * t41 * v12 * t5 + 20 * t41 * u22 * t10;
        final double t170 = 20 * t41 * u12 * t51 + 15 * t41 * v23 * t26 + 15 * t41
                * u13 * t51 + 15 * t41 * v13 * t5 + 15 * t41 * u23 * t10 + 12 * t41
                * v24 * t26 + 12 * t41 * u24 * t10 + 12 * t41 * u14 * t51 + 12 * t41
                * v14 * t5 + 10 * t41 * v25 * t26 + 10 * t41 * v15 * t5 + 10 * t41
                * u15 * t51 + 10 * t41 * u25 * t10;
        final double t208 = 30 * x * u11 * t51 + 20 * x * u12 * t51 + 15 * x * u13
                * t51 + 12 * x * u14 * t51 + 10 * x * u15 * t51 + 30 * x * v11 * t5
                + 20 * x * v12 * t5 + 15 * x * v13 * t5 + 12 * x * v14 * t5 + 10
                * v25 * t26 + 12 * v14 * t5 + 12 * u24 * t10 + 12 * u14 * t51;
        final double t234 = 12 * v24 * t26 + 15 * u23 * t10 + 12 * u04 * t63 + 15
                * u03 * t42 + 10 * u05 * t64 + 10 * v15 * t5 + 10 * u15 * t51 + 10
                * u25 * t10 + 30 * u11 * t51 + 30 * u21 * t10 + 30 * v21 * t26 + 30
                * v11 * t5 + 60 * theta0;
        final double t262 = 15 * v23 * t26 + 15 * v13 * t5 + 15 * u13 * t51 + 20
                * v22 * t26 + 20 * u22 * t10 + 20 * u12 * t51 + 20 * v12 * t5 + 30
                * u01 * x + 20 * u02 * t41 + 20 * x * u02 + 15 * x * u03 + 12 * x
                * u04 + 10 * x * u05;
        final double t283 = 30 * x * thetaTW + 30 * thetaTW + 10 * t63 * u05 + 12
                * t42 * u04 + 10 * t42 * u05 + 15 * t41 * u03 + 12 * t41 * u04 + 10
                * t41 * u05 + 10 * u05 + 30 * u01 + 20 * u02 + 15 * u03 + 12 * u04;
        return Icb * t1 * (x - 1) * (t46 + t89 + t130 + t170 + t208 + t234 + t262
                + t283) / 60;
    }

    /**
     * Compute blade flapping moment at a given azimuth.
     * 
     * @param azimuth : azimuth of the blade
     * @return blade flapping moment (N.m)
     */
    public double computeBladeFlappingMoment(double azimuth) {
        final double psi = azimuth;
        final double cpsi = FastMath.cos(psi);
        final double spsi = FastMath.sin(psi);
        final double c2psi = FastMath.cos(2 * psi);
        final double s2psi = FastMath.sin(2 * psi);

        // Mapple generated code -----------------------------------------------
        final double t2 = Gamma / FastMath.PI;
        final double t3 = mu * mu;
        final double t4 = t3 * t3;
        final double t9 = Gamma * mu;
        final double t10 = B * B;
        final double t11 = t10 * B;
        final double t12 = spsi * t11;
        final double t16 = Gamma * t3;
        final double t21 = t10 * t10;
        final double t22 = spsi * t21;
        final double t26 = c2psi * t11;
        final double t30 = t3 * mu;
        final double t42 = Gamma * t4;
        final double t45 = t21 * B;
        final double t46 = Gamma * t45;
        final double t51 = t21 * t10;
        final double t55 = t21 * t11;
        final double t59 = t21 * t21;
        final double t63 = t59 * B;
        final double t70 = t2 * t4 * spsi * theta0 / 12 + t9 * t12 * theta0 / 3
                - t16 * c2psi * t10 * theta0 / 8 + t9 * t22 * thetaTW / 4 - t16
                * t26 * thetaTW / 12 - t2 * t30 * c2psi * lambda / 12 + 0.5e1 / 0.144e3
                * t2 * t4 * c2psi * p + Gamma * t21 * theta0 / 8 - t42 * theta0 / 64
                + t46 * thetaTW / 10 + t46 * u01 / 10 + Gamma * t51 * u02 / 12
                + Gamma * t55 * u03 / 14 + Gamma * t59 * u04 / 16 + Gamma * t63 * u05
                / 18 + Gamma * t11 * lambda / 6;
        final double t71 = spsi * t51;
        final double t75 = c2psi * t45;
        final double t79 = c2psi * t51;
        final double t83 = spsi * t55;
        final double t87 = s2psi * t11;
        final double t91 = cpsi * t21;
        final double t95 = s2psi * t21;
        final double t99 = cpsi * t45;
        final double t103 = s2psi * t45;
        final double t107 = cpsi * t51;
        final double t111 = s2psi * t51;
        final double t115 = cpsi * t55;
        final double t119 = spsi * t10;
        final double t134 = -t9 * t71 * u23 / 12 + t16 * t75 * u23 / 20 + t16 * t79
                * u24 / 24 - t9 * t83 * u24 / 14 + t16 * t87 * v21 / 12 + t9 * t91
                * v21 / 8 + t16 * t95 * v22 / 16 + t9 * t99 * v22 / 10 + t16 * t103
                * v23 / 20 + t9 * t107 * v23 / 12 + t16 * t111 * v24 / 24 + t9 * t115
                * v24 / 14 + t9 * t119 * lambda / 4 + t9 * cpsi * t59 * v25 / 16
                + t9 * s2psi * t59 * u15 / 16 + t9 * t87 * q / 12;
        final double t136 = cpsi * t11;
        final double t147 = spsi * t59;
        final double t163 = spsi * t45;
        final double t174 = s2psi * t55;
        final double t181 = c2psi * t55;
        final double t191 = -t9 * t136 * a0 / 6 - t9 * c2psi * t59 * v15 / 16 - t9
                * t26 * p / 12 - t9 * t147 * u25 / 16 + t9 * t22 * u01 / 4 + t9
                * t147 * u05 / 8 + t9 * t83 * u04 / 7 + t9 * t71 * u03 / 6 + t9 * t163
                * u02 / 5 + 0.3e1 / 0.56e2 * t16 * t83 * v15 - t16 * s2psi * t10
                * a0 / 8 + t16 * t174 * v25 / 28 + t16 * t115 * u15 / 56 + t16 * t181
                * u25 / 28 - t16 * t79 * u04 / 24 - t16 * t181 * u05 / 28;
        final double t195 = c2psi * t21;
        final double t220 = Gamma * spsi;
        final double t224 = Gamma * s2psi;
        final double t228 = Gamma * cpsi;
        final double t235 = Gamma * c2psi;
        final double t245 = -t16 * t26 * u01 / 12 - t16 * t195 * u02 / 16 - t16
                * t75 * u03 / 20 + t42 * s2psi * a0 / 48 + t9 * t55 * v14 / 14 - t16
                * t11 * u21 / 24 - t16 * t21 * u22 / 32 - t16 * t45 * u23 / 40
                - t16 * t51 * u24 / 48 + t220 * t21 * p / 8 + t224 * t63 * v25 / 18
                + t228 * t63 * u15 / 18 + t220 * t63 * v15 / 18 + t235 * t63 * u25
                / 18 + t228 * t21 * q / 8 + t9 * t59 * v15 / 16;
        final double t297 = t9 * t11 * p / 12 - t16 * t55 * u25 / 56 + t16 * t51
                * u04 / 24 + t16 * t55 * u05 / 28 + t16 * t21 * u02 / 16 + t16
                * t45 * u03 / 20 + t16 * t11 * u01 / 12 - Gamma * t30 * spsi * lambda
                / 16 + 0.5e1 / 0.192e3 * t42 * spsi * p + t16 * t10 * b2 / 16 + t9
                * t21 * v11 / 8 + t9 * t45 * v12 / 10 + t9 * t51 * v13 / 12 + t2
                * t30 * lambda / 8 - 0.5e1 / 0.96e2 * t2 * t4 * p + t16 * t10 * theta0
                / 8;
        final double t346 = t42 * c2psi * theta0 / 64 + t16 * t11 * thetaTW / 12
                - t220 * t21 * a1 / 8 + t228 * t21 * b1 / 8 - t224 * t21 * a2 / 4
                + t235 * t21 * b2 / 4 + t228 * t45 * u11 / 10 + t228 * t51 * u12
                / 12 + t228 * t55 * u13 / 14 + t228 * t59 * u14 / 16 + t220 * t45
                * v11 / 10 + t220 * t51 * v12 / 12 + t220 * t55 * v13 / 14 + t220
                * t59 * v14 / 16 + t235 * t45 * u21 / 10 + t235 * t51 * u22 / 12;
        final double t398 = t235 * t55 * u23 / 14 + t235 * t59 * u24 / 16 + t224
                * t45 * v21 / 10 + t224 * t51 * v22 / 12 + t224 * t55 * v23 / 14
                + t224 * t59 * v24 / 16 - t2 * t4 * cpsi * a0 / 18 + t16 * t119 * a1
                / 16 + t9 * t26 * a1 / 6 + t9 * t87 * b1 / 6 + t16 * cpsi * t10 * b1
                / 16 - t9 * t136 * a2 / 12 - t9 * t12 * b2 / 12 + t16 * t136 * u11
                / 24 + t9 * t95 * u11 / 8 + t9 * t103 * u12 / 10;
        final double t450 = t16 * t91 * u12 / 32 + t9 * t111 * u13 / 12 + t16 * t99
                * u13 / 40 + t9 * t174 * u14 / 14 + t16 * t107 * u14 / 48 - t9 * t195
                * v11 / 8 + t16 * t12 * v11 / 8 - t9 * t75 * v12 / 10 + 0.3e1 / 0.32e2
                * t16 * t22 * v12 - t9 * t79 * v13 / 12 + 0.3e1 / 0.40e2 * t16 * t163
                * v13 - t9 * t181 * v14 / 14 + t16 * t71 * v14 / 16 - t9 * t22 * u21
                / 8 + t16 * t26 * u21 / 12 - t9 * t163 * u22 / 10 + t16 * t195 * u22
                / 16;
        return t70 + t134 + t191 + t245 + t297 + t346 + t398 + t450;
    }

    /**
     * Compute thrust density at a given radial position and given azimuth.
     * 
     * @param radialPosition : radial position (ratio versus blade radius)
     * @param azimuth : azimuth of the blade (rad)
     * @return thrust density (N/m)
     */
    public double getThrustDensity(double radialPosition,
            double azimuth) {
        final double psi = azimuth;
        final double x = radialPosition;

        // Airfoil section numbers
        final double a = bladeAirfoil.getLiftSlope();

        final double UT = getTangentialVelocityComponent(x, psi);
        final double UP = getPerpenticularVelocityComponent(x, psi);
        final double thetaR = theta0 + x * thetaTW + computePitchDeformation(x, psi);
        double d2L = 0.5 * a * rho * c * R * (thetaR * UT * UT + UP * UT);
        if (UT < 0) {
            d2L = -d2L;
        }
        return d2L;
    }

    /**
     * Get tangential velocity component UT in no-feathering plane.
     *
     * @param radialPosition : radial position (ratio versus blade radius)
     * @param azimuth : azimuth of the blade (rad)
     * @return tangential velocity (m/s)
     */
    public double getTangentialVelocityComponent(double radialPosition,
            double azimuth) {
        final double sinPsi = FastMath.sin(azimuth);
        final double x = radialPosition;
        return R * omega * (mu * sinPsi + x);
    }

    /**
     * Get perpendicular velocity component UP to no-feathering plane.
     *
     * @param radialPosition : radial position (ratio versus blade radius)
     * @param azimuth : azimuth of the blade (rad)
     * @return perpendicular velocity (m/s)
     */
    public double getPerpenticularVelocityComponent(double radialPosition,
            double azimuth) {
        final double x = radialPosition;
        final double cosPsi = FastMath.cos(azimuth);
        final double sinPsi = FastMath.sin(azimuth);
        return R * omega * (-mu * cosPsi * a0 + mu * cosPsi * cosPsi * a1
                + mu * cosPsi * b1 * sinPsi + lambda - x * a1 * sinPsi + x * b1
                * cosPsi + x * q * cosPsi + x * p * sinPsi);
    }

    @Override
    public double[] computeRadialPositionForGivenAoa(double aoa, double azimuth) {

        final double psi = azimuth;
        final double cosPsi = FastMath.cos(psi);
        final double sinPsi = FastMath.sin(psi);

        final double result[] = {0, 0};

        // Case no twist, single solution
        if (thetaTW == 0) {
            result[0] = (-aoa * mu * sinPsi + theta0 * mu * sinPsi - mu * cosPsi
                    * a0 + mu * cosPsi * cosPsi * a1 + mu * cosPsi * b1 * sinPsi
                    + lambda) / (aoa - theta0 + a1 * sinPsi - b1 * cosPsi - q * cosPsi
                    - p * sinPsi);
        } else {
            // Otherwise may be two solutions
            // We neglect x^n with n>1 pitch deformation coefficients
            final double theta1 = thetaTW + u01 + u11 * cosPsi + v11 * sinPsi
                    + u21 * FastMath.cos(psi) + v21 * FastMath.sin(psi);

            // Maple generation code -------------------------------------------
            final double t1 = theta1 * mu;
            final double t2 = b1 * cosPsi;
            final double t13 = cosPsi * cosPsi;
            final double t18 = mu * sinPsi;
            final double t21 = sinPsi * sinPsi;
            final double t22 = t21 * a1;
            final double t31 = q * cosPsi;
            final double t32 = p * sinPsi;
            final double t35 = a1 * sinPsi;
            final double t51 = -2 * t1 * t2 * sinPsi + 2 * t1 * sinPsi * q * cosPsi
                    + 4 * t1 * cosPsi * a0 - 4 * t1 * t13 * a1 + 2 * theta1 * aoa * t18
                    - 2 * t1 * t22 + 2 * t1 * t21 * p - 2 * theta1 * theta0 * t18
                    + 2 * t31 * t32 - 2 * t35 * t2 - 2 * t35 * t31
                    + 2 * t2 * t32 + 2 * theta0 * q * cosPsi
                    - 2 * theta0 * a1 * sinPsi + 2 * theta0 * b1 * cosPsi;
            final double t52 = theta1 * theta1;
            final double t53 = mu * mu;
            final double t76 = aoa * aoa;
            final double t77 = theta0 * theta0;
            final double t82 = q * q;
            final double t84 = b1 * b1;
            final double t86 = p * p;
            final double t88 = a1 * a1;
            final double t90 = t52 * t53 * t21 + 2 * theta0 * p * sinPsi
                    + 2 * aoa * a1 * sinPsi - 2 * aoa * p * sinPsi
                    - 2 * t22 * p - 2 * aoa * b1 * cosPsi - 2 * aoa * q * cosPsi
                    + 2 * b1 * t13 * q + t76 + t77 - 2 * aoa * theta0
                    - 4 * theta1 * lambda + t82 * t13 + t84 * t13 + t86 * t21 + t88 * t21;
            final double delta = t51 + t90;
            final double t91 = t1 * sinPsi;
            final double t92 = FastMath.sqrt(delta);
            final double t94 = 1 / theta1;

            if (delta >= 0) {
                result[0] = -(-aoa + theta0 + t91 - t35 + t2 + t31 + t32 + t92) * t94 / 2;
                result[1] = -(-aoa + theta0 + t91 - t35 + t2 + t31 + t32 - t92) * t94 / 2;
            }
        }

        // Test solution validity
        if (result[0] < 0 || result[0] > 1) {
            result[0] = 0;
        }
        if (result[1] < 0 || result[1] > 1) {
            result[1] = 0;
        }
        return result;
    }

    /**
     * Compute blade angle of attack for a given radial position and blade azimuth.
     * 
     * @param radialPosition : radial position (ratio versus blade radius)
     * @param azimuth : azimuth of the blade (rad)
     * @return aoa (deg)
     */
    public double computeAoAForGivenRadialPosition(double radialPosition, double azimuth) {
        final double x = radialPosition;
        return theta0 + x * thetaTW + getPerpenticularVelocityComponent(radialPosition, azimuth)
                / getTangentialVelocityComponent(radialPosition, azimuth);
    }

    @Override
    public double getTangentialVelocity() {
        return U * FastMath.cos(alphaS);
    }

  
    @Override
    public double getNormalVelocity() {
        return U * FastMath.sin(alphaS);
    }

    /**
     *
     * @return number of blade
     */
    public int getNumberOfBlade() {
        return b;
    }

    /**
     *
     * @return blade radius (m)
     */
    public double getBladeRadius() {
        return R;
    }

    /**
     *
     * @return blade chord (m)
     */
    public double getBladeChord() {
        return c;
    }

    /**
     *
     * @return blade center of gravity location (ratio versus chord length)
     */
    public double getBladeCenterOfGravityLocation() {
        return xcg;
    }

    /**
     *
     * @return blade twist (rad)
     */
    public double getBladeTwist() {
        return thetaTW;
    }

    /**
     *
     * @return blade torsional rigidity (N.m2/rad)
     */
    public double getBladeTorsionalRigidity() {
        return GJ;
    }

    /**
     *
     * @return tip loss factor
     */
    public double getTipLossFactor() {
        return B;
    }

    /**
     *
     * @return blade flapping moment of inertia (Kg.m2)
     */
    public double getBladeFlappingMomentOfInertia() {
        return Ib;
    }

    /**
     *
     * @return blade lagging moment of inertia (Kg.m2)
     */
    public double getBladeLaggingMomentOfInertia() {
        return Ic;
    }

    /**
     *
     * @return rotor aera (m2)
     */
    public double getAera() {
        return A;
    }

    /**
     *
     * @return solidity factor
     */
    public double getSolidity() {
        return s;
    }

    /**
     *
     * @return blade arifoil
     */
    public ConstantAirfoil getBladeAirfoil() {
        return bladeAirfoil;
    }

    /**
     *
     * @return induced velocity model
     */
    public ConstantInducedVelocity getInducedVelocity() {
        return inducedVelocity;
    }

    /**
     *
     * @return mean induced velocity (m/s)
     */
    @Override
    public double getMeanInducedVelocity() {
        return inducedVelocity.getVelocity();
    }

    /**
     *
     * @return blade pitch at root (rad)
     */
    @Override
    public double getBladePitch() {
        return theta0;
    }

    /**
     *
     * @return rotor air seed (m/s)
     */
    @Override
    public double getVelocity() {
        return U;
    }

    /**
     *
     * @return rotor no-featherin plane incidence (rad)
     */
    @Override
    public double getIncidence() {
        return alphaS;
    }

    /**
     *
     * @return pitch rate (rad/s)
     */
    @Override
    public double getPitchRate() {
        return pitchRate;
    }

    /**
     *
     * @return normalized pitch rate
     */
    public double getNormalizedPitchRate() {
        return q;
    }

    /**
     *
     * @return roo rate (rad/s)
     */
    @Override
    public double getRollRate() {
        return rollRate;
    }

    /**
     *
     * @return noramized roll rate (rad/s)
     */
    public double getNormalizedRollRate() {
        return p;
    }

    /**
     *
     * @return rotor speed (rad/s)
     */
    @Override
    public double getRotorSpeed() {
        return omega;
    }

    /**
     *
     * @return Lock number
     */
    public double getLockNumber() {
        return Gamma;
    }

    /**
     *
     * @return advance ratio μ
     */
    @Override
    public double getAdvanceRatio() {
        return mu;
    }

    /**
     *
     * @return inflow ratio λ
     */
    @Override
    public double getInFlowRatio() {
        return lambda;
    }

    
    @Override
    public double getConingAngle() {
        return a0;
    }

    
    @Override
    public double getLongitudinalFlapping() {
        return a1;
    }

    
    @Override
    public double getLateralFlapping() {
        return b1;
    }

   
    @Override
    public double get2ndLongitudinalFlapping() {
        return a2;
    }

    
    @Override
    public double get2ndLateralFlapping() {
        return b2;
    }

   
    @Override
    public double getLift() {
        return L;
    }

    
    @Override
    public double getDrag() {
        return D;
    }

   
    public double getLiftCoefficient() {
        if (U < 1) {
            return Double.NaN;
        }
        return CL;
    }

   
    public double getDragCoefficient() {
        if (U < 1) {
            return Double.NaN;
        }
        return CD;
    }

   
    @Override
    public double getThrust() {
        return T;
    }

    
    @Override
    public double getProfileDragRearForce() {
        return Hp;
    }

    
    @Override
    public double getInducedRearForce() {
        return Hi;
    }

    
    @Override
    public double getTotalRearForce() {
        return Hp + Hi;
    }

   
    @Override
    public double getLateralForce() {
        return Yi;
    }

    
    @Override
    public double getProfileDragTorque() {
        return Qp;
    }

    
    @Override
    public double getInducedTorque() {
        return Qi;
    }

   
    @Override
    public double getTotalTorque() {
        return Qi + Qp;
    }

    /**
     * Get periodic component of thrust at a given azimuth
     * 
     * @param azimuth : azimith of the blade (rad)
     * @return periodic thrust (N)
     */
    public double getPeriodicThrust(double azimuth) {
        return tp1S * FastMath.sin(azimuth) + tp1C * FastMath.cos(azimuth)
                + tp2S * FastMath.sin(2 * azimuth) + tp2C * FastMath.cos(2 * azimuth);
    }

    /**
     * Get periodic profile drag rear force component at a given azimuth 
     * @param azimuth : azimith of the blade (rad)
     * @return periodic profile drag rear force (N)
     */
    public double getPeriodicProfileDragRearForce(double azimuth) {
        return hpP1S * FastMath.sin(azimuth) + hpP2C * FastMath.cos(2 * azimuth);
    }

    /**
     * Get periodic induced rear force component at a given azimuth 
     * @param azimuth : azimuth of the blade (rad)
     * @return periodic induced rear force (N)
     */
    public double getPeriodicInducedRearForce(double azimuth) {
        return hiP1S * FastMath.sin(azimuth) + hiP1C * FastMath.cos(azimuth)
                + hiP2S * FastMath.sin(2 * azimuth) + hiP2C * FastMath.cos(2 * azimuth);
    }

    /**
     * Get total periodic rear force component at a given azimuth
     * @param azimuth : azimuth of the blade (rad)
     * @return total periodic rear force (N)
     */
    public double getPeridicTotalRearForce(double azimuth) {
        return getPeriodicProfileDragRearForce(azimuth)
                + getPeriodicInducedRearForce(azimuth);
    }

    /**
     * Get profile drag lateral force component  at a given azimuth
     * 
     * @param azimuth : azimuth of the blade (rad)
     * @return profile drag lateral force (N)
     */
    public double getPeriodicProfileDragLateralForce(double azimuth) {
        return ypP1C * FastMath.cos(azimuth) + ypP2S * FastMath.sin(2 * azimuth);
    }

    /**
     * Get induced lateral force component at a given azimuth
     * 
     * @param azimuth : azimuth of the blade (rad)
     * @return induced lateral force (N)
     */
    public double getPeriodicInducedLateralForce(double azimuth) {
        return yiP1S * FastMath.sin(azimuth) + yiP1C * FastMath.cos(azimuth)
                + yiP2S * FastMath.sin(2 * azimuth) + yiP2C * FastMath.cos(2 * azimuth);
    }

    /**
     * Get total lateral force component at a given azimuth
     * 
     * @param azimuth : azimuth of the blade (rad)
     * @return total lateral force (N)
     */
    public double getPeriodicTotalLateralForce(double azimuth) {
        return getPeriodicProfileDragLateralForce(azimuth)
                + getPeriodicInducedLateralForce(azimuth);
    }

    /**
     * Get profile drag rotor torque component at a given azimuth
     * 
     * @param azimuth : azimuth of the blade (rad)
     * @return profile drag rotor torque (N.m)
     */
    public double getPeriodicProfileDragTorque(double azimuth) {
        return qpP1S * FastMath.sin(azimuth) + qpP2C * FastMath.cos(2 * azimuth);
    }

    /**
     * Get induced rotor torque component at a given azimuth
     * 
     * @param azimuth : azimuth of the blade (rad)
     * @return induced rotor torque (N.m)
     */
    public double getPeriodicInducedTorque(double azimuth) {
        return qiP1S * FastMath.sin(azimuth) + qiP1C * FastMath.cos(azimuth)
                + qiP2S * FastMath.sin(2 * azimuth) + qiP2C * FastMath.cos(2 * azimuth);
    }

    /**
     * Get total torque component at a given azimuth
     * 
     * @param azimuth : azimuth of the blade (rad)
     * @return total rotor torque
     */
    public double getPeriodicTotalTorque(double azimuth) {
        return getPeriodicProfileDragTorque(azimuth)
                + getPeriodicInducedTorque(azimuth);
    }

    /**
     * Get force scale factor : ρ b c R3 Ω2
     * 
     * @return scale force factor
     */
    public double getForceScaleFactor() {
        return forceScaleFactor;
    }

    /**
     * Get moment scale factor : ρ b c R4 Ω2
     * 
     * @return moment scale factor
     */
    public double getMomentScaleFactor() {
        return momentScaleFactor;
    }

    /**
     * Get velocity scale factor : R Ω
     * 
     * @return velocity scale factor
     */
    public double getVelocityScaleFactor() {
        return velocityScaleFactor;
    }

    /**
     * Get angular velocity scale factor : Ω
     * 
     * @return angular velocity scale factor
     */
    public double getAngularVelocityScaleFactor() {
        return angularVelocityScaleFactor;
    }

    /**
     *
     * @return first sine profile drag periodique torque coefficient.
     */
    public double getProfileDragPeriodicTorque1S() {
        return qpP1S;
    }

    /**
     *
     * @return second cosine profile drag periodique torque coefficient.
     */
    public double getProfileDragPeriodicTorque2C() {
        return qpP2C;
    }

    /**
     *
     * @return first sine induced periodique torque coefficient.
     */
    public double getInducedPeriodicTorque1S() {
        return qiP1S;
    }

    /**
     *
     * @return first cosine induced periodique torque coefficient.
     */
    public double getInducedPeriodicTorque1C() {
        return qiP1C;
    }

    /**
     *
     * @return second sine induced periodique torque coefficient.
     */
    public double getInducedPeriodicTorque2S() {
        return qiP2S;
    }

    /**
     *
     * @return second cosine induced periodique torque coefficient.
     */
    public double getInducedPeriodicTorque2C() {
        return qiP2C;
    }

    /**
     *
     * @return first sine periodique thrust coefficient.
     */
    public double getPeriodicThrust1S() {
        return tp1S;
    }

    /**
     *
     * @return first cosine periodique thrust coefficient.
     */
    public double getPeriodicThrust1C() {
        return tp1C;
    }

    /**
     *
     * @return second sine periodique thrust coefficient.
     */
    public double getPeriodicThrust2S() {
        return tp2S;
    }

    /**
     *
     * @return second cosine periodique thrust coefficient.
     */
    public double getPeriodicThrust2C() {
        return tp2C;
    }

    /**
     *
     * @return first sine periodique profile drag rear force coefficient.
     */
    public double getProfileDragPeriodicRearForce1S() {
        return hpP1S;
    }

    /**
     *
     * @return second cosine periodique profile drag rear force coefficient.
     */
    public double getProfileDragPeriodicRearForce2C() {
        return hpP2C;
    }

    /**
     *
     * @return first sine periodique induced rear force coefficient.
     */
    public double getInducedPeriodicRearForce1S() {
        return hiP1S;
    }

    /**
     *
     * @return first cosine periodique induced rear force coefficient.
     */
    public double getInducedPeriodicRearForce1C() {
        return hiP1C;
    }

    /**
     *
     * @return second sine periodique induced rear force coefficient.
     */
    public double getInducedPeriodicRearForce2S() {
        return hiP2S;
    }

    /**
     *
     * @return second cosine periodique induced rear force coefficient.
     */
    public double getInducedPeriodicRearForce2C() {
        return hiP2C;
    }

    /**
     *
     * @return first cossine periodique profile drag lateral force coefficient.
     */
    public double getProfileDragPeriodicLateralForce1C() {
        return ypP1C;
    }

    /**
     *
     * @return second sine periodique profile drag lateral force coefficient.
     */
    public double getProfileDragPeriodicLateralForce2S() {
        return ypP2S;
    }

    /**
     *
     * @return first sine periodique induced lateral force coefficient.
     */
    public double getInducedPeriodicLateralForce1S() {
        return yiP1S;
    }

    /**
     *
     * @return first cosine periodique induced lateral force coefficient.
     */
    public double getInducedPeriodicLateralForce1C() {
        return yiP1C;
    }

    /**
     *
     * @return second sine periodique induced lateral force coefficient.
     */
    public double getInducedPeriodicLateralForce2S() {
        return yiP2S;
    }

    /**
     *
     * @return second cosine periodique induced lateral force coefficient.
     */
    public double getInducedPeriodicLateralForce2C() {
        return yiP2C;
    }

    /**
     * Compute mean blade twist deformation at a given radial position
     * 
     * @param radialPosition : radial position (ratio versus blade radius)
     * @return mean blade twist deformation (rad)
     */
    public double computeMeanPitchDeformation(double radialPosition) {
        final double x = radialPosition;
        return x * (u01 + x * (u02 + x * (u03 + x * (u04 + x * u05))));
    }

    /**
     * Compute first cosine twist deformation coefficient at a given radial position
     * 
     * @param radialPosition : radial position (ratio versus blade radius)
     * @return first cosine twist deformation coefficient (rad)
     */
    public double computePitchDeformation1C(double radialPosition) {
        final double x = radialPosition;
        return x * (u11 + x * (u12 + x * (u13 + x * (u14 + x * u15))));
    }

    /**
     * Compute first sine twist deformation coefficient at a given radial position
     * 
     * @param radialPosition : radial position (ratio versus blade radius)
     * @return first sine twist deformation coefficient (rad)
     */
    public double computePitchDeformation1S(double radialPosition) {
        final double x = radialPosition;
        return x * (v11 + x * (v12 + x * (v13 + x * (v14 + x * v15))));
    }

    /**
     * Compute second cosine twist deformation coefficient at a given radial position
     * 
     * @param radialPosition : radial position (ratio versus blade radius)
     * @return second cosine twist deformation coefficient (rad)
     */
    public double computePitchDeformation2C(double radialPosition) {
        final double x = radialPosition;
        return x * (u21 + x * (u22 + x * (u23 + x * (u24 + x * u25))));
    }

    /**
     * Compute second sine twist deformation coefficient at a given radial position
     * 
     * @param radialPosition : radial position (ratio versus blade radius)
     * @return second sine twist deformation coefficient (rad)
     */
    public double computePitchDeformation2S(double radialPosition) {
        final double x = radialPosition;
        return x * (v21 + x * (v22 + x * (v23 + x * (v24 + x * v25))));
    }

    /**
     * Compute blade twist deformation at a given radial position and blade azimuth.
     * 
     * @param radialPosition : radial position (ratio versus blade radius)
     * @param azimuth : blade azimtuh (rad)
     * @return twist deformation (rad)
     */
    public double computePitchDeformation(double radialPosition, double azimuth) {
        final double x = radialPosition;
        final double u0 = x * (u01 + x * (u02 + x * (u03 + x * (u04 + x * u05))));
        final double u1 = x * (u11 + x * (u12 + x * (u13 + x * (u14 + x * u15))));
        final double v1 = x * (v11 + x * (v12 + x * (v13 + x * (v14 + x * v15))));
        final double u2 = x * (u21 + x * (u22 + x * (u23 + x * (u24 + x * u25))));
        final double v2 = x * (v21 + x * (v22 + x * (v23 + x * (v24 + x * v25))));
        return u0 + u1 * FastMath.cos(azimuth) + v1 * FastMath.sin(azimuth)
                + u2 * FastMath.cos(2 * azimuth) + v2 * FastMath.cos(2 * azimuth);
    }

    /**
     *
     */
    public void print() {
        System.out.println("Rotor inputs");
        System.out.println("------------");
        System.out.println("Number of blades                         : " + b);
        System.out.println("Blade radius (m)                         : " + R);
        System.out.println("Blade chord (m)                          : " + c);
        System.out.println("Chordwide center of gravity location     : " + xcg);
        System.out.println("Blade twist (deg)                        : " + FastMath.toDegrees(thetaTW));
        System.out.println("Blade torsional rigidity (N.m2/rad)      : " + GJ);
        System.out.println("Tip loss factor                          : " + B);
        System.out.println("Blade flapping moment of inertia (kg.m2) : " + Ib);
        System.out.println("Blade lagging moment of inertia (kg.m2)  : " + Ic);
        System.out.println();
        System.out.println("Rotor characteristics");
        System.out.println("---------------------");
        System.out.println("Blade Lock number : " + Gamma);
        System.out.println("Rotor aera (m2)   : " + A);
        System.out.println("Solidity ratio    : " + s);
        System.out.println();
        System.out.println("Airfoil");
        System.out.println("-------");
        System.out.println("Lift Slope                  : " + getBladeAirfoil().getLiftSlope());
        System.out.println("Profile drag                : " + getBladeAirfoil().getProfileDrag());
        System.out.println("Moment coefficient          : " + getBladeAirfoil().getMomentCoefficient());
        System.out.println("Aerodynamic center location : " + getBladeAirfoil().getAerodynamicCenterLocation());
        System.out.println();

        System.out.println("Aerodynamic inputs");
        System.out.println("------------------");
        System.out.println("Forward Speed (km/h) : " + U * 3.6);
        System.out.println("Incidence (deg)      : " + FastMath.toDegrees(alphaS));
        System.out.println("Blade pitch (deg)    : " + FastMath.toDegrees(theta0));
        System.out.println("Pitch rate (deg/s)   : " + FastMath.toDegrees(q * omega));
        System.out.println("Roll rate (deg/s)    : " + FastMath.toDegrees(p * omega));
        System.out.println("Rotor speed (rpm)    : " + omega * 30 / FastMath.PI);

        System.out.println();
        System.out.println("Rotor State");
        System.out.println("-----------");
        System.out.println("Advance ratio                   : " + mu);
        System.out.println("Inflow ratio                    : " + lambda);
        System.out.println("Induced velocity                : " + getInducedVelocity().getVelocity());
        System.out.println("Conning angle (deg)             : " + FastMath.toDegrees(a0));
        System.out.println("Longitudinal flapping (deg)     : " + FastMath.toDegrees(a1));
        System.out.println("Lateral flapping (deg)          : " + FastMath.toDegrees(b1));
        System.out.println("2nd longitudinal flapping (deg) : " + FastMath.toDegrees(a2));
        System.out.println("2nd lateral flapping (deg)      : " + FastMath.toDegrees(b2));
        System.out.println();
        System.out.println("Rotor thrust (daN)");
        System.out.println("------------------");
        System.out.println("Rotor thrust             : " + T / 10);
        System.out.println("Periodic rotor thrust 1C : " + tp1C / 10);
        System.out.println("Periodic rotor thrust 1S : " + tp1S / 10);
        System.out.println("Periodic rotor thrust 2C : " + tp2C / 10);
        System.out.println("Periodic rotor thrust 2S : " + tp2S / 10);
        System.out.println();
        System.out.println("Rear force (daN)");
        System.out.println("----------------");
        System.out.println("Profile drag rear force             : " + Hp / 10);
        System.out.println("Induced rear force                  : " + Hi / 10);
        System.out.println("Total rear force                    : " + (Hi + Hp) / 10);
        System.out.println("Periodic profile drag rear force 1S : " + hpP1S / 10);
        System.out.println("Periodic profile drag rear force 2C : " + hpP2C / 10);
        System.out.println("Periodic induced rear force 1C      : " + hiP1C / 10);
        System.out.println("Periodic induced rear force 1S      : " + hiP1S / 10);
        System.out.println("Periodic induced rear force 2C      : " + hiP2C / 10);
        System.out.println("Periodic induced rear force 2S      : " + hiP2S / 10);
        System.out.println();
        System.out.println("Lateral force (daN)");
        System.out.println("-------------------");
        System.out.println("Induced lateral force                  : " + Yi / 10);
        System.out.println("Periodic profile drag lateral force 1C : " + ypP1C / 10);
        System.out.println("Periodic profile drag lateral force 2S : " + ypP2S / 10);
        System.out.println("Periodic induced lateral force 1C      : " + yiP1C / 10);
        System.out.println("Periodic induced lateral force 1S      : " + yiP1S / 10);
        System.out.println("Periodic induced lateral force 2C      : " + yiP2C / 10);
        System.out.println("Periodic induced lateral force 2S      : " + yiP2S / 10);

        System.out.println();
        System.out.println("Torque (daN.m)");
        System.out.println("--------------");
        System.out.println("Profile drag rotor torque             :  " + Qp / 10);
        System.out.println("Induced rotor torque                  :  " + Qi / 10);
        System.out.println("Total rotor torque                    :  " + (Qp + Qi) / 10);
        System.out.println("Periodic profile drag rotor torque 1S : " + qpP1S / 10);
        System.out.println("Periodic profile drag rotor torque 2C : " + qpP2C / 10);
        System.out.println("Periodic induced rotor torque 1C      : " + qiP1C / 10);
        System.out.println("Periodic induced rotor torque 1S      : " + qiP1S / 10);
        System.out.println("Periodic induced rotor torque 2C      : " + qiP2C / 10);
        System.out.println("Periodic induced rotor torque 2S      : " + qiP2S / 10);

        System.out.println();
        System.out.println("Tip pitch deformation (deg)");
        System.out.println("---------------------------");
        System.out.println("Mean deformation        : " + FastMath.toDegrees(
                computeMeanPitchDeformation(1)));
        System.out.println("Periodic deformation 1C : " + FastMath.toDegrees(
                computePitchDeformation1C(1)));
        System.out.println("Periodic deformation 1S : " + FastMath.toDegrees(
                computePitchDeformation1S(1)));
        System.out.println("Periodic deformation 2C : " + FastMath.toDegrees(
                computePitchDeformation2C(1)));
        System.out.println("Periodic deformation 2S : " + FastMath.toDegrees(
                computePitchDeformation2S(1)));

    }
}
